; -----------------------------------------------------------------------------
; Star Castle 2600 - Copyright (C) 2012 Chris Walton (cd-w) <cwalton@gmail.com>
; Bank 0 - Game Logic
; -----------------------------------------------------------------------------

  ; Bank 0
  SEG     BANK0
  ORG     $0000
  RORG    $F000
  
  ; Space for RAM+
  DS.B    512,35

Init0
  nop     $FFFB             ; Switch to Bank 6
EnterCollision
  nop     $FFF6             ; Switch to Bank 1
EndGame
  nop     $FFFB             ; Switch To Bank 6
  jmp     GameInit
  jmp     MainLoop

; -----------------------------------------------------------------------------
; Game Initialisation
; -----------------------------------------------------------------------------

GameInit
  ; Initialise Audio
  lda #12
  sta SFX
  lda #0
  sta AUDV0
  sta AUDV1
  
  ; Initialise Score & Wave
  sta SCORE+0
  sta SCORE+1
  sta SCORE+2
  sta WAVE

  ; Start with 3 Ships
  ldx #2
  stx SHIPS

  ; Set Wave Data
  jsr WaveSetup

  ; Set Line Pointers
  jsr SetKernelLines

  ; Get Ready Message
  lda #%00101111
  sta MODE

; -----------------------------------------------------------------------------
; Main Loop
; -----------------------------------------------------------------------------

MainLoop
  ; Finish Overscan
WaitMainOverscan
  lda INTIM
  bne WaitMainOverscan
  
  ; VSYNC enable
  lda #2
  sta WSYNC                 ; [0]
  sta VSYNC                 ; [0] + 3
  
  ; Frame Counter
  dec CYCLE                 ; [3] + 5

  ; Pseudo-Random Number
  lda RANDOM                ; [8] + 3
  bne NextRandom            ; [11] + 2/3      Prevent Zeroes
ResetRandom
  lda #RANDOMSEED           ; [13] + 2
NextRandom
  asl                       ; [15] + 2
  bcc NoRndEor              ; [17] + 2/3
  eor #$AF                  ; [19] + 2
NoRndEor
  sta RANDOM                ; [21] + 3

  ; Test B&W/Pause Switch
  lda SWCHB                 ; [24] + 3
  and #%00001000            ; [27] + 2
  cmp #%00001000            ; [29] + 2
  ; Fetch Pause Status
  lda DEBOUNCE              ; [31] + 3
  ; Check For 7800 Mode
  ldx INITIALS+1            ; [34] + 3
  bmi Pause2600             ; [37] + 2/3
Pause7800
  bcs PauseNotPressed       ; [39] + 2/3
  ; Previously Pressed?
  and #%00010000            ; [41] + 2
  bne EndPause              ; [43] + 2/3
  ; Toggle Pause & Button Status
  lda DEBOUNCE              ; [45] + 3
  eor #%00110000            ; [48] + 2
  jmp StorePause            ; [50] + 3
PauseNotPressed
  ; Clear Previous Status
  and #%11101111            ; [42] + 2
  jmp StorePause            ; [44] + 3
Pause2600
  bcs NoPause               ; [40] + 2/3
PauseGame
  ora #%00100000            ; [42] + 2
  bne StorePause            ; [44] + 2/3
NoPause
  and #%11011111            ; [46] + 2
StorePause
  sta DEBOUNCE              ; [53] + 3
EndPause

  ; Check For Reset Switch
  lda SWCHB                 ; [56] + 3
  lsr                       ; [59] + 2
  bcs NoReset               ; [61] + 2/3
  ; Reset Score
  sta WSYNC
  ldx #0
  stx SCORE+0
  stx SCORE+1
  stx SCORE+2
  ; Do VBlank
  lda #1
  ldx #VBLANKDELAY
  sta WSYNC
  sta WSYNC
  sta VSYNC
  stx TIM64T
  jmp EndGame
NoReset

; -----------------------------------------------------------------------------

  sta WSYNC                 ; [0]
  
  ; Play Sound Effects
  ldy SFX                   ; [0] + 3
  beq ResetGSounds          ; [3] + 2/3
  lax SoundData,Y           ; [5] + 4
  beq ResetGSounds          ; [9] + 2/3
  cmp #$FF                  ; [11] + 2
  bne PlayGSound            ; [13] + 2/3
  dey                       ; [15] + 2
  lda SoundData,Y           ; [17] + 4
  sta AUDC0                 ; [21] + 3
  dey                       ; [24] + 2
  lax SoundData,Y           ; [26] + 4
PlayGSound
  dey                       ; [30] + 2
  sty SFX                   ; [32] + 3
  and #%00011111            ; [35] + 2
  sta AUDF0                 ; [37] + 3
  txa                       ; [40] + 2
  and #%11100000            ; [42] + 2
  lsr                       ; [44] + 2
  lsr                       ; [46] + 2
  lsr                       ; [48] + 2
  lsr                       ; [50] + 2
  jmp EndGSounds            ; [52] + 3
ResetGSounds
  lda #0
  sta SFX
EndGSounds
  sta AUDV0                 ; [55] + 3

; -----------------------------------------------------------------------------

  sta WSYNC                 ; [0]

  IF (DEBUG)
  ; Debugging Mode
  lda SWCHB
  and #%00000010
  bne NoSelect
  lda MODE
  cmp #%11100000
  beq NoSelect
  ; Set Explosion Mode
  lda #%11100000
  sta MODE
  lda #EXPLODEFRAMES
  sta EXPLODE
  lda #254
  sta SFX
  ; Increment Wave
  lax WAVE
  and #%11000000
  sta TEMP
  txa
  clc
  ; and #%00111111
  adc #1
  and #%00111111
  ora TEMP
  sta WAVE
  jmp EndModeCounter
NoSelect
  ENDIF

  ; Update Mode Counter
  lda CYCLE                 ; [0] + 3
  and #%00000111            ; [3] + 2
  cmp #%00000111            ; [5] + 2       Update Every 8 Frames (7.5Hz)
  bne EndModeCounter        ; [7] + 2/3
  lax MODE                  ; [9] + 3
  and #%11100000            ; [12] + 2
  sta TEMP                  ; [14] + 3
  txa                       ; [17] + 2
  and #%00011111            ; [19] + 2
  beq EndModeCounter        ; [21] + 2/3
  sec                       ; [23] + 2
  sbc #1                    ; [25] + 2
  ora TEMP                  ; [27] + 3
  sta MODE                  ; [30] + 3
EndModeCounter

  ; Debounce Fire Button
  lax DEBOUNCE              ; [33] + 3
  bit INPT4                 ; [36] + 3
  bmi ResetFirePressed      ; [39] + 2/3
  ; Ignore If Previously Pressed
  and #%01000000            ; [41] + 2
  bne ClearFirePressed      ; [43] + 2/3
  ; Set Fire Pressed & State
  txa                       ; [45] + 2
  ora #%11000000            ; [47] + 2
  bne StoreFire             ; [49] + 3
ResetFirePressed
  and #%00111111            ; [42] + 2
  jmp StoreFire             ; [44] + 3
ClearFirePressed
  txa                       ; [46] + 2
  and #%01111111            ; [48] + 2
StoreFire
  sta DEBOUNCE              ; [52] + 3

; -----------------------------------------------------------------------------

  ; VSYNC disable
  lda #1
  ldx #VBLANKDELAY
  sta WSYNC                 ; [0]
  sta VSYNC                 ; [0] + 3
  stx TIM64T                ; [3] + 3

  ; Check Game Mode
  lda MODE                  ; [6] + 3
  and #%11100000            ; [9] + 2
  beq NormalMode            ; [11] + 2/3
  cmp #%11100000            ; [13] + 2
  beq ExplodeMode           ; [15] + 2/3
  jmp ShowMessages          ; [17] + 3

ExplodeMode
  lda #0
  sta AUDV1
  jmp CheckShipExplosion

NormalMode
  ; Check for Pause Switch
  lda DEBOUNCE
  and #%00100000
  beq SkipPause
  lda #%10100000
  sta MODE
  lda #12
  sta SFX
  jmp ShowMessages
SkipPause

; -----------------------------------------------------------------------------

  ; Convert Sectors to Range 0->18
  lda SECTORS
  lsr
  lsr
  clc
  adc #7
  sta SIRENOFFSET

  ; Set Hum Channel
  lda #6
  sta AUDC1

  ; Load Siren Data
  lda CYCLE
  lsr
  lda SIRENCOUNT
  adc #0
  tay
  lax SirenData,Y    ; Bytes = VVVVFFFF (volume and frequency)
  lsr
  lsr
  lsr
  lsr
  sta AUDV1
  txa
  and #$0F
  adc SIRENOFFSET    ; Add pitch
  sta SF1
  sta AUDF1
  lda #3
  adc SIRENOFFSET    ; Add pitch
  sta SF2

  ; Update Siren Counter
  lda CYCLE
  and #%00000011
  bne NoSirenDec
  ldy SIRENCOUNT
  dey
  dey
  bpl StoreSirenCounter
  ldy #24
StoreSirenCounter
  sty SIRENCOUNT
NoSirenDec

; -----------------------------------------------------------------------------

CheckShipExplosion
  ; Check For Ship Explosion
  lda SHIPS                 ; [0] + 3
  bpl CheckJoystick         ; [3] + 2/3   = 6

  ; Update Explosion Frame
  lda SHIPDIR_HI            ; [5] + 3
  beq EndShipExplosion      ; [8] + 2/3
  ; Skip Joystick Controls
  dec SHIPDIR_HI            ; [10] + 5
  jmp MoveBullets           ; [15] + 3    = 18

EndShipExplosion
  ; Store Ship Bits & Clear Dead Flag
  lax SHIPS 
  and #%01000000
  sta TEMP
  ; Check Ring Explosion Mode
  ldy MODE
  cpy #%11100000
  bcc NoRingExplosion
  ; Wait For End Of Ring Explosion
  lda EXPLODE
  bne CheckJoystick
  ; Finish Sound Effects
  sta AUDV1
NoRingExplosion
  ; Check For Game Over
  txa
  and #%00111111
  beq SetGameOverMode
  ; Decrease Ship Count
  sec
  sbc #1
  ora TEMP
  sta SHIPS
  ; Check For Explode Mode
  cpy #%11100000
  bcc SetNextShipMode
SetNextWaveMode
  ; Show New Wave
  lda #%01011111
  bne SetMode
SetNextShipMode
  ; Show Remaining Ships
  lda #%01111111
  bne SetMode
SetGameOverMode
  ; Show Game Over Message
  lda #%10001111
SetMode
  sta MODE
  jmp ShowMessages

; -----------------------------------------------------------------------------
; Joystick Handler
; -----------------------------------------------------------------------------

CheckJoystick
  ; Check For Rotation
  lax SHIPVDIR              ; [0] + 3
  bit SWCHA                 ; [3] + 3
  bpl RotRight              ; [6] + 2/3
  bvc RotLeft               ; [8] + 2/3
  ; Damp Movement
  txa                       ; [10] + 2
  beq EndRotate             ; [12] + 2/3    Middle
  bpl RotLeft               ; [14] + 2/3    Reverse Direction
RotRight
  cmp #126                  ; [16] + 2      Max Rotation (126)
  beq RotateShip            ; [18] + 2/3
  clc                       ; [20] + 2
  adc #14                   ; [22] + 2
  jmp StoreRotate           ; [24] + 3
RotLeft
  cmp #%10000010            ; [17] + 2      Min Rotation (-126)
  beq RotateShip            ; [19] + 2/3
  sec                       ; [21] + 2
  sbc #14                   ; [23] + 2
  nop                       ; [25] + 2
StoreRotate
  sta SHIPVDIR              ; [27] + 3
RotateShip
  ; Sign Extend
  lda #$7F                  ; [30] + 2
  cmp SHIPVDIR              ; [32] + 3
  adc #$80                  ; [35] + 2
  tax                       ; [37] + 2
  ; Add Rotation
  clc                       ; [39] + 2
  lda SHIPVDIR              ; [41] + 3
  adc SHIPDIR_LO            ; [44] + 3
  sta SHIPDIR_LO            ; [47] + 3
  txa                       ; [50] + 2
  adc SHIPDIR_HI            ; [52] + 3
  and #%00011111            ; [55] + 2
  sta SHIPDIR_HI            ; [57] + 3
EndRotate                   ; WORST = 60 CYCLES

; -----------------------------------------------------------------------------

  ; Check For Thrust
CheckThrust
  lda SWCHA                 ; [0] + 3
  and #%00010000            ; [3] + 2
  beq AccelerateXY          ; [5] + 2/3
  ; Decelerate Ship
DecelerateXY
  ; Skip If X Velocity = 0
  lax SHIPVX_HI             ; [7] + 3
  ora SHIPVX_LO             ; [10] + 3
  beq EndDecelerateX        ; [13] + 2/3
  ; X Veclocity Decrement
  lda SHIPVX_LO             ; [15] + 3
  asl                       ; [18] + 2
  txa                       ; [20] + 2
  rol                       ; [22] + 2
  sec                       ; [24] + 2
  rol                       ; [26] + 2
  sta TEMP2                 ; [28] + 3
  ; Extend Sign
  lda #$7F                  ; [31] + 2
  cmp TEMP2                 ; [33] + 3
  adc #$80                  ; [36] + 2
  sta TEMP                  ; [38] + 3
  ; Decrement X Velocity
  sec                       ; [41] + 2
  lda SHIPVX_LO             ; [43] + 3
  sbc TEMP2                 ; [46] + 3
  sta SHIPVX_LO             ; [49] + 3
  lda SHIPVX_HI             ; [52] + 3
  sbc TEMP                  ; [55] + 3
  sta SHIPVX_HI             ; [58] + 3
EndDecelerateX
  ; Skip If Y Velocity = 0
  lax SHIPVY_HI             ; [61] + 3
  ora SHIPVY_LO             ; [64] + 3
  beq EndDecelerateY        ; [67] + 2/3
  ; Y Velocity Decrement
  lda SHIPVY_LO             ; [69] + 3
  asl                       ; [72] + 2
  txa                       ; [74] + 2
  rol                       ; [76] + 2
  sec                       ; [78] + 2
  rol                       ; [80] + 2
  sta TEMP2                 ; [82] + 3
  ; Extend Sign
  lda #$7F                  ; [85] + 2
  cmp TEMP2                 ; [87] + 3
  adc #$80                  ; [90] + 2
  sta TEMP                  ; [92] + 3
  ; Decrement Y Velocity
  sec                       ; [95] + 2
  lda SHIPVY_LO             ; [97] + 3
  sbc TEMP2                 ; [100] + 3
  sta SHIPVY_LO             ; [103] + 3
  lda SHIPVY_HI             ; [106] + 3
  sbc TEMP                  ; [109] + 3
  sta SHIPVY_HI             ; [112] + 3
EndDecelerateY
  jmp EndThrust             ; [115] + 3
AccelerateXY
  ; Accelerate X Axis
  ldx SHIPDIR_HI            ; [8] + 3
  lda ShipAccelX,X          ; [11] + 4
  bmi DecVelocityX          ; [15] + 2/3
IncVelocityX
  ; Check Max Velocity
  lda ShipMaxLoX,X          ; [17] + 4
  cmp SHIPVX_LO             ; [21] + 3
  lda ShipMaxHiX,X          ; [24] + 4
  sbc SHIPVX_HI             ; [28] + 3
  bvc SkipIncEorX           ; [31] + 2/3
  eor #$80                  ; [33] + 2
SkipIncEorX
  bmi EndAccelerateX        ; [35] + 2/3
  ldy #0                    ; [37] + 2
  beq AccelerateX           ; [39] + 3
DecVelocityX
  ; Check Min Velocity
  lda ShipMaxLoX,X          ; [18] + 4
  cmp SHIPVX_LO             ; [22] + 3
  lda ShipMaxHiX,X          ; [25] + 4
  sbc SHIPVX_HI             ; [29] + 3
  bvc SkipDecEorX           ; [32] + 2/3
  eor #$80                  ; [34] + 2
SkipDecEorX
  bpl EndAccelerateX        ; [36] + 2/3
  ldy #$FF                  ; [38] + 2
  nop                       ; [40] + 2
AccelerateX
  clc                       ; [42] + 2
  lda ShipAccelX,X          ; [44] + 4
  adc SHIPVX_LO             ; [48] + 3
  sta SHIPVX_LO             ; [51] + 3
  tya                       ; [54] + 2
  adc SHIPVX_HI             ; [56] + 3
  sta SHIPVX_HI             ; [59] + 3
EndAccelerateX
  ; Accelerate Y Axis
  lda ShipAccelY,X          ; [62] + 4
  bmi DecVelocityY          ; [66] + 2/3
IncVelocityY
  lda ShipMaxLoY,X          ; [68] + 4
  cmp SHIPVY_LO             ; [72] + 3
  lda ShipMaxHiY,X          ; [75] + 4
  sbc SHIPVY_HI             ; [79] + 3
  bvc SkipIncEorY           ; [82] + 2/3
  eor #$80                  ; [85] + 2
SkipIncEorY
  bmi EndAccelerateY        ; [87] + 2/3
  ldy #0                    ; [89] + 2
  beq AccelerateY           ; [91] + 3
DecVelocityY
  lda ShipMaxLoY,X          ; [69] + 4
  cmp SHIPVY_LO             ; [73] + 3
  lda ShipMaxHiY,X          ; [76] + 4
  sbc SHIPVY_HI             ; [80] + 3
  bvc SkipDecEorY           ; [83] + 2/3
  eor #$80                  ; [85] + 2
SkipDecEorY
  bpl EndAccelerateY        ; [87] + 2/3
  ldy #$FF                  ; [89] + 2
  nop                       ; [91] + 2
AccelerateY
  ; Accelerate
  clc                       ; [93] + 2
  lda ShipAccelY,X          ; [95] + 4
  adc SHIPVY_LO             ; [99] + 3
  sta SHIPVY_LO             ; [102] + 3
  tya                       ; [105] + 2
  adc SHIPVY_HI             ; [107] + 3
  sta SHIPVY_HI             ; [110] + 3
EndAccelerateY
  ; Play Thrust Sound
  lda #8                    ; [113] + 2
  sta AUDC1                 ; [115] + 3
  lda #10                   ; [118] + 2
  sta AUDF1                 ; [120] + 3
  lda #4
  sta AUDV1
EndThrust                   ; WORST = 123 CYCLES

; -----------------------------------------------------------------------------

  ; Find Space for Bullet
  ldx #1                    ; [0] + 2
  lda BULLETDIR+1           ; [2] + 3
  bmi StoreBulletSpace      ; [5] + 2/3
  dex                       ; [7] + 2
  lda BULLETDIR+0           ; [9] + 3
  bmi StoreBulletSpace      ; [12] + 2/3
  dex                       ; [14] + 2
StoreBulletSpace
  stx TEMPBPOS              ; [16] + 3 
                            ; WORST = 19 CYCLES

  ; Check Fire Button
CheckFire
  bit DEBOUNCE              ; [0] + 3
  bpl EndFirePressed        ; [3] + 2/3
  ; Skip If All Bullets Used
  ldx TEMPBPOS              ; [10] + 3
  bmi EndFirePressed        ; [13] + 2/3
  ; Set Bullet to Ship Centre
  ldy SHIPX_HI              ; [15] + 3
  iny                       ; [18] + 2
  sty BULLETX_HI,X          ; [20] + 4
  lda SHIPY_HI              ; [24] + 3
  sta BULLETY_HI,X          ; [27] + 4
  lda SHIPDIR_HI            ; [31] + 3
  sta BULLETDIR,X           ; [34] + 4
  lda #0                    ; [38] + 2
  sta BULLETX_LO,X          ; [40] + 4
  sta BULLETY_LO,X          ; [44] + 4
  lda #BULLETXMAX           ; [48] + 2
  sta BULLETXD,X            ; [50] + 4
  lda #BULLETYMAX           ; [54] + 2
  sta BULLETYD,X            ; [56] + 4
  ; Play Bullet Sound
  lda #50                   ; [60] + 2
  cmp SFX
  bcc EndFirePressed
  lda #34
  sta SFX                   ; [62] + 3
EndFirePressed              ; WORST = 76 CYCLES

; -----------------------------------------------------------------------------
; Move Sprites
; -----------------------------------------------------------------------------

  ; Move Ship Sprite
  clc                       ; [0] + 2
  lda SHIPX_LO              ; [2] + 3
  adc SHIPVX_LO             ; [5] + 3
  sta SHIPX_LO              ; [8] + 3
  lda SHIPX_HI              ; [11] + 3
  adc SHIPVX_HI             ; [14] + 3
  cmp #160                  ; [17] + 2
  bcc StoreX                ; [19] + 2/3
  ; Check Hi/Low Wrap
  lda #0                    ; [21] + 2
  ldx SHIPVX_HI             ; [23] + 3
  bmi XMinWrap              ; [26] + 2/3
  bpl StoreX                ; [28] + 3
XMinWrap
  lda #160                  ; [29] + 2
StoreX
  sta SHIPX_HI              ; [31] + 3
  clc                       ; [34] + 2
  lda SHIPY_LO              ; [36] + 3
  adc SHIPVY_LO             ; [39] + 3
  sta SHIPY_LO              ; [42] + 3
  lda SHIPY_HI              ; [45] + 3
  adc SHIPVY_HI             ; [48] + 3
  cmp #YSTART               ; [51] + 2
  bcc StoreY                ; [53] + 2/3
  lda #0                    ; [55] + 2
  ldx SHIPVY_HI             ; [57] + 3
  bmi YMinWrap              ; [60] + 2/3
  bpl StoreY                ; [62] + 3
YMinWrap
  lda #YSTART               ; [63] + 2
StoreY
  sta SHIPY_HI              ; [65] + 3
                            ; WORST = 68 CYCLES

; -----------------------------------------------------------------------------

MoveBullets
  ; Move Bullet 0
  ldy BULLETDIR+0           ; [0] + 3
  bmi EndMoveBullet0        ; [3] + 2/3
  
  ; Update X Position
  clc                       ; [5] + 2
  lda BulletLoX,Y           ; [7] + 4
  adc BULLETX_LO+0          ; [11] + 3
  sta BULLETX_LO+0          ; [14] + 3
  lax BulletHiX,Y           ; [17] + 4
  adc BULLETX_HI+0          ; [21] + 3
  cmp #159                  ; [24] + 2
  bcc StoreXBullet0         ; [26] + 2/3
  lda #0                    ; [28] + 2
  cpx #%10000000            ; [30] + 2
  bcc StoreXBullet0         ; [32] + 2/3
BulletXMinWrap0
  lda #159                  ; [34] + 2
StoreXBullet0
  cmp BULLETX_HI+0          ; [36] + 3
  beq EndBulletX0           ; [39] + 2/3
  dec BULLETXD+0            ; [41] + 5
  bmi StopBullet0           ; [46] + 2/3
  sta BULLETX_HI+0          ; [48] + 3
EndBulletX0

  ; Update Y Position
  clc                       ; [51] + 2
  lda BulletLoY,Y           ; [53] + 4
  adc BULLETY_LO+0          ; [57] + 3
  sta BULLETY_LO+0          ; [60] + 3
  lax BulletHiY,Y           ; [63] + 4
  adc BULLETY_HI+0          ; [67] + 3
  cmp #YSTART               ; [70] + 2
  bcc StoreYBullet0         ; [72] + 2/3
  lda #0                    ; [74] + 2
  cpx #%10000000            ; [76] + 2
  bcc StoreYBullet0         ; [78] + 2/3
BulletYMinWrap0
  lda #YSTART               ; [80] + 2
StoreYBullet0
  cmp BULLETY_HI+0          ; [82] + 3
  beq EndBulletY0           ; [85] + 2/3
  sta BULLETY_HI+0          ; [87] + 3
  dec BULLETYD+0            ; [90] + 5
  bpl EndBulletY0           ; [95] + 2/3
StopBullet0
  lda #$FF                  ; [97] + 2
  sta BULLETDIR+0           ; [99] + 3
EndBulletY0
EndMoveBullet0              ; WORST = 102 CYCLES

; -----------------------------------------------------------------------------

  ; Move Bullet 1
  ldy BULLETDIR+1           ; [0] + 3
  bmi EndMoveBullet1        ; [3] + 2/3
  
  ; Update X Position
  clc                       ; [5] + 2
  lda BulletLoX,Y           ; [7] + 4
  adc BULLETX_LO+1          ; [11] + 3
  sta BULLETX_LO+1          ; [14] + 3
  lax BulletHiX,Y           ; [17] + 4
  adc BULLETX_HI+1          ; [21] + 3
  cmp #160-1                ; [24] + 2
  bcc StoreXBullet1         ; [26] + 2/3
  lda #0                    ; [28] + 2
  cpx #%10000000            ; [30] + 2
  bcc StoreXBullet1         ; [32] + 2/3
BulletXMinWrap1
  lda #160-1                ; [34] + 2
StoreXBullet1
  cmp BULLETX_HI+1          ; [36] + 3
  beq EndBulletX1           ; [39] + 2/3
  dec BULLETXD+1            ; [41] + 5
  bmi StopBullet1           ; [46] + 2/3
  sta BULLETX_HI+1          ; [48] + 3
EndBulletX1

  ; Update Y Position
  clc                       ; [51] + 2
  lda BulletLoY,Y           ; [53] + 4
  adc BULLETY_LO+1          ; [57] + 3
  sta BULLETY_LO+1          ; [60] + 3
  lax BulletHiY,Y           ; [63] + 4
  adc BULLETY_HI+1          ; [67] + 3
  cmp #YSTART               ; [70] + 2
  bcc StoreYBullet1         ; [72] + 2/3
  lda #0                    ; [74] + 2
  cpx #%10000000            ; [76] + 2
  bcc StoreYBullet1         ; [78] + 2/3
BulletYMinWrap1
  lda #YSTART               ; [80] + 2
StoreYBullet1
  cmp BULLETY_HI+1          ; [82] + 3
  beq EndBulletY1           ; [85] + 2/3
  sta BULLETY_HI+1          ; [87] + 3
  dec BULLETYD+1            ; [90] + 5
  bpl EndBulletY1           ; [95] + 2/3
StopBullet1
  lda #$FF                  ; [97] + 2
  sta BULLETDIR+1           ; [99] + 3
EndBulletY1
EndMoveBullet1              ; WORST = 102 CYCLES

; -----------------------------------------------------------------------------

  ; Move Projectile
  ldx PROJDIR               ; [0] + 4
  bmi EndMoveProjectile     ; [4] + 2/3
  ; Projectile Sound
  ;lda #8                    ; [6] + 2
  ;sta AUDC1                 ; [8] + 3
  ;lsr                       ; [11] + 2
  ;sta AUDV1                 ; [13] + 3
  ;lsr                       ; [16] + 2
  ;sta AUDF1                 ; [18] + 3
  ; Update X Position
  ; clc
  lda PROJX_LO              ; [21] + 4
  adc ShipMaxLoX,X          ; [25] + 4
  sta PROJX_LO              ; [29] + 4
  lda PROJX_HI              ; [33] + 4
  adc ShipMaxHiX,X          ; [37] + 4
  sta PROJX_HI              ; [41] + 4
  cmp #160-4                ; [45] + 2
  bcs StopProjectile        ; [47] + 2/3
  ; Update Y Position
  clc                       ; [49] + 2
  lda PROJY_LO              ; [51] + 4
  adc ShipMaxLoY,X          ; [55] + 4
  sta PROJY_LO              ; [59] + 4
  lda PROJY_HI              ; [63] + 4
  adc ShipMaxHiY,X          ; [67] + 4
  sta PROJY_HI              ; [71] + 4
  cmp #YSTART               ; [75] + 2
  bcc EndMoveProjectile     ; [77] + 2/3
StopProjectile
  lda #$FF                  ; [79] + 2
  sta PROJDIR               ; [81] + 4
EndMoveProjectile           ; WORST = 85 CYCLES

; -----------------------------------------------------------------------------
; Flicker Mines & Bullets
; -----------------------------------------------------------------------------

  ; Flicker Mines
  ldx MPOS                  ; [0] + 3
  dex                       ; [3] + 2
  bpl CheckMine0            ; [5] + 2/3
  ldx #2                    ; [7] + 2
CheckMine0
  lda MINEDIR_R,X           ; [9] + 4
  and #%11100000            ; [13] + 2
  bne StoreMinePos          ; [15] + 2/3
  dex                       ; [17] + 2
  bpl CheckMine1            ; [19] + 2/3
  ldx #2                    ; [21] + 2
CheckMine1
  lda MINEDIR_R,X           ; [23] + 4
  and #%11100000            ; [27] + 2
  bne StoreMinePos          ; [29] + 2/3
  dex                       ; [31] + 2
  bpl StoreMinePos          ; [33] + 2/3
  ldx #2                    ; [35] + 2
StoreMinePos
  stx MPOS                  ; [37] + 3 
                            ; WORST = 40 CYCLES

  ; Flicker Bullets
  ldx BPOS                  ; [0] + 3
  dex                       ; [3] + 2
  bpl CheckBullet0          ; [5] + 2/3
  ldx #1                    ; [7] + 2
CheckBullet0
  lda BULLETDIR,X           ; [9] + 4
  bpl StoreBulletPos        ; [13] + 2/3
  dex                       ; [15] + 2
  bpl StoreBulletPos        ; [17] + 2/3
  ldx #1                    ; [19] + 2
StoreBulletPos
  stx BPOS                  ; [21] + 3 
                            ; WORST = 24 CYCLES

  ; Begin Collision Detection
  jmp EnterCollision

; -----------------------------------------------------------------------------
; Mode Transitions (from Message Screen)
; -----------------------------------------------------------------------------

ModeTransition
  lax MODE
  and #%11100000
  cmp #%10000000
  bcc MainGameTransition
  beq EndGameTransition
  cmp #%10100000
  bne EndTransitions        ; Paranoia
PauseTransition
  ; Check Pause Button
  lda DEBOUNCE
  and #%00100000
  bne EndTransitions
  beq ResumeGameTransition
EndGameTransition
  ; Check Timer
  txa
  and #%00011111
  bne EndTransitions
  ; End of Game
  BEGIN_VBLANK
  jmp EndGame
MainGameTransition
  cmp #%01000000
  beq NextWaveTransition
  bcs NextShipTransition
GetReadyTransition
  ; Skip If Fire Pressed
  bit DEBOUNCE
  bmi ResumeGameTransition
  ; Check Timer
  txa
  and #%00011111
  bne EndTransitions
  beq ResumeGameTransition
NextWaveTransition
  ; Skip If Fire Pressed
  bit DEBOUNCE
  bmi EndWaveTransition
  ; Check Timer
  txa
  and #%00011111
  bne EndTransitions
EndWaveTransition
  ; Setup Next Wave
  jsr WaveSetup
  jmp ResumeGameTransition
NextShipTransition
  ; Skip If Fire Pressed
  bit DEBOUNCE
  bmi EndShipTransition
  ; Check Timer
  txa
  and #%00011111
  bne EndTransitions
EndShipTransition
  jsr ShipSetup
ResumeGameTransition
  lda #0
  sta MODE
EndTransitions
  jmp MainLoop

; -----------------------------------------------------------------------------

WaveSetup
  ; Ring Colour Index
  lda WAVE
  and #%00000011            ; 0-> 3
  sta TEMP
  lda COLMODE
  asl
  asl                       ; 0, 4, 8
  ; clc
  adc TEMP
  tay

  ; Set Ring Colours
  lda RingCols0,Y
  sta RINGCOL0
  lda RingCols1,Y
  sta RINGCOL1
  lda RingCols2,Y
  sta RINGCOL2

  ; Sector Count
  lda #36
  sta SECTORS

  ; Set Sector Offsets
  ldx #11
  lda #0
  sta BREACH
SectorInitLoop
  sta SECTORMAP0_W,X
  sta SECTORMAP1_W,X
  sta SECTORMAP2_W,X
  ldy Sector0End,X
  sty OFFSET0,X
  ldy Sector1End,X
  sty OFFSET1,X
  ldy Sector2End,X
  sty OFFSET2,X
  dex
  bpl SectorInitLoop
  jmp ShipSetup

  DC.B  "KIRK"

; -----------------------------------------------------------------------------

  ALIGN 256

ShipSetup
  ; Set Ship Position
  lda #(MIDDLEX+45)
  sta SHIPX_HI
  lda RANDOM
  lsr
  and #%00000011
  tay
  lda ShipYStart,Y
  sta SHIPY_HI
  
  ; Reset Game Variables
  lda #0
  ldx #<SHIPX_LO
ClearVarsLoop
  sta $00,X
  inx
  cpx #<SECTORS
  bne ClearVarsLoop

  ; Clear Mine Modes
  sta MINEDIR_W+0
  sta MINEDIR_W+1
  sta MINEDIR_W+2

  ; Initial Mine Counters
  lda #16
  sta MINECOUNT_W+0
  asl
  sta MINECOUNT_W+1
  asl
  sta MINECOUNT_W+2

  ; Hide Bullets & Projectile
  lda #$FF
  sta BCLEAR
  sta BULLETDIR+0
  sta BULLETDIR+1
  sta PROJDIR
  rts

; -----------------------------------------------------------------------------
; Game Messages
; -----------------------------------------------------------------------------

ShowMessages
  ; Set Number and Message Pointers (30Hz)
  sta WSYNC                 ; [0]
  lda CYCLE                 ; [0] + 3
  lsr                       ; [3] + 2
  bcs TextBPointers         ; [5] + 2/3
TextAPointers
  lda #<NumberTableA        ; [7] + 2
  sta NPTR                  ; [9] + 3
  lda #>NumberTableA        ; [12] + 2
  ldx #<MessageTableA       ; [14] + 2
  ldy #>MessageTableA       ; [16] + 2
  jmp StoreTextPointers     ; [18] + 3
TextBPointers
  lda #<NumberTableB        ; [8] + 2
  sta NPTR                  ; [10] + 3
  lda #>NumberTableB        ; [13] + 2
  ldx #<MessageTableB       ; [15] + 2
  ldy #>MessageTableB       ; [17] + 2
  nop                       ; [19] + 2
StoreTextPointers
  sta NPTR+1                ; [21] + 3
  stx LPTR                  ; [24] + 3
  sty LPTR+1                ; [27] + 3

  ; Reset Sprite Copies & Movements
  lda #0                    ; [30] + 2
  sta NUSIZ0                ; [32] + 3
  sta NUSIZ1                ; [35] + 3
  sta ENAM1                 ; [38] + 3
  sta HMCLR                 ; [41] + 3

  ; Silence Audio
  sta AUDV1                 ; [44] + 3

  ; Set Star Colour
  ldx COLMODE               ; [47] + 3
  lda StarCols0,X           ; [50] + 4
  sta COLUP1                ; [54] + 3

  ; Star Position
  nop                       ; [57] + 2
  sta RESM1                 ; [59] + 3    = 62

  ; Begin Frame
WaitMessagesVblank
  lda INTIM
  bne WaitMessagesVblank
  sta VBLANK

; -----------------------------------------------------------------------------

  ; Set Line Counter & Jump To Top Kernel
  ldx #(YOFFSET-1)
  lda CYCLE
  lsr
  bcc TopStarLoopB

TopStarLoopA
  sta WSYNC                 ; [0]
  ; Draw Stars
  lda TopStars_A,X          ; [0] + 4
  sta ENAM1                 ; [4] + 3
  sta HMM1                  ; [7] + 3
  lda #0                    ; [10] + 2
  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3
  ; Clear Stars
  sta ENAM1                 ; [3] + 3
  dex                       ; [6] + 2
  bpl TopStarLoopA          ; [8] + 2/3
  bmi EndTopStarLoop

TopStarLoopB
  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3
  ; Clear Stars
  lda #0                    ; [3] + 2
  sta ENAM1                 ; [5] + 3
  ; Load Star Data
  lda TopStars_B,X          ; [8] + 4
  ; Draw Stars
  sta WSYNC                 ; [0]
  sta HMM1                  ; [0] + 3
  sta ENAM1                 ; [3] + 3
  dex                       ; [6] + 2
  bpl TopStarLoopB          ; [8] + 2/3
EndTopStarLoop

; -----------------------------------------------------------------------------

  ; Check Game Mode
  lda MODE
  and #%11100000
  cmp #%00100000
  bne ShowScore
ShowGetReady
  ; Show Get Ready Message
  ldy #34
  jsr SkipLines
  ldy #0
  jsr LoadMessage
  ldy #4
  jsr TextDisplay
  ldy #36
  jsr SkipLines
  jmp ShowBottomStars

ShowScore
  ; Display Score
  ldy #16
  jsr SkipLines
  jsr LoadScore
  ldy #10
  jsr TextDisplay
  ldy #12
  jsr SkipLines
  lax MODE
  and #%11100000
  jmp ShowMessagesB
EndShowMessagesA
  if (>ShowMessages != >EndShowMessagesA)
    echo "WARNING: ShowMessagesA crosses a page boundary!"
  endif

  DC.B  "SPOCK"

  ALIGN 256

ShowMessagesB
  ; Check Game Mode
  cmp #%01000000
  beq ShowWave
  cmp #%01100000
  beq ShowShips
  cmp #%10000000
  beq ShowGameOver
ShowPaused
  ; Display Paused Message
  ldy #6
  jsr SkipLines
  ldy #18
  jsr LoadMessage
  ldy #4
  jsr TextDisplay
  jmp ShowBottomGap
ShowGameOver
  ; Display Game Over Message
  ldy #6
  jsr SkipLines
  ldy #12
  jsr LoadMessage
  ldy #4
  jsr TextDisplay
  jmp ShowBottomGap
ShowShips
  txa
  and #%00011111
  cmp #%00001111
  bcc ShowWave2
ShowShips2
  ; Check For Final Ship
  lda SHIPS
  and #%00111111
  bne ShowRemainingShips
  ; Display Last Ship Message
  ldy #6
  jsr SkipLines
  ldy #6
  jsr LoadMessage
  ldy #4
  jsr TextDisplay
  jmp ShowBottomGap
ShowRemainingShips
  jsr LoadShips
  ldy #10
  jsr TextDisplay
  jmp ShowBottomGap
ShowWave
  txa
  and #%00011111
  cmp #%00001111
  bcc ShowShips2
ShowWave2
  jsr LoadWave
  ldy #10
  jsr TextDisplay

ShowBottomGap
  ldy #19
  jsr SkipLines
ShowBottomStars
  ldx COLMODE
  lda StarCols0,X
  sta COLUP1

; -----------------------------------------------------------------------------

  ; Set Line Counter
  ldx #(YOFFSET-1)
  lda CYCLE
  lsr
  bcc BottomStarLoopB

BottomStarLoopA
  sta WSYNC                 ; [0]
  ; Draw Stars
  lda BottomStars_A,X       ; [21] + 4
  sta ENAM1                 ; [25] + 3
  sta HMM1                  ; [28] + 3
  lda #0                    ; [31] + 2
  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3
  ; Clear Stars
  sta ENAM1                 ; [3] + 3
  dex                       ; [14] + 2
  bpl BottomStarLoopA       ; [16] + 2/3
  bmi EndBottomStarLoop

BottomStarLoopB
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3
  ; Clear Stars
  lda #0                    ; [3] + 2
  sta ENAM1                 ; [5] + 3
  ; Load Star Data
  lda BottomStars_B,X       ; [8] + 4
  ; Draw Stars
  sta WSYNC                 ; [0]
  sta HMM1                  ; [0] + 3
  sta ENAM1                 ; [3] + 3
  dex                       ; [6] + 2
  bpl BottomStarLoopB       ; [8] + 2/3
EndBottomStarLoop

  ; Begin Overscan
  lda #2
  sta WSYNC
  sta VBLANK
  lda #OVERDELAY
  sta TIM64T

  ; Restore Lines
  jsr SetKernelLines
  
  ; Update Modes
  jmp ModeTransition

EndShowMessagesB
  if (>ShowMessagesB != >EndShowMessagesB)
    echo "WARNING: ShowMessagesB crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------
; Subroutines
; -----------------------------------------------------------------------------

SetKernelLines
  ; Set Grid Pointers (Low)
  lda #<(GRID_R+0)-YOFFSET      ; [0] + 2
  sta LINE0                     ; [2] + 3
  lda #<(GRID_R+40)-YOFFSET     ; [5] + 2
  sta LINE1                     ; [7] + 3
  lda #<(GRID_R+80)-YOFFSET     ; [10] + 2
  sta LINE2                     ; [12] + 3
  lda #<(GRID_R+120)-YOFFSET    ; [15] + 2
  sta LINE3                     ; [17] + 3
  lda #<(GRID_R+160)-YOFFSET    ; [20] + 2
  sta LINE4                     ; [22] + 3
  ; Set Grid Pointers (High)
  lda #>GRID_R              ; [25] + 2
  sta LINE0+1               ; [27] + 3
  sta LINE1+1               ; [30] + 3
  sta LINE2+1               ; [33] + 3
  sta LINE3+1               ; [36] + 3
  sta LINE4+1               ; [39] + 3
  rts                       ; [42] + 6 
                            ; TOTAL = 48 CYCLES

  ; Load Message Pointers
LoadMessage
  lda (LPTR),Y              ; [0] + 5
  sta PTR0                  ; [5] + 3
  iny                       ; [8] + 2
  lda (LPTR),Y              ; [10] + 5
  sta PTR1                  ; [15] + 3
  iny                       ; [18] + 2
  lda (LPTR),Y              ; [20] + 5
  sta PTR2                  ; [25] + 3
  iny                       ; [28] + 2
  lda (LPTR),Y              ; [30] + 5
  sta PTR3                  ; [35] + 3
  iny                       ; [38] + 2
  lda (LPTR),Y              ; [40] + 5
  sta PTR4                  ; [45] + 3
  iny                       ; [48] + 2
  lda (LPTR),Y              ; [50] + 5
  sta PTR5                  ; [55] + 3
  lda LPTR+1                ; [58] + 3
  jmp LoadHi                ; [61] + 3 + 22
                            ; TOTAL = 86 CYCLES

  DC.B  "BONES"

; -----------------------------------------------------------------------------

  ALIGN 256
  
  ; Load Score Pointers
LoadScore
  ; Digits 5 & 4
  lax SCORE+2               ; [0] + 3
  lsr                       ; [3] + 2
  lsr                       ; [5] + 2
  lsr                       ; [7] + 2
  lsr                       ; [9] + 2
  tay                       ; [11] + 2
  lda (NPTR),Y              ; [13] + 5
  sta PTR0                  ; [18] + 3
  txa                       ; [21] + 2
  and #%00001111            ; [23] + 2
  tay                       ; [25] + 2
  lda (NPTR),Y              ; [27] + 5
  sta PTR1                  ; [32] + 3
  ; Digits 3 & 2
  lax SCORE+1               ; [35] + 3
  lsr                       ; [38] + 2
  lsr                       ; [40] + 2
  lsr                       ; [42] + 2
  lsr                       ; [44] + 2
  tay                       ; [46] + 2
  lda (NPTR),Y              ; [48] + 5
  sta PTR2                  ; [53] + 3
  txa                       ; [56] + 2
  and #%00001111            ; [58] + 2
  tay                       ; [60] + 2
  lda (NPTR),Y              ; [62] + 5
  sta PTR3                  ; [67] + 3
  ; Digits 1 & 0
  lax SCORE+0               ; [70] + 3
  lsr                       ; [73] + 2
  lsr                       ; [75] + 2
  lsr                       ; [77] + 2
  lsr                       ; [79] + 2
  tay                       ; [81] + 2
  lda (NPTR),Y              ; [83] + 5
  sta PTR4                  ; [88] + 3
  txa                       ; [91] + 2
  and #%00001111            ; [93] + 2
  tay                       ; [95] + 2
  lda (NPTR),Y              ; [97] + 5
  sta PTR5                  ; [102] + 3
  lda NPTR+1                ; [105] + 3
LoadHi
  ; High Pointers
  sta PTR0+1                ; [108] + 3
  sta PTR1+1                ; [111] + 3
  sta PTR2+1                ; [114] + 3
  sta PTR3+1                ; [117] + 3
  sta PTR4+1                ; [120] + 3
  sta PTR5+1                ; [123] + 3
  rts                       ; [126] + 6
EndLoadScore                ; TOTAL = 132 CYCLES
  if (>LoadScore != >EndLoadScore)
    echo "WARNING: LoadScore crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Load Wave & Ships Messages
LoadShips
  lda SHIPS                 ; [0] + 3
  ldy #28                   ; [3] + 2
  bne LoadMixed             ; [5] + 3
LoadWave
  lda WAVE                  ; [0] + 3
  ldy #24                   ; [3] + 2
  SLEEP 3                   ; [5] + 3
LoadMixed
  ; Message Pointers
  clc                       ; [8] + 2
  and #%00111111            ; [10] + 2
  adc #1                    ; [12] + 2     Show + 1
  sta TEMP                  ; [14] + 3
  lda (LPTR),Y              ; [17] + 5
  sta PTR0                  ; [22] + 3
  iny                       ; [25] + 2
  lda (LPTR),Y              ; [27] + 5
  sta PTR1                  ; [32] + 3
  iny                       ; [35] + 2
  lda (LPTR),Y              ; [37] + 5
  sta PTR2                  ; [42] + 3
  iny                       ; [45] + 2
  lda (LPTR),Y              ; [47] + 5
  sta PTR3                  ; [52] + 3
  ; Number Pointers
  ldy TEMP                  ; [55] + 3
  lax BCDTable,Y            ; [58] + 4
  lsr                       ; [62] + 2
  lsr                       ; [64] + 2
  lsr                       ; [66] + 2
  lsr                       ; [68] + 2
  tay                       ; [70] + 2
  lda (NPTR),Y              ; [72] + 4
  sta PTR4                  ; [76] + 3
  txa                       ; [79] + 2
  and #%00001111            ; [81] + 2
  tay                       ; [83] + 2
  lda (NPTR),Y              ; [85] + 5
  sta PTR5                  ; [90] + 3
  ; High Pointers
  lda LPTR+1                ; [93] + 3
  sta PTR0+1                ; [96] + 3
  sta PTR1+1                ; [99] + 3
  sta PTR2+1                ; [102] + 3
  sta PTR3+1                ; [105] + 3
  lda NPTR+1                ; [108] + 3
  sta PTR4+1                ; [111] + 3
  sta PTR5+1                ; [114] + 3
  rts                       ; [117] + 6 
EndLoadWave                 ; TOTAL = 123 CYCLES
  if (>LoadWave != >EndLoadWave)
    echo "WARNING: LoadWave crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

TextDisplay
  sta WSYNC                 ; [0]

  ; Store Text Height
  sty ROW                   ; [0] + 3

  ; 3 Copies Close
  lda #%00000011            ; [3] + 2
  sta NUSIZ0                ; [5] + 3
  sta NUSIZ1                ; [8] + 3

  ; Sprite Colours
  ldx COLMODE               ; [11] + 3
  lda TextCols0,X           ; [14] + 4
  sta COLUP0                ; [18] + 3
  sta COLUP1                ; [21] + 3

  ; Delay Sprites
  lda #%00010001            ; [24] + 2
  sta VDELP0                ; [26] + 3
  sta VDELP1                ; [29] + 3
  
  ; Fine Positions
  sta HMCLR                 ; [32] + 3
  sta HMP1                  ; [35] + 3

  ; Position Sprites
  sta RESP0                 ; [38] + 3      = 41
  sta RESP1                 ; [41] + 3      = 44

  ; Load First Chars
  lax (PTR0),Y              ; [44] + 5
  lda (PTR1),Y              ; [49] + 5

  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3       > 74 < 4
  stx GRP0                  ; [3] + 3
  jmp StartText             ; [6] + 3
TextLoop
  ; Fetch Current Line
  ldy ROW                   ; [63] + 3
  SLEEP 6                   ; [66] + 6
  ; Display First 3 Chars
  lda (PTR0),Y              ; [72] + 5
  sta GRP0                  ; [1] + 3       > 54
  lda (PTR1),Y              ; [4] + 5
StartText
  sta GRP1                  ; [9] + 3       < 43
  lda (PTR2),Y              ; [12] + 5
  sta GRP0                  ; [17] + 3      < 46
  ; Pre-fetch Remaining 3 Chars
  lax (PTR3),Y              ; [20] + 5
  lda (PTR4),Y              ; [25] + 5
  sta TEMP                  ; [30] + 3
  lda (PTR5),Y              ; [33] + 5
  tay                       ; [38] + 2
  lda TEMP                  ; [40] + 3
  ; Display Remaining 3 Chars
  stx GRP1                  ; [43] + 3      > 45 < 48
  sta GRP0                  ; [46] + 3      > 48 < 51
  sty GRP1                  ; [49] + 3      > 50 < 54
  sta GRP0                  ; [52] + 3      > 53 < 56
  ; Update Counter
  dec ROW                   ; [55] + 5
  bpl TextLoop              ; [60] + 2/3
  ; Clear Sprite Data
  lda #0                    ; [62] + 2
  sta GRP1                  ; [64] + 3
  sta GRP0                  ; [67] + 3
  sta GRP1                  ; [70] + 3
  sta VDELP0                ; [73] + 3
  sta VDELP1                ; [0] + 3
  sta NUSIZ0                ; [3] + 3
  sta NUSIZ1                ; [6] + 3
  rts                       ; [9] + 6

  ALIGN 256

  ; Skip Scanlines Subroutine
SkipLines
  sta WSYNC
  dey
  bpl SkipLines
  rts

; -----------------------------------------------------------------------------
; Game Data
; -----------------------------------------------------------------------------

  ; Message Pointers A
MessageTableA
  DC.B  <GetReady0_A, <GetReady1_A, <GetReady2_A
  DC.B  <GetReady3_A, <GetReady4_A, <GetReady5_A
  DC.B  <LastShip0_A, <LastShip1_A, <LastShip2_A
  DC.B  <LastShip3_A, <LastShip4_A, <LastShip5_A
  DC.B  <GameOver0_A, <GameOver1_A, <GameOver2_A
  DC.B  <GameOver3_A,< GameOver4_A, <GameOver5_A
  DC.B  <Paused0_A, <Paused1_A, <Paused2_A
  DC.B  <Paused3_A, <Paused4_A, <Paused0_A
  DC.B  <Wave0_A, <Wave1_A, <Wave2_A, <Wave3_A
  DC.B  <ShipsLeft0_A, ShipsLeft1_A, ShipsLeft2_A, Wave3_A

  ; Messages A
TextA
GetReady0_A                 ; Get Ready
  DC.B  %00011110
  DC.B  %00000010
  DC.B  %00011110
  DC.B  %00010010
  DC.B  %00011110
GetReady1_A
  DC.B  %11110010
  DC.B  %10000010
  DC.B  %11110010
  DC.B  %10010010
  DC.B  %11110111
GetReady2_A
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00011000
  DC.B  %00010110
GetReady3_A
  DC.B  %11110111
  DC.B  %10000100
  DC.B  %11110111
  DC.B  %10010000
  DC.B  %11110111
GetReady4_A
  DC.B  %10111101
  DC.B  %10100100
  DC.B  %10111101
  DC.B  %10000101
  DC.B  %10000101
GetReady5_A
  DC.B  %11101000
  DC.B  %00100000
  DC.B  %11101000
  DC.B  %00101000
  DC.B  %00101000
LastShip0_A                 ; Last Ship
  DC.B  %00000111
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
LastShip1_A
  DC.B  %10111101
  DC.B  %00100100
  DC.B  %00111101
  DC.B  %00000101
  DC.B  %00111101
LastShip2_A
  DC.B  %11100100
  DC.B  %00100100
  DC.B  %11100100
  DC.B  %00000100
  DC.B  %11101110
LastShip3_A
  DC.B  %00111101
  DC.B  %00000101
  DC.B  %00111101
  DC.B  %00100001
  DC.B  %00111101
LastShip4_A
  DC.B  %00101010
  DC.B  %00101010
  DC.B  %11101011
  DC.B  %00101010
  DC.B  %00101011
LastShip5_A
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %11000000
  DC.B  %01000000
  DC.B  %11000000
GameOver0_A                 ; Game Over
  DC.B  %00111101
  DC.B  %00000101
  DC.B  %00111101
  DC.B  %00100100
  DC.B  %00111101
GameOver1_A
  DC.B  %11101010
  DC.B  %00101010
  DC.B  %11101010
  DC.B  %00101010
  DC.B  %11101101
GameOver2_A
  DC.B  %10111100
  DC.B  %10100000
  DC.B  %10111100
  DC.B  %10100100
  DC.B  %00111100
GameOver3_A
  DC.B  %01111000
  DC.B  %01001001
  DC.B  %01001001
  DC.B  %01001010
  DC.B  %01111010
GameOver4_A
  DC.B  %10001111
  DC.B  %01001000
  DC.B  %01001111
  DC.B  %00101001
  DC.B  %00101111
GameOver5_A
  DC.B  %01000000
  DC.B  %01000000
  DC.B  %01000000
  DC.B  %01100000
  DC.B  %01011000
Paused0_A                   ; Paused
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Paused1_A
  DC.B  %01000011
  DC.B  %01000010
  DC.B  %01111011
  DC.B  %01001000
  DC.B  %01111011
Paused2_A
  DC.B  %11011110
  DC.B  %01010010
  DC.B  %11010010
  DC.B  %01010010
  DC.B  %11010010
Paused3_A
  DC.B  %11110111 
  DC.B  %00010100 
  DC.B  %11110111 
  DC.B  %10000100 
  DC.B  %11110111 
Paused4_A
  DC.B  %10111100
  DC.B  %00100100
  DC.B  %10111100
  DC.B  %10000100
  DC.B  %10000100
Wave0_A                     ; Wave
  DC.B  %00110110
  DC.B  %00101010
  DC.B  %00101010
  DC.B  %00100010
  DC.B  %00100010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Wave1_A
  DC.B  %11110001
  DC.B  %10010010
  DC.B  %11110010
  DC.B  %00010100
  DC.B  %11110100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Wave2_A
  DC.B  %00011110
  DC.B  %10010000
  DC.B  %10011110
  DC.B  %01010010
  DC.B  %01011110
Wave3_A
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
ShipsLeft0_A                ; Ships Left
  DC.B  %00111101
  DC.B  %00000101
  DC.B  %00111101
  DC.B  %00100001
  DC.B  %00111101
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
ShipsLeft1_A
  DC.B  %00101010
  DC.B  %00101010
  DC.B  %11101011
  DC.B  %00101010
  DC.B  %00101011
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
ShipsLeft2_A
  DC.B  %00011110
  DC.B  %00000010
  DC.B  %11011110
  DC.B  %01010000
  DC.B  %11011110
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
EndTextA
  if (>TextA != >EndTextA)
    echo "WARNING: TextA crosses a page boundary!"
  endif

  ; Ship X Acceleration Table (+0.1 pixels/frame)
ShipAccelX
  DC.B  %00000000, %00000101, %00001010, %00001110
  DC.B  %00010010, %00010101, %00011000, %00011001
  DC.B  %00011010, %00011001, %00011000, %00010101
  DC.B  %00010010, %00001110, %00001010, %00000101
  DC.B  %00000000, %11111011, %11110110, %11110010
  DC.B  %11101110, %11101011, %11101000, %11100111
  DC.B  %11100110, %11100111, %11101000, %11101011
  DC.B  %11101110, %11110010, %11110110, %11111011
EndShipAccelX
  ;if (>ShipAccelX != >EndShipAccelX)
  ;  echo "WARNING: ShipAccelX crosses a page boundary!"
  ;endif

  ; DC.B  "SULU"

  ALIGN 256

  ; Message Pointers B
MessageTableB
  DC.B  <GetReady0_B, <GetReady1_B, <GetReady2_B
  DC.B  <GetReady3_B, <GetReady4_B, <GetReady5_B
  DC.B  <LastShip0_B, <LastShip1_B, <LastShip2_B
  DC.B  <LastShip3_B, <LastShip4_B, <LastShip5_B
  DC.B  <GameOver0_B, <GameOver1_B, <GameOver2_B
  DC.B  <GameOver3_B,< GameOver4_B, <GameOver5_B
  DC.B  <Paused0_B, <Paused1_B, <Paused2_B
  DC.B  <Paused3_B, <Paused4_B, <Paused0_B
  DC.B  <Wave0_B, <Wave1_B, <Wave2_B, <Wave3_B
  DC.B  <ShipsLeft0_B, ShipsLeft1_B, ShipsLeft2_B, Wave3_B

  ; Messages B
TextB
GetReady0_B
  DC.B  %00000010
  DC.B  %00000000
  DC.B  %00010010
  DC.B  %00000000
  DC.B  %00010010
GetReady1_B
  DC.B  %10000000
  DC.B  %00000000
  DC.B  %10010000
  DC.B  %00000000
  DC.B  %10010010
GetReady2_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00010000
  DC.B  %00000100
GetReady3_B
  DC.B  %10000100
  DC.B  %00000000
  DC.B  %10010100
  DC.B  %00000000
  DC.B  %10010000
GetReady4_B
  DC.B  %10100100
  DC.B  %00000000
  DC.B  %10100101
  DC.B  %00000000
  DC.B  %10000000
GetReady5_B
  DC.B  %00100000
  DC.B  %00000000
  DC.B  %00100000
  DC.B  %00000000
  DC.B  %00000000
LastShip0_B
  DC.B  %00000100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
LastShip1_B
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00100101
  DC.B  %00000000
  DC.B  %00000101
LastShip2_B
  DC.B  %00100000
  DC.B  %00000000
  DC.B  %00100000
  DC.B  %00000000
  DC.B  %00000100
LastShip3_B
  DC.B  %00000100
  DC.B  %00000000
  DC.B  %00100101
  DC.B  %00000000
  DC.B  %00100000
LastShip4_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100010
  DC.B  %00000000
  DC.B  %00000010
LastShip5_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01000000
  DC.B  %00000000
  DC.B  %01000000
GameOver0_B
  DC.B  %00000101
  DC.B  %00000000
  DC.B  %00100101
  DC.B  %00000000
  DC.B  %00100100
GameOver1_B
  DC.B  %00100000
  DC.B  %00000000
  DC.B  %00100000
  DC.B  %00000010
  DC.B  %00101001
GameOver2_B
  DC.B  %00100000
  DC.B  %00000000
  DC.B  %00100100
  DC.B  %10000000
  DC.B  %00100100
GameOver3_B
  DC.B  %01001000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01001000
GameOver4_B
  DC.B  %10001000
  DC.B  %00000000
  DC.B  %00001001
  DC.B  %00000000
  DC.B  %00001001
GameOver5_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01000000
  DC.B  %00010000
Paused0_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Paused1_B
  DC.B  %00000010
  DC.B  %00000000
  DC.B  %01001010
  DC.B  %00000000
  DC.B  %01001000
Paused2_B
  DC.B  %01010010
  DC.B  %00000000
  DC.B  %01000000
  DC.B  %00000000
  DC.B  %01000000
Paused3_B
  DC.B  %00010100
  DC.B  %00000000
  DC.B  %10010100
  DC.B  %00000000
  DC.B  %10000100
Paused4_B
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %10100100
  DC.B  %00000000
  DC.B  %10000000
Wave0_B
  DC.B  %00100010
  DC.B  %00001000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Wave1_B
  DC.B  %10010001
  DC.B  %00000000
  DC.B  %10010000
  DC.B  %00000000
  DC.B  %00010000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Wave2_B
  DC.B  %00010000
  DC.B  %00000000
  DC.B  %00010010
  DC.B  %00000000
  DC.B  %00010010
Wave3_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
ShipsLeft0_B
  DC.B  %00000100
  DC.B  %00000000
  DC.B  %00100101
  DC.B  %00000000
  DC.B  %00100000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
ShipsLeft1_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100010
  DC.B  %00000000
  DC.B  %00000010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
ShipsLeft2_B
  DC.B  %00000010
  DC.B  %00000000
  DC.B  %01010010
  DC.B  %00000000
  DC.B  %01010000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
EndTextB
  if (>TextB != >EndTextB)
    echo "WARNING: TextB crosses a page boundary!"
  endif

  ; Ship Y Acceleration Table (+0.1 pixels/frame)
ShipAccelY
  DC.B  %00011010, %00011001, %00011000, %00010101
  DC.B  %00010010, %00001110, %00001010, %00000101
  DC.B  %00000000, %11111011, %11110110, %11110010
  DC.B  %11101110, %11101011, %11101000, %11100111
  DC.B  %11100110, %11100111, %11101000, %11101011
  DC.B  %11101110, %11110010, %11110110, %11111011
  DC.B  %00000000, %00000101, %00001010, %00001110
  DC.B  %00010010, %00010101, %00011000, %00011001
EndShipAccelY
  if (>ShipAccelY != >EndShipAccelY)
    echo "WARNING: ShipAccelY crosses a page boundary!"
  endif

ShipYStart
  DC.B  MIDDLEY+30, MIDDLEY+15, MIDDLEY-15, MIDDLEY-30
EndShipYStart
  if (>ShipYStart != >EndShipYStart)
    echo "WARNING: ShipYStart crosses a page boundary!"
  endif

  ; DC.B  "UHURA"

  ALIGN 256

  ; Number Pointers
NumberTableA
  DC.B  <Number0_A, <Number1_A, <Number2_A, <Number3_A, <Number4_A
  DC.B  <Number5_A, <Number6_A, <Number7_A, <Number8_A, <Number9_A
NumberTableB
  DC.B  <Number0_B, <Number1_B, <Number2_B, <Number3_B, <Number4_B
  DC.B  <Number5_B, <Number6_B, <Number7_B, <Number8_B, <Number9_B

  ; NumbersA
Number1_A
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00010000
Number7_A
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00010000
  DC.B  %00001000
  DC.B  %00001000
  DC.B  %00001000
  DC.B  %00001000
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
Number0_A
  DC.B  %00111100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
Number3_A
  DC.B  %00111100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00011100
  DC.B  %00001000
  DC.B  %00001000
  DC.B  %00000100
  DC.B  %00000100
Number9_A
  DC.B  %00111100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
Number8_A
  DC.B  %00111100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
Number6_A
  DC.B  %00111100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
  DC.B  %00100100
Number2_A
  DC.B  %00111100
  DC.B  %00100000
  DC.B  %00100000
  DC.B  %00100000
  DC.B  %00100000
Number5_A
  DC.B  %00111100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00111100
  DC.B  %00100000
  DC.B  %00100000
  DC.B  %00100000
  DC.B  %00100000
  DC.B  %00111100
Number4_A
  DC.B  %00001000
  DC.B  %00001000
  DC.B  %00001000
  DC.B  %00001000
  DC.B  %00001000
  DC.B  %00111100
  DC.B  %00101000
  DC.B  %00101000
  DC.B  %00101000
  DC.B  %00011000
  DC.B  %00011000

  ; Numbers B
Number9_B
  DC.B  %00000100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Number0_B
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Number4_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00011000
Number1_B
  DC.B  %00000000
Number7_B
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000100
Number8_B
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Number6_B
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Number2_B
  DC.B  %00100000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Number3_B
  DC.B  %00000100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00010100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
Number5_B
  DC.B  %00000100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100100
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100000

SectorOffsets
Sector0End
  DC.B  0, 9, 18, 27, 36, 45, 54, 63, 72, 81, 90, 99
Sector1End
  DC.B  0, 7, 14, 21, 28, 35, 42, 49, 56, 63, 70, 77
Sector2End
  DC.B  0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55
EndSectorOffsets
  if (>SectorOffsets != >EndSectorOffsets)
    echo "WARNING: SectorOffsets table crosses a page boundary!"
  endif

;HumOffset
;  DC.B  %00000000   ; 0
;  DC.B  %00000001   ; +1
;  DC.B  %00000000   ; 0
;  DC.B  %11111111   ; -1

SirenData
  DC.B $45,$46,$45,$33,$34,$33,$23,$32,$23,$22,$22,$21,$12
  DC.B $12,$11,$22,$13,$22,$13,$24,$33,$24,$35,$34,$45,$36
  
  DC.B  "CHEKOV"

  ALIGN 256

  ; Max Ship Speed (1.5 pixels/frame)
ShipMaxLoX
  DC.B  %00000000, %01001011, %10010011, %11010101
  DC.B  %00010000, %00111111, %01100011, %01111001
  DC.B  %10000000, %01111001, %01100011, %00111111
  DC.B  %00010000, %11010101, %10010011, %01001011
  DC.B  %00000000, %10110101, %01101101, %00101011
  DC.B  %11110000, %11000001, %10011101, %10000111
  DC.B  %10000000, %10000111, %10011101, %11000001
  DC.B  %11110000, %00101011, %01101101, %10110101
ShipMaxHiX
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000001, %00000001, %00000001, %00000001
  DC.B  %00000001, %00000001, %00000001, %00000001
  DC.B  %00000001, %00000000, %00000000, %00000000
  DC.B  %00000000, %11111111, %11111111, %11111111
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111111, %11111111, %11111111
ShipMaxLoY
  DC.B  %10000000, %01111001, %01100011, %00111111
  DC.B  %00010000, %11010101, %10010011, %01001011
  DC.B  %00000000, %10110101, %01101101, %00101011
  DC.B  %11110000, %11000001, %10011101, %10000111
  DC.B  %10000000, %10000111, %10011101, %11000001
  DC.B  %11110000, %00101011, %01101101, %10110101
  DC.B  %00000000, %01001011, %10010011, %11010101
  DC.B  %00010000, %00111111, %01100011, %01111001
ShipMaxHiY
  DC.B  %00000001, %00000001, %00000001, %00000001
  DC.B  %00000001, %00000000, %00000000, %00000000
  DC.B  %00000000, %11111111, %11111111, %11111111
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111111, %11111111, %11111111
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000001, %00000001, %00000001, %00000001
  
  ; Bullet Velocity Table (2 pixels/frame)
BulletLoX
  DC.B  %00000000, %01100100, %11000100, %00011100
  DC.B  %01101010, %10101010, %11011001, %11110110
  DC.B  %00000000, %11110110, %11011001, %10101010
  DC.B  %01101010, %00011100, %11000100, %01100100
  DC.B  %00000000, %10011100, %00111100, %11100100
  DC.B  %10010110, %01010110, %00100111, %00001010
  DC.B  %00000000, %00001010, %00100111, %01010110
  DC.B  %10010110, %11100100, %00111100, %10011100
BulletHiX
  DC.B  %00000000, %00000000, %00000000, %00000001
  DC.B  %00000001, %00000001, %00000001, %00000001
  DC.B  %00000010, %00000001, %00000001, %00000001
  DC.B  %00000001, %00000001, %00000000, %00000000
  DC.B  %00000000, %11111111, %11111111, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111111, %11111111
BulletLoY
  DC.B  %00000000, %11110110, %11011001, %10101010
  DC.B  %01101010, %00011100, %11000100, %01100100
  DC.B  %00000000, %10011100, %00111100, %11100100
  DC.B  %10010110, %01010110, %00100111, %00001010
  DC.B  %00000000, %00001010, %00100111, %01010110
  DC.B  %10010110, %11100100, %00111100, %10011100
  DC.B  %00000000, %01100100, %11000100, %00011100
  DC.B  %01101010, %10101010, %11011001, %11110110
BulletHiY
  DC.B  %00000010, %00000001, %00000001, %00000001
  DC.B  %00000001, %00000001, %00000000, %00000000
  DC.B  %00000000, %11111111, %11111111, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111111, %11111111
  DC.B  %00000000, %00000000, %00000000, %00000001
  DC.B  %00000001, %00000001, %00000001, %00000001

  ALIGN 256

  ; Sound Effects (Lowest Priority First)
SoundData
  ; Switch
  DC.B  $00                                                               ; 0
  DC.B  %10000100, %01100011, %10000100, %01100101, %01000110, %01100101  ; 6
  DC.B  %01000111, %00100110, %01000111, %00101000                        ; 10
  DC.B  $0F, $FF                                                          ; 12
  
  ; Ring Bounce
  DC.B  $00                                                               ; 13
  DC.B  %11101010, %00000001, %11101010, %00000001, %10101010             ; 18
  DC.B  $03, $FF                                                          ; 20

  ; Bullet 
  DC.B  $00                                                               ; 21
  DC.B  %00101110, %01001000, %10000011, %00000001, %01101101, %10000111  ; 27
  DC.B  %10100010, %00000001, %10001100, %10100110, %11100001             ; 32
  DC.B  $01, $FF                                                          ; 34

  ; Sector Explosion
  DC.B  $00                                                               ; 35
  DC.B  %01001101, %01001101, %01101100, %01101100, %10001100, %10101011  ; 41
  DC.B  %11001010, %11101001, %11000011, %11100101, %11000001, %11000010  ; 47
  DC.B  %11100001                                                         ; 48
  DC.B  $02, $FF                                                          ; 50

  ; Mine Explosion
  DC.B  $00                                                               ; 51
  DC.B  %00011111, %01001110, %00011111, %01101101, %00011111, %10001110  ; 57
  DC.B  %00111111, %10101011, %00111111, %11001010, %00111111, %11011001  ; 63
  DC.B  %00111111, %11111001, %00111111, %11101000                        ; 67
  DC.B  $04, $FF                                                          ; 69

  ; Cannon
  DC.B  $00                                                               ; 70
  DC.B  %00101100, %01001011, %00101100, %00101011, %01001100, %01001011  ; 76
  DC.B  %01101100, %01001010, %01001011, %01101010, %01001011, %01101010  ; 82
  DC.B  %10001010, %01101011, %10001010, %10001011, %10001010, %10001010  ; 88
  DC.B  %10001011, %10101011, %10001010, %10101010, %10001100, %10101011  ; 94
  DC.B  %10101010, %10101011, %10101010, %10101011, %11001011, %10101010  ; 100
  DC.B  %11001010, %10101011, %11001011, %11001010, %10101010, %11001100  ; 106
  DC.B  %11001010, %11101100, %11001011, %11001010, %11101010, %11101011  ; 112
  DC.B  %11001011, %11101010, %11101011, %11001010, %11101001, %11001001  ; 118
  DC.B  %11101001, %11001000, %11101000, %11100100, %11100110, %11101000  ; 124
  DC.B  $08, $FF                                                          ; 126

  ; Ship Explosion
  DC.B  $00                                                               ; 127
  DC.B  %00101111, %00000001, %00101111, %00110010, %00110000, %00101111  ; 133
  DC.B  %00000001, %00101111, %01011001, %01011111, %00101100, %00111111  ; 139
  DC.B  %01001100, %00101011, %01001100, %01001010, %01101100, %01001011  ; 145
  DC.B  %01101010, %01101010, %01100111, %10001010, %10101000, %01101010  ; 151
  DC.B  %10001000, %11001001, %10001000, %10100111, %10101000, %10000110  ; 157
  DC.B  %01100110, %10100101, %01000100, %11100011, %11000100, %11000011  ; 163
  DC.B  %11100010, %11100010, %11100001, %11100011                        ; 167
  DC.B  $02, $FF                                                          ; 169

  ; Ring Explosion
  DC.B  $00                                                               ; 170
  DC.B  %00111110, %00000001, %00111100, %00000001, %00111110, %00111100  ; 176
  DC.B  %00111110, %01011101, %00111101, %00111111, %00111011, %00111011  ; 182
  DC.B  %00111101, %01011001, %00111111, %01011001, %00111110, %01011111  ; 188
  DC.B  %01011100, %00111111, %01011011, %00111101, %01111011, %00111011  ; 194
  DC.B  %01011011, %00111101, %01111001, %00111111, %01011001, %00111110  ; 200
  DC.B  %00111111, %00111100, %01011101, %00111011, %01111100, %00111011  ; 206
  DC.B  %01011010, %01011011, %00111010, %01111001, %01111010, %10011001  ; 212
  DC.B  %01111010, %01011101, %01111000, %01110111, %01011000, %01110110  ; 218
  DC.B  %10010111, %01011001, %11011101, %10010111, %10110110, %01111000  ; 224
  DC.B  %10011000, %01110101, %10011001, %10110110, %10110111, %11010100  ; 230
  DC.B  %10010110, %10110111, %11110110, %10110101, %11010110, %10111101  ; 236
  DC.B  %10110100, %11010110, %10110010, %11010100, %10110110, %11010101  ; 242
  DC.B  %11101111, %10111010, %11010101, %11101111, %11110110, %11010101  ; 248
  DC.B  %11110100, %11110110, %11010100, %11110100                        ; 252
  DC.B  $08, $FF                                                          ; 254
EndSoundData
  if (>SoundData != >EndSoundData)
    echo "WARNING: SoundData crosses a page boundary!"
  endif

  ; DC.B  "RAND"

  ALIGN 256

  ; Star Data
TopStars_A
  DC.B  %00000000
  DC.B  %00110010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %11100000
  DC.B  %10000010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01000010
TopStars_B
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000010
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00110000
  DC.B  %01110010
  DC.B  %00010000
  DC.B  %01110000
BottomStars_A
  DC.B  %00000000
  DC.B  %00000010
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000010
  DC.B  %00100000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01010000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
BottomStars_B
  DC.B  %00000010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %00000000
  DC.B  %10110010
  DC.B  %00100000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
EndStars
  if (>TopStars_A != >EndStars)
    echo "WARNING: Stars Data crosses a page boundary!"
  endif

   ; Binary To BCD Table (100 Entries)
BCDTable:
  DC.B  %00000000, %00000001, %00000010, %00000011, %00000100
  DC.B  %00000101, %00000110, %00000111, %00001000, %00001001
  DC.B  %00010000, %00010001, %00010010, %00010011, %00010100
  DC.B  %00010101, %00010110, %00010111, %00011000, %00011001
  DC.B  %00100000, %00100001, %00100010, %00100011, %00100100
  DC.B  %00100101, %00100110, %00100111, %00101000, %00101001
  DC.B  %00110000, %00110001, %00110010, %00110011, %00110100
  DC.B  %00110101, %00110110, %00110111, %00111000, %00111001
  DC.B  %01000000, %01000001, %01000010, %01000011, %01000100
  DC.B  %01000101, %01000110, %01000111, %01001000, %01001001
  DC.B  %01010000, %01010001, %01010010, %01010011, %01010100
  DC.B  %01010101, %01010110, %01010111, %01011000, %01011001
  DC.B  %01100000, %01100001, %01100010, %01100011, %01100100
  DC.B  %01100101, %01100110, %01100111, %01101000, %01101001
  DC.B  %01110000, %01110001, %01110010, %01110011, %01110100
  DC.B  %01110101, %01110110, %01110111, %01111000, %01111001
  DC.B  %10000000, %10000001, %10000010, %10000011, %10000100
  DC.B  %10000101, %10000110, %10000111, %10001000, %10001001
  DC.B  %10010000, %10010001, %10010010, %10010011, %10010100
  DC.B  %10010101, %10010110, %10010111, %10011000, %10011001
EndBCDTable
  if (>BCDTable != >EndBCDTable)
    echo "WARNING: BCD Table Cross Page Boundary!"
  endif

ColTable0
StarCols0
  DC.B  $84, $B4, $04       ; NTSC/PAL60/B&W
TextCols0
  DC.B  $8C, $BC, $0C       ; NTSC/PAL60/B&W
RingCols0
  DC.B  $4A, $FC, $3C, $6C  ; NTSC
  DC.B  $68, $2C, $4C, $AC, ; PAL60
  DC.B  $0C, $0C, $0C, $0C  ; B&W
RingCols1
  DC.B  $2C, $CC, $5C, $8C  ; NTSC
  DC.B  $4A, $5C, $8C, $BC  ; PAL60
  DC.B  $0C, $0C, $0C, $0C  ; B&W
RingCols2
  DC.B  $1C, $AC, $7C, $CC  ; NTSC
  DC.B  $2C, $9C, $DC, $5C  ; PAL60
  DC.B  $0C, $0C, $0C, $0C  ; B&W
EndColTable0
  if (>ColTable0 != >EndColTable0)
    echo "WARNING: ColTable0 crosses a page boundary!"
  endif

  echo "----",($FFF4 - *) , "bytes left (BANK0 - GAME LOGIC)"

  ORG     $0FF4
  RORG    $FFF4
  DC.B    "BANK0", 0, 0, 0
  DC.W    Init0, Init0


