; -----------------------------------------------------------------------------
; Star Castle 2600 - Copyright (C) 2012 Chris Walton (cd-w) <cwalton@gmail.com>
; Bank 1 - Mines & Collision Detection
; -----------------------------------------------------------------------------

  ; Bank 1
  SEG     BANK1
  ORG     $1000
  RORG    $F000
  
  ; Space for RAM+
  DS.B    512,35

Init1
  nop     $FFFB             ; Switch to Bank 6
EnterKernel
  nop     $FFF7             ; Switch to Bank 2
  jmp     MoveMines
  nop
  nop
  nop     ;
LeaveStartup
  nop     $FFFB             ; Switch To Bank 6
  jmp     EnterStartup

; -----------------------------------------------------------------------------
; Mine Movements
; -----------------------------------------------------------------------------

MoveMines
  ; Mine Table Offset (Increase Every 2 Waves)
  lda WAVE                  ; [0] + 3
  and #%00001110            ; [3] + 2
  asl                       ; [5] + 2
  asl                       ; [7] + 2
  asl                       ; [9] + 2
  sta TEMP                  ; [11] + 3
  
  ; Iterate Mines
  ldy #2
MoveMineLoop
  ; Check For Homing Mine
  lda MINEDIR_R,Y           ; [0] + 4
  cmp #%11100000            ; [4] + 2
  bcc RingMine              ; [6] + 2/3
  ; Store Direction
  and #%00011111            ; [8] + 2
  sta TEMP2                 ; [10] + 3
  tax                       ; [13] + 2
  ; Calculate X Table Index
  clc                       ; [15] + 2
  lda MineXIndex,X          ; [17] + 4
  bmi SkipMineX             ; [21] + 2/3
  adc TEMP                  ; [23] + 3
  tax                       ; [26] + 2
  ; Move Mine X
  ; clc
  lda MINEX_LO_R,Y          ; [28] + 4
  adc MineLoX,X             ; [32] + 4
  sta MINEX_LO_W,Y          ; [36] + 4
  lda MINEX_HI_R,Y          ; [40] + 4
  adc MineHiX,X             ; [44] + 4
  cmp #160-2                ; [48] + 2
  bcc StoreMineX            ; [50] + 2/3
  ; Wrap Around X Edges
  lda MineHiX,X             ; [52] + 4
  bmi MineWrapX             ; [56] + 2/3
  lda #0                    ; [58] + 2
  beq StoreMineX            ; [60] + 3
MineWrapX
  lda #160-2                ; [59] + 2
StoreMineX
  sta MINEX_HI_W,Y          ; [63] + 4
SkipMineX
  ; Calculate X Table Index
  clc                       ; [67] + 2
  ldx TEMP2                 ; [69] + 3
  lda MineYIndex,X          ; [72] + 4
  bmi NextMine              ; [76] + 2/3
  adc TEMP                  ; [78] + 3
  tax                       ; [81] + 2
  ; Move Mine Y
  ; clc
  lda MINEY_LO_R,Y          ; [83] + 4
  adc MineLoY,X             ; [87] + 4
  sta MINEY_LO_W,Y          ; [91] + 4
  lda MINEY_HI_R,Y          ; [95] + 4
  adc MineHiY,X             ; [99] + 4
  cmp #YSTART               ; [103] + 2
  bcc StoreMineY            ; [105] + 2/3
  ; Wrap Around Y Edges
  lda MineHiY,X             ; [107] + 3
  bmi MineWrapY             ; [110] + 2/3
  lda #0                    ; [112] + 2
  beq StoreMineY            ; [114] + 3
MineWrapY
  lda #YSTART               ; [113] + 2
StoreMineY
  sta MINEY_HI_W,Y          ; [117] + 4
NextMine
  dey                       ; [121] + 2
  bpl MoveMineLoop          ; [123] + 2/3
  bmi EndMineMove           ; WORST = 126*3 = 5 SCANLINES

; -----------------------------------------------------------------------------

  ; Mine Ring Movements
RingMine
  ; Store Sector
  and #%00011111            ; [9] + 2
  tax                       ; [11] + 2
  lda MINEDIR_R,Y           ; [13] + 4
  bmi NextMine              ; [17] + 2/3    Skip Exploding Mine
  and #%11100000            ; [19] + 2
  beq NextMine              ; [21] + 2/3    Skip Unspawned Mine
  cmp #%01000000            ; [23] + 2
  beq MineRing1             ; [25] + 2/3
  bcs MineRing2             ; [27] + 2/3

MineRing0
  ; clc
  lda OFFSET0,X             ; [29] + 4
  adc #4                    ; [33] + 2
  cmp #108                  ; [35] + 2
  bcc MineWrap0             ; [37] + 2/3
  sbc #108                  ; [39] + 2
MineWrap0
  tax                       ; [41] + 2
  lda Ring0XPosition,X      ; [43] + 4
  sta MINEX_HI_W,Y          ; [47] + 4
  lda Ring0YPosition,X      ; [51] + 4
  sta MINEY_HI_W,Y          ; [55] + 4
  jmp NextMine              ; [59] + 3
  
MineRing1
  clc                       ; [28] + 2
  lda OFFSET1,X             ; [30] + 4
  adc #3                    ; [34] + 2
  cmp #84                   ; [36] + 2
  bcc MineWrap1             ; [38] + 2/3
  sbc #84                   ; [40] + 2
MineWrap1
  tax                       ; [42] + 2
  lda Ring1XPosition,X      ; [44] + 4
  sta MINEX_HI_W,Y          ; [48] + 4
  lda Ring1YPosition,X      ; [52] + 4
  sta MINEY_HI_W,Y          ; [56] + 4
  jmp NextMine              ; [60] + 3

MineRing2
  clc                       ; [30] + 2
  lda OFFSET2,X             ; [32] + 4
  adc #2                    ; [36] + 2
  cmp #60                   ; [38] + 2
  bcc MineWrap2             ; [40] + 2/3
  sbc #60                   ; [42] + 2
MineWrap2
  tax                       ; [44] + 2
  lda Ring2XPosition,X      ; [46] + 4
  sta MINEX_HI_W,Y          ; [50] + 4
  lda Ring2YPosition,X      ; [54] + 4
  sta MINEY_HI_W,Y          ; [58] + 4
  jmp NextMine              ; [62] + 3
EndMineMove

; -----------------------------------------------------------------------------
; Bullet/Mine Collisions
; -----------------------------------------------------------------------------

  ldx #1                    ; [0] + 2       = 2
BulletMineOuterLoop
  lda BULLETDIR,X           ; [0] + 4
  bmi NextBulletColl        ; [4] + 2/3
  ldy #2                    ; [6] + 2       = 8
BulletMineInnerLoop
  ; Only Check Homing Mines
  lda MINEDIR_R,Y           ; [0] + 4
  cmp #%11100000            ; [4] + 2
  bcc NextBulletMineColl    ; [6] + 2/3
  ; Check Region Overlap
  ; clc
  lda MINEX_HI_R,Y          ; [8] + 4
  sbc BULLETX_HI,X          ; [12] + 4
  sbc #2                    ; [16] + 2
  adc #4                    ; [18] + 2
  bcc NextBulletMineColl    ; [20] + 2/3
  ; clc
  lda MINEY_HI_R,Y          ; [22] + 4
  sbc BULLETY_HI,X          ; [26] + 4
  sbc #2                    ; [30] + 2
  adc #4                    ; [32] + 2
  bcs BulletMineCollision   ; [34] + 2/3
NextBulletMineColl
  dey                       ; [36] + 2
  bpl BulletMineInnerLoop   ; [38] + 2/3    INNER = 41*2 + 40 = 122
NextBulletColl
  dex                       ; [130] + 2
  bpl BulletMineOuterLoop   ; [132] + 2/3
  bmi EndBulletMineColl     ; [269] + 3     OUTER = 135 + 134 = 269
BulletMineCollision
  ; Mine Explode
  lda #%10000000
  sta MINEDIR_W,Y
  lda #2
  sta MINECOUNT_W,Y
  lda #$0E
  sta MINECOL,Y
  ; Clear Bullet
  lda #$FF
  sta BULLETDIR,X
  ; Play Sound
  lda #69
  cmp SFX
  bcc EndBulletMineColl
  sta SFX
EndBulletMineColl           ; WORST = 274 (3.5 SCANLINES)

; -----------------------------------------------------------------------------
; Ship/Projectile Collision
; -----------------------------------------------------------------------------

  ; Skip Projectile & Ring Collisions If Dead
  ldx SHIPS                 ; [0] + 3
  bpl ProjectileColl        ; [3] + 2/3
  ; Skip Bullet Checks If Exploding
  lda MODE
  cmp #%11100000
  bcc SkipProjectileChecks
  jmp EnterKernel
SkipProjectileChecks
  jmp EndProjectile

ProjectileColl
  ; Projectile Visible
  lda PROJDIR               ; [6] + 3
  bmi NoProjCollision       ; [9] + 2/3
  ; Projectile Start Position
  clc                       ; [11] + 2
  lda PROJX_HI              ; [13] + 3
  adc #2                    ; [16] + 2
  sta TEMPX                 ; [18] + 3
  ; clc
  lda PROJY_HI              ; [21] + 3
  adc #2                    ; [24] + 2
  sta TEMPY                 ; [26] + 3
  ; Check Overlap
  sec                       ; [29] + 2
  lda SHIPX_HI              ; [31] + 3
  sbc #3                    ; [34] + 2
  sbc TEMPX                 ; [36] + 3
  sbc #5                    ; [39] + 2
  adc #6+6-1                ; [41] + 2
  bcc NoProjCollision       ; [43] + 2/3
  ; sec
  lda SHIPY_HI              ; [45] + 3
  sbc #3                    ; [48] + 2
  sbc TEMPY                 ; [50] + 3
  sbc #5                    ; [53] + 2
  adc #6+6-1                ; [55] + 2
  bcc NoProjCollision       ; [57] + 2/3
  ; Explode Ship
  txa                       ; [59] + 2
  ora #%10000000            ; [61] + 2
  sta SHIPS                 ; [63] + 3
  lda #10*4                 ; [66] + 2
  sta SHIPDIR_HI            ; [68] + 3
  ; Play Sound
  lda #169                  ; [71] + 2
  cmp SFX
  bcc SkipShipExplodeSound
  sta SFX                   ; [73] + 3
SkipShipExplodeSound
  ; Skip Ring & Gun Calculations
  jmp EndProjectile         ; [76] + 3
NoProjCollision             ; WORST = 79 CYCLES

; -----------------------------------------------------------------------------
; Ship/Ring Collision
; -----------------------------------------------------------------------------

  ; Ship Angle From Centre
  sec                       ; [0] + 2
  lda SHIPX_HI              ; [2] + 3
  sta TEMPBX                ; [5] + 3
  sbc #MIDDLEX              ; [8] + 2
  sta TEMPX                 ; [10] + 3
  sec                       ; [13] + 2
  lda SHIPY_HI              ; [15] + 3
  sta TEMPBY                ; [18] + 3
  sbc #MIDDLEY              ; [21] + 2
  sta TEMPY                 ; [23] + 3
  jsr VectorAngle           ; [26] + 6 + 174
  sta SHIPANGLE             ; [206] + 3

  ; Disable Ring Bounce if Right Difficulty Set To A
  bit SWCHB                 ; [209] + 3
  bmi EndShipCheck          ; [212] + 2/3
  
  ; Ship Distance from Centre
  jsr CentreDistance        ; [214] + 6 + 59 = 279

  ; Compare With Ring Radius
  ; lda DISTANCE_HI
  cmp #%00000010            ; [0] + 2      24^2 (HI)
  bcc ShipInsideRing        ; [2] + 2/3
  bne EndShipCheck          ; [4] + 2/3
  lda DISTANCE_LO           ; [6] + 3
  cmp #%01000000            ; [9] + 2      24^2 (LO)
  bcs EndShipCheck          ; [11] + 2/3
ShipInsideRing

  ; Negate Velocity
  sec                       ; [13] + 2
  lda #0                    ; [15] + 2
  sbc SHIPVX_LO             ; [17] + 3
  sta SHIPVX_LO             ; [20] + 3
  lda #0                    ; [23] + 2
  sbc SHIPVX_HI             ; [25] + 3
  sta SHIPVX_HI             ; [28] + 3
  sec                       ; [31] + 2
  lda #0                    ; [33] + 2
  sbc SHIPVY_LO             ; [35] + 3
  sta SHIPVY_LO             ; [38] + 3
  lda #0                    ; [41] + 2
  sbc SHIPVY_HI             ; [43] + 3
  sta SHIPVY_HI             ; [46] + 3

  ; Divide Velocity By 2
  ; lda SHIPVY_HI
  cmp #$80                  ; [49] + 2
  ror SHIPVY_HI             ; [51] + 5
  ror SHIPVY_LO             ; [56] + 5
  lda SHIPVX_HI             ; [61] + 3
  cmp #$80                  ; [64] + 2
  ror SHIPVX_HI             ; [66] + 5
  ror SHIPVX_LO             ; [71] + 5

  ; Update Ship Direction
  lda SHIPANGLE             ; [76] + 3
  sta SHIPDIR_HI            ; [79] + 3

  ; Update Ship Position (bounce out of ring)
  tax                       ; [82] + 2
  clc                       ; [84] + 2
  lda SHIPX_LO              ; [86] + 3
  adc KickLoX,X             ; [89] + 4
  sta SHIPX_LO              ; [93] + 3
  lda SHIPX_HI              ; [96] + 3
  adc KickHiX,X             ; [99] + 4
  sta SHIPX_HI              ; [103] + 3
  clc                       ; [106] + 2
  lda SHIPY_LO              ; [108] + 3
  adc KickLoY,X             ; [111] + 4
  sta SHIPY_LO              ; [115] + 3
  lda SHIPY_HI              ; [118] + 3
  adc KickHiY,X             ; [111] + 4
  sta SHIPY_HI              ; [115] + 3

  ; Play Sound
  lda #20                   ; [118] + 2
  cmp SFX                   ; [120] + 3
  bcc EndShipCheck          ; [123] + 2/3
  sta SFX                   ; [125] + 3
EndShipCheck                ; WORST = 274 + 128 = 402 CYCLES (5 SCANLINES)

; -----------------------------------------------------------------------------
; Gun Rotation & Firing
; -----------------------------------------------------------------------------

  ; Skip Gun & Bullet Collisions if Exploding
  lda MODE                  ; [0] + 3
  cmp #%11100000            ; [3] + 2
  bcc RotateGun             ; [5] + 2/3
  jmp EnterKernel           ; [7] + 3
  
  ; Gun Rotation Speed
RotateGun
  lda WAVE                  ; [8] + 3
  and #%00011100            ; [11] + 2
  lsr                       ; [13] + 2
  lsr                       ; [15] + 2
  tay                       ; [17] + 2
  ; clc
  lda GUNCOUNT              ; [19] + 3
  adc GunSpeeds,Y           ; [22] + 4
  sta GUNCOUNT              ; [26] + 3
  bcc EndGunRotate          ; [29] + 2/3 

  ; Rotate Gun To Face Ship
  ; sec                     ; [31] + 2
  lda GUNDIR                ; [33] + 3
  sbc SHIPANGLE             ; [36] + 3
  beq EndGunRotate          ; [39] + 2/3
  bmi GunDistance2          ; [41] + 2/3
GunDistance1
  sta TEMP                  ; [43] + 3
  sec                       ; [46] + 2
  lda #31                   ; [48] + 2
  sbc GUNDIR                ; [50] + 3
  ; clc
  adc SHIPANGLE             ; [53] + 3
  tax                       ; [56] + 2
  jmp GunCheck              ; [58] + 3
GunDistance2
  ; sec
  lda SHIPANGLE             ; [44] + 3
  sbc GUNDIR                ; [47] + 3
  tax                       ; [50] + 2
  sec                       ; [52] + 2
  lda #31                   ; [54] + 2
  sbc SHIPANGLE             ; [56] + 3
  ; clc 
  adc GUNDIR                ; [59] + 3
  sta TEMP                  ; [62] + 3
GunCheck
  cpx TEMP                  ; [65] + 3
  bcs GunAntiClockwise      ; [68] + 2/3
GunClockwise
  ; clc
  lda GUNDIR                ; [70] + 3
  adc #1                    ; [73] + 2
  jmp StoreGunRotate        ; [75] + 3
GunAntiClockwise
  ; sec
  lda GUNDIR                ; [71] + 3
  sbc #1                    ; [74] + 2
StoreGunRotate
  and #%00011111            ; [78] + 2
  sta GUNDIR                ; [80] + 3
EndGunRotate                ; WORST = 83 CYCLES

  ; Decrement Mine Counter
  lda GUNCOUNT2             ; [0] + 3
  beq FireProjectile        ; [3] + 2/3
  sec
  sbc #1
  sta GUNCOUNT2
  jmp EndProjectile
FireProjectile
  ; Set Random Time Until Next Fire
  lda WAVE                  ; [6] + 3
  and #%00001100            ; [9] + 2
  lsr                       ; [11] + 2
  lsr                       ; [13] + 2
  tax                       ; [15] + 2
  lda RANDOM                ; [17] + 3
  and WaveMask,X            ; [20] + 4
  sta GUNCOUNT2             ; [24] + 3
  ; Skip If Projectile Already Fired
  lda PROJDIR               ; [27] + 3
  bpl EndProjectile         ; [30] + 2/3
  ; Check Hole In Centre Ring
  lda BREACH                ; [32] + 3
  beq EndProjectile         ; [35] + 2/3
  ; Skip If Gun Facing Wrong Way
  lda GUNDIR                ; [37] + 3
  cmp SHIPANGLE             ; [40] + 3
  bne EndProjectile         ; [43] + 2/3
  ; Fire Projectile
  sta PROJDIR               ; [45] + 3
  lda #MIDDLEX-4            ; [48] + 2
  sta PROJX_HI              ; [50] + 3
  lda #MIDDLEY-4            ; [53] + 2
  sta PROJY_HI              ; [55] + 3
  lda #0                    ; [58] + 2
  sta PROJX_LO              ; [60] + 3
  sta PROJY_LO              ; [63] + 3
  lda #15                   ; [66] + 2
  sta PROJFRAME             ; [68] + 3
  ; Firing Sound
  lda #126                  ; [71] + 2
  cmp SFX                   ; [73] + 3
  bcc EndProjectile         ; [76] + 2/3
  sta SFX                   ; [78] + 3
EndProjectile               ; WORST = 81 CYCLES

; -----------------------------------------------------------------------------
; Bullet/Ring/Gun Collision
; -----------------------------------------------------------------------------

  ; Skip If No Bullet
  ldx BPOS
  lda BULLETDIR,X
  bmi Bullet1Check

  ; Store Bullet Coordinates
  stx TEMPBPOS
  lda BULLETY_HI,X
  sta TEMPBY
  lda BULLETX_HI,X
  sta TEMPBX

  ; Bullet0 Distance from Centre
  jsr CentreDistance

  ; Compare With Ring Radii
  ; lda DISTANCE_HI
  ldx DISTANCE_LO 
  cmp #%00000001            ; 22^2 (HI)
  bcc Bullet0InsideRing0    ;
  bne Bullet1Check          ;
  cpx #%11100100            ; 22^2 (LO)
  bcs Bullet1Check          ;
  jmp BulletCollideRing0    ;
Bullet0InsideRing0
  cpx #%10010000            ; 12^2 (LO)
  bcs BulletCollideRing1    ;
  cpx #%00110001            ; 7^2 (LO)
  bcs BulletCollideRing2    ;
  jmp CollideGun            ;

; -----------------------------------------------------------------------------

Bullet1Check
  ldx BPOS
  dex
  bpl CheckBullet1
  ldx #1

CheckBullet1
  lda BULLETDIR,X
  bmi EndBulletCheck
  
  ; Store Bullet Coordinates
  stx TEMPBPOS
  lda BULLETY_HI,X
  sta TEMPBY
  lda BULLETX_HI,X
  sta TEMPBX

  ; Bullet1 Distance from Centre
  jsr CentreDistance

  ; Compare With Ring Radii
  ; lda DISTANCE_HI
  ldx DISTANCE_LO           ;
  cmp #%00000001            ; 22^2 (HI)
  bcc Bullet1InsideRing0    ;
  bne EndBulletCheck        ;
  cpx #%11100100            ; 22^2 (LO)
  bcs EndBulletCheck        ;
  jmp BulletCollideRing0    ;
Bullet1InsideRing0
  cpx #%10010000            ; 12^2 (LO)
  bcs BulletCollideRing1    ;
  cpx #%00110001            ; 7^2 (LO)
  bcs BulletCollideRing2    ;
  jmp CollideGun            ;

EndBulletCheck
  jmp MineRingColl

; -----------------------------------------------------------------------------

  ; Bullet Collision With Ring 2
BulletCollideRing2
  jsr CollideRing2
  bpl BulletCollide2
  jmp EnterKernel
  
BulletCollide2
  ldy CURRSECTOR            ; [0] + 3
  ; Check Difficulty Switch
  bit SWCHB                 ; [3] + 3
  bvc CheckDots2            ; [6] + 2/3
SkipDots2
  lda #$FF                  ; [8] + 2
  bne StoreMap2             ; [10] + 3
CheckDots2
  lda SECTORMAP2_R,Y        ; [9] + 4
  bne SkipDots2             ; [13] + 2/3
  lda OFFSET2,Y             ; [15] + 4
  lsr                       ; [19] + 2
  lda #%10101010            ; [21] + 2
  bcc StoreMap2             ; [23] + 2/3
  lsr                       ; [25] + 2
StoreMap2
  sta SECTORMAP2_W,Y        ; [27] + 4
  tya                       ; [31] + 2
  asl                       ; [33] + 2
  asl                       ; [35] + 2
  ora #%00000010            ; [37] + 2
  sta BCLEAR                ; [39] + 3
  jmp ClearBullet           ; [42] + 3

; -----------------------------------------------------------------------------

  ; Bullet Collision With Ring 1
BulletCollideRing1
  jsr CollideRing1
  bpl BulletCollide1
  jmp EnterKernel

BulletCollide1
  ldy CURRSECTOR            ; [0] + 3
  ; Check Difficulty Switch
  bit SWCHB                 ; [3] + 3
  bvc CheckDots1            ; [6] + 2/3
SkipDots1
  lda #$FF                  ; [8] + 2
  bne StoreMap1             ; [10] + 3
CheckDots1
  lda SECTORMAP1_R,Y        ; [9] + 4
  bne SkipDots1             ; [13] + 2/3
  lda OFFSET1,Y             ; [15] + 4
  lsr                       ; [19] + 2
  lda #%10101010            ; [21] + 2
  bcc StoreMap1             ; [23] + 2/3
  lsr                       ; [25] + 2
StoreMap1
  sta SECTORMAP1_W,Y        ; [27] + 4
  tya                       ; [31] + 2
  asl                       ; [33] + 2
  asl                       ; [35] + 2
  ora #%00000001            ; [37] + 2
  sta BCLEAR                ; [39] + 3
  jmp ClearBullet           ; [42] + 3

; -----------------------------------------------------------------------------

  ; Bullet Collision with Ring 0
BulletCollideRing0
  jsr CollideRing0
  bpl BulletCollide0
  jmp EnterKernel

BulletCollide0
  ldy CURRSECTOR            ; [0] + 3
  ; Check Difficulty Switch
  bit SWCHB                 ; [3] + 3
  bvc CheckDots0            ; [6] + 2/3
SkipDots0
  lda #$FF                  ; [8] + 2
  bne StoreMap0             ; [10] + 3
CheckDots0
  lda SECTORMAP0_R,Y        ; [9] + 4
  bne SkipDots0             ; [13] + 2/3
  lda OFFSET0,Y             ; [15] + 4
  lsr                       ; [19] + 2
  lda #%10101010            ; [21] + 2
  bcc StoreMap0             ; [23] + 2/3
  lsr                       ; [25] + 2
StoreMap0
  sta SECTORMAP0_W,Y        ; [27] + 4
  tya                       ; [31] + 2
  asl                       ; [33] + 2
  asl                       ; [35] + 2
  sta BCLEAR                ; [37] + 3
  jmp ClearBullet           ; [40] + 3

; -----------------------------------------------------------------------------

EndGunCollision
  jmp MineLogic

  ; Bullet Collision with Gun
CollideGun
  clc                       ; [0] + 2
  lda TEMPBX                ; [2] + 3
  sbc #MIDDLEX-3            ; [5] + 2
  sbc #7                    ; [7] + 2
  adc #8                    ; [9] + 2
  bcc EndGunCollision       ; [11] + 2/3
  clc                       ; [13] + 2
  lda TEMPBY                ; [15] + 3
  sbc #MIDDLEY-3            ; [18] + 2
  sbc #7                    ; [20] + 2
  adc #8                    ; [22] + 2
  bcc EndGunCollision       ; [24] + 2/3
  ; Set Explosion Mode
  lda #%11100000            ; [26] + 2
  sta MODE                  ; [28] + 3
  lda #EXPLODEFRAMES        ; [31] + 2
  sta EXPLODE               ; [33] + 3
  ; Explosion Sound
  lda #254
  sta SFX
  ; Increment Wave
  lax WAVE                  ; [36] + 3
  and #%11000000            ; [39] + 2
  sta TEMP                  ; [41] + 3
  txa                       ; [44] + 2
  clc                       ; [46] + 2
  ; and #%00111111
  adc #1                    ; [48] + 2
  and #%00111111            ; [50] + 2
  ora TEMP                  ; [52] + 3
  sta WAVE                  ; [55] + 3
  ; New Ship
  lax SHIPS                 ; [58] + 3
  and #%11000000            ; [61] + 2
  sta TEMP                  ; [63] + 3
  ; Max 60 Ships
  txa                       ; [66] + 2
  and #%00111111            ; [68] + 2
  cmp #%00111100            ; [70] + 2
  bcs SkipNewShip           ; [72] + 2/3
  ; clc
  adc #1                    ; [74] + 2
  ora TEMP                  ; [76] + 3
  sta SHIPS                 ; [79] + 3
SkipNewShip
  ; Increment Score
  sed                       ; [82] + 2
  clc                       ; [84] + 2
  lda SCORE+0               ; [86] + 3
  adc #%01000000            ; [89] + 2    40
  sta SCORE+0               ; [91] + 3
  lda SCORE+1               ; [94] + 3
  adc #%00010100            ; [97] + 2    14
  sta SCORE+1               ; [99] + 3
  lda SCORE+2               ; [102] + 3
  adc #0                    ; [105] + 2
  sta SCORE+2               ; [107] + 3
  cld                       ; [110] + 2
ClearBullet
  lda #$FF
  ldx TEMPBPOS
  sta BULLETDIR,X
  ; Play Sound
  lda #50
  cmp SFX
  bcc SkipBulletSound
  sta SFX
SkipBulletSound
  ; Skip Mine Logic
  jmp EnterKernel

; -----------------------------------------------------------------------------
; Mine/Ring Collision
; -----------------------------------------------------------------------------

MineRingColl
  ; Only Check Homing Mines
  ldy MPHASE
  lda MINEDIR_R,Y
  cmp #%11100000
  bcc EndMineCheck

  ; Mine Coordinates
  lda MINEY_HI_R,Y
  sta TEMPBY
  lda MINEX_HI_R,Y
  sta TEMPBX

  ; Mine Distance from Centre
  jsr CentreDistance
  ldy MPHASE

  ; Compare With Ring Radii
  ; lda DISTANCE_HI
  ldx DISTANCE_LO           ; 
  cmp #%00000001            ; 22^2 (HI)
  bcc MineInsideRing0       ; 
  bne MineOutsideRings      ; 
  cpx #%11100100            ; 22^2 (LO)
  bcs MineOutsideRings      ;
  jmp MineCollideRing0      ;
MineInsideRing0
  cpx #%10010000            ; 12^2 (LO)
  bcs MineCollideRing1      ; 
  cpx #%00110001            ; 7^2 (LO)
  bcs MineCollideRing2      ;
  ; Mine At Centre
  lda RINGCOL2
  sta MINECOL,Y
  jmp MineLogic
MineOutsideRings
  ldx COLMODE
  lda MineCols,X
  sta MINECOL,Y
EndMineCheck
  jmp MineLogic

; -----------------------------------------------------------------------------

MineCollideRing0
  lda RINGCOL0
  sta MINECOL,Y
  jsr CollideRing0
  bmi EndMineColl
MineCollide0
  ; Attach Mine To Ring 0
  ora #%00100000
  jmp SetMinePhase

MineCollideRing1
  lda RINGCOL1
  sta MINECOL,Y
  jsr CollideRing1
  bmi EndMineColl
MineCollide1
  ; Attach Mine To Ring 1
  ora #%01000000
  jmp SetMinePhase

MineCollideRing2
  lda RINGCOL2
  sta MINECOL,Y
  jsr CollideRing2
  bmi EndMineColl
MineCollide2
  ; Attach Mine To Ring 2
  ora #%01100000
SetMinePhase
  ldy MPHASE
  sta MINEDIR_W,Y
  ; Set Random Time Until Next Jump
  lda WAVE
  and #%00001100
  lsr
  lsr
  tax
  lda RANDOM
  and WaveMask,X
  sta MINECOUNT_W,Y
EndMineColl
  ; Skip Mine Logic
  jmp EndMineLogic

; -----------------------------------------------------------------------------
; Mine/Projectile Logic
; -----------------------------------------------------------------------------

MineLogic
  ; Extract Direction/Sector
  ldy MPHASE
  lax MINEDIR_R,Y
  and #%00011111
  sta CURRSECTOR
  
  ; Decrement Mine Counter
  lda MINECOUNT_R,Y
  beq SkipMineCounter
  sec
  sbc #1
  sta MINECOUNT_W,Y
SkipMineCounter

  ; Check For Phase 0 to 3
  txa
  bpl NonHoming

  ; Expire Mines
  lda MINECOUNT_R,Y
  beq ExpireMine

  ; Check For Phase 4
  cpx #%11100000
  bcc EndMineLogicA
  jmp MinePhase4

ExpireMine
  ; Expire Mine
  lda #0
  sta MINEDIR_W,Y
  sta MINECOL,Y
  ; Spawn Delay
  lda WAVE
  and #%00001100
  lsr
  lsr
  tax
  lda RANDOM
  and WaveMask,X
  sta MINECOUNT_W,Y
EndMineLogicA
  jmp EndMineLogic

NonHoming
  ; Check For Phase 0
  and #%11100000
  beq MinePhase0

  ; Mine Angle From Centre
  sec
  lda MINEX_HI_R,Y
  sbc #MIDDLEX
  sta TEMPX
  sec
  lda MINEY_HI_R,Y
  sbc #MIDDLEY
  sta TEMPY
  jsr VectorAngle
  sta MINEANGLE

  ldy MPHASE
  lda MINEDIR_R,Y
  ldx CURRSECTOR
  and #%11100000
  cmp #%01000000
  beq MinePhase2
  bcs MinePhase3
MinePhase1
  lda SECTORMAP0_R,X
  cmp #$FF
  beq KickMine
  bne MinePhase123
MinePhase2
  lda SECTORMAP1_R,X
  cmp #$FF
  beq KickMine
  bne MinePhase123
MinePhase3
  lda SECTORMAP2_R,X
  cmp #$FF
  beq KickMine
MinePhase123
  ; Change Phase When Counter Reaches Zero
  lda MINECOUNT_R,Y
  bne EndMineLogicB
KickMine
  ; Change to Phase 4
  lax MINEANGLE
  ora #%11100000
  sta MINEDIR_W,Y
  ; Kick Mine From Ring
  clc
  lda MINEX_LO_R,Y
  adc KickLoX,X
  sta MINEX_LO_W,Y
  lda MINEX_HI_R,Y
  adc KickHiX,X
  sta MINEX_HI_W,Y
  clc
  lda MINEY_LO_R,Y
  adc KickLoY,X 
  sta MINEY_LO_W,Y
  lda MINEY_HI_R,Y
  adc KickHiY,X
  sta MINEY_HI_W,Y
  ; Mine Expire Counter
  lda #255
  sta MINECOUNT_W,Y
EndMineLogicB
  jmp EndMineLogic

MinePhase0
  ; Spawn When Counter Reaches Zero
  lda MINECOUNT_R,Y
  bne EndMineLogicB
  ; Ensure Gun If Facing Correct Direction
  lda SHIPANGLE
  cmp GUNDIR
  bne EndMineLogicB
  ; Change to Phase 4
  ora #%11100000
  sta MINEDIR_W,Y
  ; Set To Centre Position
  lda #MIDDLEX
  sta MINEX_HI_W,Y
  lda #MIDDLEY
  sta MINEY_HI_W,Y
  ; Mine Expire Counter
  lda #255
  sta MINECOUNT_W,Y
  ; Mine Colour
  lda RINGCOL2
  sta MINECOL,Y
  jmp EndMineLogic

MinePhase4
  ; Calculate Ship Angle From Mine
  sec
  lda SHIPX_HI
  sbc MINEX_HI_R,Y
  sta TEMPX
  sec
  lda SHIPY_HI
  sbc MINEY_HI_R,Y
  sta TEMPY
  jsr VectorAngle
  sta MINEANGLE

  ; Rotate Mine To Face Ship
  sec
  lda CURRSECTOR
  sbc MINEANGLE
  beq EndMineLogic
  bmi MineDistance1
  sta TEMP
  sec
  lda #31
  sbc CURRSECTOR
  ; clc
  adc MINEANGLE
  tax
  jmp MineCheck1
MineDistance1
  ; sec
  lda MINEANGLE
  sbc CURRSECTOR
  tax
  sec
  lda #31
  sbc MINEANGLE
  ; clc
  adc CURRSECTOR
  sta TEMP
MineCheck1
  cpx TEMP
  bcs MineAntiClockwise
MineClockwise
  ; clc
  lda CURRSECTOR
  adc #1
  jmp StoreMineRotate
MineAntiClockwise
  ; sec
  lda CURRSECTOR
  sbc #1
StoreMineRotate
  and #%00011111
  ora #%11100000
  ldy MPHASE
  sta MINEDIR_W,Y

EndMineLogic
  ; Update Mine Phase
  ldy MPHASE
  dey
  bpl StoreMinePhase
  ldy #2
StoreMinePhase
  sty MPHASE
  ; Display Kernel
  jmp EnterKernel

  ; Mine Indices (255 = no movement)
MineXIndex
  DC.B  255, 0, 1, 2, 3, 4, 5, 6, 7, 6, 5, 4, 3, 2, 1, 0
  DC.B  255, 8, 9, 10, 11, 12, 13, 14, 15, 14, 13, 12, 11, 10, 9, 8
MineYIndex
  DC.B  0, 1, 2, 3, 4, 5, 6, 7, 255, 8, 9, 10, 11, 12, 13, 14, 15
  DC.B  14, 13, 12, 11, 10, 9, 8, 255, 7, 6, 5, 4, 3, 2, 1, 0
EndMineIndex
  if (>MineXIndex != >EndMineIndex)
    echo "WARNING: MineIndex crosses a page boundary!"
  endif

  DC.B  "ERIC0"

; -----------------------------------------------------------------------------
; Subroutines
; -----------------------------------------------------------------------------

  ALIGN 256

  ; Distance from Centre
  ; Uses: TEMPBX, TEMPBY
  ; Returns: DISTANCE_HI, DISTANCE_LO
CentreDistance
  sec                       ; [0] + 2
  lda TEMPBX                ; [2] + 3
  sbc #MIDDLEX              ; [5] + 2
  bcs EndLeftCheck          ; [7] + 2/3
  ; sec
  lda #MIDDLEX+1            ; [9] + 2       +1 for clear carry
  sbc TEMPBX                ; [11] + 3
EndLeftCheck
  tax                       ; [14] + 2
  sec                       ; [16] + 2
  lda TEMPBY                ; [18] + 3
  sbc #MIDDLEY              ; [21] + 2
  bcs EndDownCheck          ; [23] + 2/3
  ; sec
  lda #MIDDLEY+1            ; [25] + 2      +1 for clear carry
  sbc TEMPBY                ; [27] + 3
EndDownCheck
  tay                       ; [30] + 2
  ; Square Distance
  clc                       ; [32] + 2
  lda SquaresLo,X           ; [34] + 4
  adc SquaresLo,Y           ; [38] + 4
  sta DISTANCE_LO           ; [42] + 3
  lda SquaresHi,X           ; [45] + 4
  adc SquaresHi,Y           ; [49] + 4
  ; sta DISTANCE_HI
  rts                       ; [53] + 6 
                            ; WORST = 59 CYCLES

; -----------------------------------------------------------------------------

  ; Check Collision With Ring 0
  ; Uses: CURRSECTOR, TEMPX, TEMPY, TEMPBX, TEMPBY
CollideRing0
  ldy #11
  sty CURRSECTOR
  clc
CollideRing0Loop
  ; Fetch Positions
  ldy CURRSECTOR            ; [0] + 3
  lda SECTORMAP0_R,Y        ; [3] + 4
  ; Skip Empty Sectors
  cmp #$FF                  ; [7] + 2
  beq EndCollide0Loop       ; [9] + 2/3
  ; Fetch Sector Offsets
  lda OFFSET0,Y             ; [11] + 4
  ldx NextSectorIndexA,Y    ; [15] + 4
  ldy OFFSET0,X             ; [19] + 4
  tax                       ; [23] + 2
  ; Calculate X Box
  lda Ring0XPosition,Y      ; [25] + 4
  sbc Ring0XPosition,X      ; [29] + 4
  bcc ReverseX0             ; [33] + 2/3
  adc #2                    ; [35] + 2      +3 inc carry
  sta TEMPX                 ; [37] + 3
  lda Ring0XPosition,X      ; [40] + 4
  jmp TestX0                ; [44] + 3
ReverseX0
  eor #$FF                  ; [36] + 2
  adc #4                    ; [38] + 2      +1 for negation
  sta TEMPX                 ; [40] + 3
  lda Ring0XPosition,Y      ; [43] + 4
TestX0
  ; Check X Overlap
  sbc TEMPBX                ; [47] + 3
  sbc #2                    ; [50] + 2
  adc TEMPX                 ; [52] + 3
  bcc EndCollide0Loop       ; [55] + 2/3
  ; Calculate Y Box
  lda Ring0YPosition,Y      ; [57] + 4
  sbc Ring0YPosition,X      ; [61] + 4
  bcc ReverseY0             ; [65] + 2/3
  adc #2                    ; [67] + 2      +3 inc carry
  sta TEMPY                 ; [69] + 3
  lda Ring0YPosition,X      ; [72] + 4
  jmp TestY0                ; [76] + 3
ReverseY0
  eor #$FF                  ; [68] + 2
  adc #4                    ; [70] + 2      +1 for negation
  sta TEMPY                 ; [72] + 3
  lda Ring0YPosition,Y      ; [75] + 4
TestY0
  ; Check Y Overlap
  sbc TEMPBY                ; [79] + 3
  sbc #2                    ; [82] + 2
  adc TEMPY                 ; [84] + 3
  bcs Collide0              ; [87] + 2/3
EndCollide0Loop
  dec CURRSECTOR            ; [89] + 5
  bpl CollideRing0Loop      ; [94] + 2/3
Collide0
  lda CURRSECTOR            ; WORST = 97 * 12 = 1164 (15.3 SCANLINES)
  rts
EndCollideRing0
  if (>CollideRing0 != >EndCollideRing0)
    echo "WARNING: CollideRing0 crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Check Collision With Ring 1
CollideRing1
  ldy #11
  sty CURRSECTOR
  clc
CollideRing1Loop
  ; Fetch Positions
  ldy CURRSECTOR
  lda SECTORMAP1_R,Y
  ; Skip Empty Sectors
  cmp #$FF
  beq EndCollide1Loop
  ; Fetch Sector Offsets
  lda OFFSET1,Y
  ldx NextSectorIndexA,Y
  ldy OFFSET1,X
  tax
  ; Calculate X Box
  lda Ring1XPosition,Y
  sbc Ring1XPosition,X
  bcc ReverseX1
  adc #2
  sta TEMPX
  lda Ring1XPosition,X 
  jmp TestX1
ReverseX1
  eor #$FF
  adc #4
  sta TEMPX
  lda Ring1XPosition,Y
TestX1
  sbc TEMPBX
  sbc #2
  adc TEMPX
  bcc EndCollide1Loop
  ; Calculate Y Box
  lda Ring1YPosition,Y
  sbc Ring1YPosition,X
  bcc ReverseY1
  adc #2
  sta TEMPY
  lda Ring1YPosition,X
  jmp TestY1
ReverseY1
  eor #$FF
  adc #4
  sta TEMPY
  lda Ring1YPosition,Y
TestY1
  sbc TEMPBY
  sbc #2
  adc TEMPY
  bcs Collide1
EndCollide1Loop
  dec CURRSECTOR
  bpl CollideRing1Loop
Collide1
  lda CURRSECTOR
  rts
EndCollideRing1
  if (>CollideRing1 != >EndCollideRing1)
    echo "WARNING: CollideRing1 crosses a page boundary!"
  endif

  DC.B  "ERIC1"

; -----------------------------------------------------------------------------

  ALIGN 256

  ; Check Collision With Ring 2
CollideRing2
  ldy #11
  sty CURRSECTOR
  clc
CollideRing2Loop
  ; Fetch Positions
  ldy CURRSECTOR
  lda SECTORMAP2_R,Y
  ; Skip Empty Sectors
  cmp #$FF
  beq EndCollide2Loop
  ; Fetch Sector Offsets
  lda OFFSET2,Y
  ldx NextSectorIndexA,Y
  ldy OFFSET2,X
  tax
  ; Calculate X Box
  lda Ring2XPosition,Y
  sbc Ring2XPosition,X
  bcc ReverseX2
  adc #2
  sta TEMPX
  lda Ring2XPosition,X
  jmp TestX2
ReverseX2
  eor #$FF
  adc #4
  sta TEMPX
  lda Ring2XPosition,Y
TestX2
  sbc TEMPBX
  sbc #2
  adc TEMPX
  bcc EndCollide2Loop
  ; Calculate Y Box
  lda Ring2YPosition,Y
  sbc Ring2YPosition,X
  bcc ReverseY2
  adc #2
  sta TEMPY
  lda Ring2YPosition,X
  jmp TestY2
ReverseY2
  eor #$FF
  adc #4
  sta TEMPY
  lda Ring2YPosition,Y
TestY2
  sbc TEMPBY
  sbc #2
  adc TEMPY
  bcs Collide2
EndCollide2Loop
  dec CURRSECTOR
  bpl CollideRing2Loop
Collide2
  lda CURRSECTOR
  rts
EndCollideRing2
  if (>CollideRing2 != >EndCollideRing2)
    echo "WARNING: CollideRing2 crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Vector Angle (TEMPX, TEMPY)
  ; Returns ANGLE
  ; (very approximate CORDIC atan2 algorithm)
VectorAngle
  ldy #0                    ; [0] + 2         A = 0
AngleTest0
  lda TEMPY                 ; [2] + 3
  bpl AngleTest1            ; [5] + 2/3       Y < 0
  clc                       ; [7] + 2
  eor #$FF                  ; [9] + 2
  adc #1                    ; [11] + 2
  sta TEMPY                 ; [13] + 2        Y = -Y
  sec                       ; [15] + 2
  lda #0                    ; [17] + 2
  sbc TEMPX                 ; [19] + 3
  sta TEMPX                 ; [22] + 3        X = -X
  ldy #16                   ; [25] + 2        A = 16
AngleTest1
  lda TEMPX                 ; [27] + 3
  bpl AngleTest2            ; [30] + 2/3      X < 0
  ldx TEMPY                 ; [32] + 3
  stx TEMPX                 ; [35] + 3        X = Y
  clc                       ; [38] + 2
  eor #$FF                  ; [40] + 2
  adc #1                    ; [42] + 2
  sta TEMPY                 ; [44] + 3        Y = -X
  tya                       ; [47] + 2
  clc                       ; [49] + 2
  adc #8                    ; [51] + 2
  tay                       ; [53] + 2        A += 8
AngleTest2
  clc                       ; [55] + 2
  lax TEMPY                 ; [57] + 3
  sbc TEMPX                 ; [60] + 3
  bvc SkipEor2              ; [63] + 2/3
  eor #$80                  ; [65] + 2
SkipEor2
  bmi AngleTest3            ; [67] + 2/3      Y > X
  sec                       ; [69] + 2
  txa                       ; [71] + 2
  sbc TEMPX                 ; [73] + 3
  sta TEMPY                 ; [76] + 3        Y = Y - X
  clc                       ; [79] + 2
  txa                       ; [81] + 2
  adc TEMPX                 ; [83] + 3
  sta TEMPX                 ; [86] + 3        X = X + Y
  iny                       ; [89] + 2
  iny                       ; [91] + 2
  iny                       ; [93] + 2
  iny                       ; [95] + 2        A += 4
AngleTest3
  lax TEMPY                 ; [97] + 3
  asl                       ; [100] + 2
  clc                       ; [102] + 2
  sbc TEMPX                 ; [104] + 3
  bvc SkipEor3              ; [107] + 2/3
  eor #$80                  ; [109] + 2
SkipEor3
  bmi AngleTest4            ; [111] + 2/3     2Y > X
  sec                       ; [113] + 2
  txa                       ; [115] + 2
  asl                       ; [117] + 2
  sec                       ; [119] + 2
  sbc TEMPX                 ; [121] + 3
  sta TEMPY                 ; [124] + 3       Y = 2Y - X
  asl TEMPX                 ; [127] + 3
  clc                       ; [130] + 2
  txa                       ; [132] + 2
  adc TEMPX                 ; [134] + 3
  sta TEMPX                 ; [137] + 3       X = 2X + Y
  iny                       ; [140] + 2
  iny                       ; [142] + 2       A += 2
AngleTest4
  lda TEMPY                 ; [144] + 3
  asl                       ; [147] + 2
  asl                       ; [149] + 2
  clc                       ; [151] + 2
  sbc TEMPX                 ; [153] + 3
  bvc SkipEor4              ; [156] + 2/3
  eor #$80                  ; [158] + 2
SkipEor4
  bmi EndAngleTest          ; [160] + 2/3     4Y > X
  iny                       ; [162] + 2       A += 1
EndAngleTest
  lda AngleMap,Y            ; [164] + 4
  rts                       ; [168] + 6
                            ; TOTAL = 174 CYCLES
EndVectorAngle
  if (>VectorAngle != >EndVectorAngle)
    echo "WARNING: VectorAngle crosses a page boundary!"
  endif

  ; Map Angle to Sprite Direction
AngleMap
  DC.B  7,6,5,4,3,2,1,0,31,30,29,28,27,26,25,24
  DC.B  23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8

  DC.B  "ERIC2"

; -----------------------------------------------------------------------------
; Game Data
; -----------------------------------------------------------------------------

  ALIGN 256

  ; Ring Positions (X Coordinates)
Ring0XPosition
  DC.B  15+XXX, 14+XXX, 13+XXX, 12+XXX, 11+XXX, 10+XXX,  9+XXX,  8+XXX
  DC.B   7+XXX,  6+XXX,  5+XXX,  4+XXX,  3+XXX,  3+XXX,  2+XXX,  2+XXX
  DC.B   1+XXX,  1+XXX,  1+XXX,  0+XXX,  0+XXX,  0+XXX,  0+XXX,  0+XXX
  DC.B   0+XXX,  0+XXX,  0+XXX,  1+XXX,  1+XXX,  1+XXX,  2+XXX,  2+XXX
  DC.B   3+XXX,  3+XXX,  4+XXX,  5+XXX,  6+XXX,  7+XXX,  8+XXX,  9+XXX
  DC.B  10+XXX, 11+XXX, 12+XXX, 13+XXX, 14+XXX, 15+XXX, 16+XXX, 17+XXX
  DC.B  18+XXX, 19+XXX, 20+XXX, 21+XXX, 22+XXX, 23+XXX, 24+XXX, 25+XXX
  DC.B  26+XXX, 27+XXX, 28+XXX, 29+XXX, 30+XXX, 31+XXX, 32+XXX, 33+XXX
  DC.B  34+XXX, 35+XXX, 36+XXX, 36+XXX, 37+XXX, 37+XXX, 38+XXX, 38+XXX
  DC.B  38+XXX, 39+XXX, 39+XXX, 39+XXX, 39+XXX, 39+XXX, 39+XXX, 39+XXX
  DC.B  39+XXX, 38+XXX, 38+XXX, 38+XXX, 37+XXX, 37+XXX, 36+XXX, 36+XXX
  DC.B  35+XXX, 34+XXX, 33+XXX, 32+XXX, 31+XXX, 30+XXX, 29+XXX, 28+XXX
  DC.B  27+XXX, 26+XXX, 25+XXX, 24+XXX, 23+XXX, 22+XXX, 21+XXX, 20+XXX
  DC.B  19+XXX, 18+XXX, 17+XXX, 16+XXX
EndRing0XPosition
  if (>Ring0XPosition != >EndRing0XPosition)
    echo "WARNING: Ring0XPosition table crosses a page boundary!"
  endif

Ring1XPosition
  DC.B  16+XXX, 17+XXX, 18+XXX, 19+XXX, 20+XXX, 21+XXX, 22+XXX, 23+XXX
  DC.B  24+XXX, 25+XXX, 26+XXX, 27+XXX, 28+XXX, 29+XXX, 30+XXX, 31+XXX
  DC.B  32+XXX, 32+XXX, 33+XXX, 33+XXX, 33+XXX, 34+XXX, 34+XXX, 34+XXX
  DC.B  34+XXX, 34+XXX, 34+XXX, 34+XXX, 34+XXX, 33+XXX, 33+XXX, 33+XXX
  DC.B  32+XXX, 32+XXX, 31+XXX, 30+XXX, 29+XXX, 28+XXX, 27+XXX, 26+XXX
  DC.B  25+XXX, 24+XXX, 23+XXX, 22+XXX, 21+XXX, 20+XXX, 19+XXX, 18+XXX
  DC.B  17+XXX, 16+XXX, 15+XXX, 14+XXX, 13+XXX, 12+XXX, 11+XXX, 10+XXX
  DC.B   9+XXX,  8+XXX,  7+XXX,  7+XXX,  6+XXX,  6+XXX,  6+XXX,  5+XXX
  DC.B   5+XXX,  5+XXX,  5+XXX,  5+XXX,  5+XXX,  5+XXX,  5+XXX,  6+XXX
  DC.B   6+XXX,  6+XXX,  7+XXX,  7+XXX,  8+XXX,  9+XXX, 10+XXX, 11+XXX
  DC.B  12+XXX, 13+XXX, 14+XXX, 15+XXX
EndRing1XPosition
  if (>Ring1XPosition != >EndRing1XPosition)
    echo "WARNING: Ring0XPosition table crosses a page boundary!"
  endif
  
Ring2XPosition
  DC.B  17+XXX, 16+XXX, 15+XXX, 14+XXX, 13+XXX, 12+XXX, 11+XXX, 11+XXX
  DC.B  11+XXX, 10+XXX, 10+XXX, 10+XXX, 10+XXX, 10+XXX, 10+XXX, 11+XXX
  DC.B  11+XXX, 11+XXX, 12+XXX, 13+XXX, 14+XXX, 15+XXX, 16+XXX, 17+XXX
  DC.B  17+XXX, 18+XXX, 19+XXX, 20+XXX, 21+XXX, 22+XXX, 22+XXX, 23+XXX
  DC.B  24+XXX, 25+XXX, 26+XXX, 27+XXX, 28+XXX, 28+XXX, 28+XXX, 29+XXX
  DC.B  29+XXX, 29+XXX, 29+XXX, 29+XXX, 29+XXX, 28+XXX, 28+XXX, 28+XXX
  DC.B  27+XXX, 26+XXX, 25+XXX, 24+XXX, 23+XXX, 22+XXX, 22+XXX, 21+XXX
  DC.B  20+XXX, 19+XXX, 18+XXX, 17+XXX
EndRing2XPosition
  if (>Ring2XPosition != >EndRing2XPosition)
    echo "WARNING: Ring2XPosition table crosses a page boundary!"
  endif

  DC.B  "E3"

  ALIGN 256

  ; Ring Positions (Y Coordinates)
Ring0YPosition
  DC.B   1+YYY,  1+YYY,  1+YYY,  2+YYY,  2+YYY,  3+YYY,  3+YYY,  4+YYY
  DC.B   5+YYY,  6+YYY,  7+YYY,  8+YYY,  9+YYY, 10+YYY, 11+YYY, 12+YYY
  DC.B  13+YYY, 14+YYY, 15+YYY, 16+YYY, 17+YYY, 18+YYY, 19+YYY, 20+YYY
  DC.B  21+YYY, 22+YYY, 23+YYY, 24+YYY, 25+YYY, 26+YYY, 27+YYY, 28+YYY
  DC.B  29+YYY, 30+YYY, 31+YYY, 32+YYY, 33+YYY, 34+YYY, 35+YYY, 36+YYY
  DC.B  36+YYY, 37+YYY, 37+YYY, 38+YYY, 38+YYY, 38+YYY, 39+YYY, 39+YYY
  DC.B  39+YYY, 39+YYY, 39+YYY, 39+YYY, 39+YYY, 39+YYY, 38+YYY, 38+YYY
  DC.B  38+YYY, 37+YYY, 37+YYY, 36+YYY, 36+YYY, 35+YYY, 34+YYY, 33+YYY
  DC.B  32+YYY, 31+YYY, 30+YYY, 29+YYY, 28+YYY, 27+YYY, 26+YYY, 25+YYY
  DC.B  24+YYY, 23+YYY, 22+YYY, 21+YYY, 20+YYY, 19+YYY, 18+YYY, 17+YYY
  DC.B  16+YYY, 15+YYY, 14+YYY, 13+YYY, 12+YYY, 11+YYY, 10+YYY,  9+YYY
  DC.B   8+YYY,  7+YYY,  6+YYY,  5+YYY,  4+YYY,  3+YYY,  3+YYY,  2+YYY
  DC.B   2+YYY,  1+YYY,  1+YYY,  1+YYY,  0+YYY,  0+YYY,  0+YYY,  0+YYY
  DC.B   0+YYY,  0+YYY,  0+YYY,  0+YYY
EndRing0YPosition
  if (>Ring0YPosition != >EndRing0YPosition)
    echo "WARNING: Ring0YPosition table crosses a page boundary!"
  endif

Ring1YPosition
  DC.B   5+YYY,  5+YYY,  5+YYY,  5+YYY,  5+YYY,  5+YYY,  5+YYY,  5+YYY
  DC.B   6+YYY,  6+YYY,  7+YYY,  7+YYY,  8+YYY,  9+YYY, 10+YYY, 11+YYY
  DC.B  12+YYY, 13+YYY, 14+YYY, 15+YYY, 16+YYY, 16+YYY, 17+YYY, 18+YYY
  DC.B  19+YYY, 20+YYY, 21+YYY, 22+YYY, 23+YYY, 23+YYY, 24+YYY, 25+YYY
  DC.B  26+YYY, 27+YYY, 28+YYY, 29+YYY, 30+YYY, 31+YYY, 32+YYY, 32+YYY
  DC.B  33+YYY, 33+YYY, 34+YYY, 34+YYY, 34+YYY, 34+YYY, 34+YYY, 34+YYY
  DC.B  34+YYY, 34+YYY, 33+YYY, 33+YYY, 32+YYY, 32+YYY, 31+YYY, 30+YYY
  DC.B  29+YYY, 28+YYY, 27+YYY, 26+YYY, 25+YYY, 24+YYY, 23+YYY, 23+YYY
  DC.B  22+YYY, 21+YYY, 20+YYY, 19+YYY, 18+YYY, 17+YYY, 16+YYY, 16+YYY
  DC.B  15+YYY, 14+YYY, 13+YYY, 12+YYY, 11+YYY, 10+YYY,  9+YYY,  8+YYY
  DC.B   7+YYY,  7+YYY,  6+YYY,  6+YYY
EndRing1YPosition
  if (>Ring1YPosition != >EndRing1YPosition)
    echo "WARNING: Ring1YPosition table crosses a page boundary!"
  endif

Ring2YPosition
  DC.B  11+YYY, 11+YYY, 11+YYY, 12+YYY, 13+YYY, 14+YYY, 15+YYY, 16+YYY
  DC.B  17+YYY, 17+YYY, 18+YYY, 19+YYY, 20+YYY, 21+YYY, 22+YYY, 22+YYY
  DC.B  23+YYY, 24+YYY, 25+YYY, 26+YYY, 27+YYY, 28+YYY, 28+YYY, 28+YYY
  DC.B  29+YYY, 29+YYY, 29+YYY, 29+YYY, 29+YYY, 29+YYY, 28+YYY, 28+YYY
  DC.B  28+YYY, 27+YYY, 26+YYY, 25+YYY, 24+YYY, 23+YYY, 22+YYY, 22+YYY
  DC.B  21+YYY, 20+YYY, 19+YYY, 18+YYY, 17+YYY, 17+YYY, 16+YYY, 15+YYY
  DC.B  14+YYY, 13+YYY, 12+YYY, 11+YYY, 11+YYY, 11+YYY, 10+YYY, 10+YYY
  DC.B  10+YYY, 10+YYY, 10+YYY, 10+YYY
EndRing2YPosition
  if (>Ring2YPosition != >EndRing2YPosition)
    echo "WARNING: Ring2YPosition table crosses a page boundary!"
  endif

  DC.B  "E4"

  ALIGN 256

  ; Mine X Offsets (LO)
MineLoX
  ; 0.25
  DC.B  %00001100, %00011000, %00100100, %00101101
  DC.B  %00110101, %00111011, %00111111, %01000000
  DC.B  %11110100, %11101000, %11011100, %11010011
  DC.B  %11001011, %11000101, %11000001, %11000000
  ; 0.35
  DC.B  %00010001, %00100010, %00110010, %00111111
  DC.B  %01001010, %01010011, %01011000, %01011010
  DC.B  %11101111, %11011110, %11001110, %11000001
  DC.B  %10110110, %10101101, %10101000, %10100110
  ; 0.45
  DC.B  %00010110, %00101100, %01000000, %01010001
  DC.B  %01100000, %01101010, %01110001, %01110011
  DC.B  %11101010, %11010100, %11000000, %10101111
  DC.B  %10100000, %10010110, %10001111, %10001101
  ; 0.55
  DC.B  %00011011, %00110110, %01001110, %01100100
  DC.B  %01110101, %10000010, %10001010, %10001101
  DC.B  %11100101, %11001010, %10110010, %10011100
  DC.B  %10001011, %01111110, %01110110, %01110011
  ; 0.65
  DC.B  %00100000, %01000000, %01011100, %01110110
  DC.B  %10001010, %10011010, %10100011, %10100110
  DC.B  %11100000, %11000000, %10100100, %10001010
  DC.B  %01110110, %01100110, %01011101, %01011010
  ; 0.75
  DC.B  %00100101, %01001001, %01101011, %10001000
  DC.B  %10100000, %10110001, %10111100, %11000000
  DC.B  %11011011, %10110111, %10010101, %01111000
  DC.B  %01100000, %01001111, %01000100, %01000000
  ; 0.85
  DC.B  %00101010, %01010011, %01111001, %10011010
  DC.B  %10110101, %11001001, %11010101, %11011010
  DC.B  %11010110, %10101101, %10000111, %01100110
  DC.B  %01001011, %00110111, %00101011, %00100110
  ; 0.95
  DC.B  %00101111, %01011101, %10000111, %10101100
  DC.B  %11001010, %11100001, %11101111, %11110011
  DC.B  %11010001, %10100011, %01111001, %01010100
  DC.B  %00110110, %00011111, %00010001, %00001101

  ; Mine Y Offsets (LO)
MineLoY
  ; 0.25
  DC.B  %01000000, %00111111, %00111011, %00110101
  DC.B  %00101101, %00100100, %00011000, %00001100
  DC.B  %11110100, %11101000, %11011100, %11010011
  DC.B  %11001011, %11000101, %11000001, %11000000
  ; 0.35
  DC.B  %01011010, %01011000, %01010011, %01001010
  DC.B  %00111111, %00110010, %00100010, %00010001
  DC.B  %11101111, %11011110, %11001110, %11000001
  DC.B  %10110110, %10101101, %10101000, %10100110
  ; 0.45
  DC.B  %01110011, %01110001, %01101010, %01100000
  DC.B  %01010001, %01000000, %00101100, %00010110
  DC.B  %11101010, %11010100, %11000000, %10101111
  DC.B  %10100000, %10010110, %10001111, %10001101
  ; 0.55
  DC.B  %10001101, %10001010, %10000010, %01110101
  DC.B  %01100100, %01001110, %00110110, %00011011
  DC.B  %11100101, %11001010, %10110010, %10011100
  DC.B  %10001011, %01111110, %01110110, %01110011
  ; 0.65
  DC.B  %10100110, %10100011, %10011010, %10001010
  DC.B  %01110110, %01011100, %01000000, %00100000
  DC.B  %11100000, %11000000, %10100100, %10001010
  DC.B  %01110110, %01100110, %01011101, %01011010
  ; 0.75
  DC.B  %11000000, %10111100, %10110001, %10100000
  DC.B  %10001000, %01101011, %01001001, %00100101
  DC.B  %11011011, %10110111, %10010101, %01111000
  DC.B  %01100000, %01001111, %01000100, %01000000
  ; 0.85
  DC.B  %11011010, %11010101, %11001001, %10110101
  DC.B  %10011010, %01111001, %01010011, %00101010
  DC.B  %11010110, %10101101, %10000111, %01100110
  DC.B  %01001011, %00110111, %00101011, %00100110
  ; 0.95
  DC.B  %11110011, %11101111, %11100001, %11001010
  DC.B  %10101100, %10000111, %01011101, %00101111
  DC.B  %11010001, %10100011, %01111001, %01010100
  DC.B  %00110110, %00011111, %00010001, %00001101

  ALIGN 256

  ; Mine X & Y Offsets (HI)
MineHiX
MineHiY
  ; 0.25
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %11111111, %11111111, %11111111, %11111111
  DC.B  %11111111, %11111111, %11111111, %11111111
  ; 0.35
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %11111111, %11111111, %11111111, %11111111
  DC.B  %11111111, %11111111, %11111111, %11111111
  ; 0.45
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %11111111, %11111111, %11111111, %11111111
  DC.B  %11111111, %11111111, %11111111, %11111111
  ; 0.55
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %11111111, %11111111, %11111111, %11111111
  DC.B  %11111111, %11111111, %11111111, %11111111
  ; 0.65
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %11111111, %11111111, %11111111, %11111111
  DC.B  %11111111, %11111111, %11111111, %11111111
  ; 0.75
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %11111111, %11111111, %11111111, %11111111
  DC.B  %11111111, %11111111, %11111111, %11111111
  ; 0.85
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %11111111, %11111111, %11111111, %11111111
  DC.B  %11111111, %11111111, %11111111, %11111111
  ; 0.95
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %11111111, %11111111, %11111111, %11111111
  DC.B  %11111111, %11111111, %11111111, %11111111

  ; Ship/Mine Kick (2.0)
KickTable
KickLoX
  DC.B  %00000000, %01100100, %11000100, %00011100
  DC.B  %01101010, %10101010, %11011001, %11110110
  DC.B  %00000000, %11110110, %11011001, %10101010
  DC.B  %01101010, %00011100, %11000100, %01100100
  DC.B  %00000000, %10011100, %00111100, %11100100
  DC.B  %10010110, %01010110, %00100111, %00001010
  DC.B  %00000000, %00001010, %00100111, %01010110
  DC.B  %10010110, %11100100, %00111100, %10011100
KickHiX
  DC.B  %00000000, %00000000, %00000000, %00000001
  DC.B  %00000001, %00000001, %00000001, %00000001
  DC.B  %00000010, %00000001, %00000001, %00000001
  DC.B  %00000001, %00000001, %00000000, %00000000
  DC.B  %00000000, %11111111, %11111111, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111111, %11111111
KickLoY
  DC.B  %00000000, %11110110, %11011001, %10101010
  DC.B  %01101010, %00011100, %11000100, %01100100
  DC.B  %00000000, %10011100, %00111100, %11100100
  DC.B  %10010110, %01010110, %00100111, %00001010
  DC.B  %00000000, %00001010, %00100111, %01010110
  DC.B  %10010110, %11100100, %00111100, %10011100
  DC.B  %00000000, %01100100, %11000100, %00011100
  DC.B  %01101010, %10101010, %11011001, %11110110
KickHiY
  DC.B  %00000010, %00000001, %00000001, %00000001
  DC.B  %00000001, %00000001, %00000000, %00000000
  DC.B  %00000000, %11111111, %11111111, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111110, %11111110
  DC.B  %11111110, %11111110, %11111111, %11111111
  DC.B  %00000000, %00000000, %00000000, %00000001
  DC.B  %00000001, %00000001, %00000001, %00000001
EndKickTable
;  if (>KickTable != >EndKickTable)
;    echo "WARNING: KickTable crosses a page boundary!"
;  endif

  ALIGN 256

  ; Squares Table (LO)
SquaresLo
  DC.B  %00000000, %00000001, %00000100, %00001001
  DC.B  %00010000, %00011001, %00100100, %00110001
  DC.B  %01000000, %01010001, %01100100, %01111001
  DC.B  %10010000, %10101001, %11000100, %11100001
  DC.B  %00000000, %00100001, %01000100, %01101001
  DC.B  %10010000, %10111001, %11100100, %00010001
  DC.B  %01000000, %01110001, %10100100, %11011001
  DC.B  %00010000, %01001001, %10000100, %11000001
  DC.B  %00000000, %01000001, %10000100, %11001001
  DC.B  %00010000, %01011001, %10100100, %11110001
  DC.B  %01000000, %10010001, %11100100, %00111001
  DC.B  %10010000, %11101001, %01000100, %10100001
  DC.B  %00000000, %01100001, %11000100, %00101001
  DC.B  %10010000, %11111001, %01100100, %11010001
  DC.B  %01000000, %10110001, %00100100, %10011001
  DC.B  %00010000, %10001001, %00000100, %10000001
  DC.B  %00000000, %10000001, %00000100, %10001001
  DC.B  %00010000, %10011001, %00100100, %10110001
  DC.B  %01000000, %11010001, %01100100, %11111001
  DC.B  %10010000, %00101001, %11000100, %01100001
  DC.B  %00000000, %10100001, %01000100, %11101001
  DC.B  %10010000, %00111001, %11100100, %10010001
  DC.B  %01000000, %11110001, %10100100, %01011001
EndSquaresLo
  if (>SquaresLo != >EndSquaresLo)
    echo "WARNING: SquaresLo table crosses a page boundary!"
  endif

  ; Squares Table (HI)
SquaresHi
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000000, %00000000, %00000000, %00000000
  DC.B  %00000001, %00000001, %00000001, %00000001
  DC.B  %00000001, %00000001, %00000001, %00000010
  DC.B  %00000010, %00000010, %00000010, %00000010
  DC.B  %00000011, %00000011, %00000011, %00000011
  DC.B  %00000100, %00000100, %00000100, %00000100
  DC.B  %00000101, %00000101, %00000101, %00000101
  DC.B  %00000110, %00000110, %00000110, %00000111
  DC.B  %00000111, %00000111, %00001000, %00001000
  DC.B  %00001001, %00001001, %00001001, %00001010
  DC.B  %00001010, %00001010, %00001011, %00001011
  DC.B  %00001100, %00001100, %00001101, %00001101
  DC.B  %00001110, %00001110, %00001111, %00001111
  DC.B  %00010000, %00010000, %00010001, %00010001
  DC.B  %00010010, %00010010, %00010011, %00010011
  DC.B  %00010100, %00010100, %00010101, %00010101
  DC.B  %00010110, %00010111, %00010111, %00011000
  DC.B  %00011001, %00011001, %00011010, %00011010
  DC.B  %00011011, %00011100, %00011100, %00011101
  DC.B  %00011110, %00011110, %00011111, %00100000
EndSquaresHi
  if (>SquaresHi != >EndSquaresHi)
    echo "WARNING: SquaresHi table crosses a page boundary!"
  endif

  ; Next Sectors
NextSectorIndexA
  DC.B  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 0
EndNextSectorIndexA
  if (>NextSectorIndexA != >EndNextSectorIndexA)
    echo "WARNING: NextSectorIndexA table crosses a page boundary!"
  endif

  ; Gun Movement Speeds
GunSpeeds
  DC.B  64, 88, 112, 136, 160, 184, 208, 255
EndGunSpeeds
  if (>GunSpeeds != >EndGunSpeeds)
    echo "WARNING: GunSpeeds table crosses a page boundary!"
  endif

  ; Mine and Projectile Masks
WaveMask
  DC.B  %00111111, %00011111, %00001111, %00000111
EndWaveMask
  if (>WaveMask != >EndWaveMask)
    echo "WARNING: WaveMask table crosses a page boundary!"
  endif

  ; Homing Mine Colours
MineCols
  DC.B  $8C, $BC, $0C       ; NTSC/PAL60/B&W
EndMineCols
  if (>MineCols != >EndMineCols)
    echo "WARNING: MineCols crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------
; Startup Logo
; -----------------------------------------------------------------------------

AALogoA
AALogo0
  DC.B  %11111000
  DC.B  %11110000
  DC.B  %11100111
  DC.B  %11001111
  DC.B  %11011000
  DC.B  %10010000
  DC.B  %10111000
  DC.B  %10101000
  DC.B  %00101101
  DC.B  %01100101
  DC.B  %01000101
  DC.B  %01000111
  DC.B  %01001010
  DC.B  %01001010
  DC.B  %01001101
  DC.B  %01100101
  DC.B  %00100101
  DC.B  %10100111
  DC.B  %10110010
  DC.B  %10010010
  DC.B  %11011000
  DC.B  %11001111
  DC.B  %11100111
  DC.B  %11110000
  DC.B  %11111000
EndAALogoA
  if (>AALogoA != >EndAALogoA)
    echo "WARNING: AALogoA crosses a page boundary!"
  endif

  DC.B  "ERIC7"

  ALIGN 256

EnterStartup
  ; Skip Lines
  ldy #68
StartupSkipLoop
  sta WSYNC
  dey
  bpl StartupSkipLoop

  ; Non-Reflected Playfield & Black Sprites
  ldx #0
  stx CTRLPF
  stx COLUP0
  stx COLUP1

  ; Blank Screen and Set Playfield
  ldx #%00110010
  stx VBLANK
  stx ENAM0
  stx PF0
  ldx #%11111100
  stx PF2

  ; Logo Colours (For Both NTSC & PAL)
  ldx #$96
  stx COLUPF
  ldx #$26
  stx COLUBK

  ; Load First Sprite
  ldy #24
  sty ROW
  lda AALogo0,Y

  ; Begin Logo Display
  jmp AALogoStart

AALogoLoop
  ldy ROW                 ; [65] + 3
  lda AALogo0,Y           ; [68] + 4
AALogoStart
  sta WSYNC               ; [0]
  nop                     ; [0] + 2
  sta GRP0                ; [2] + 3
  lda AALogo1,Y           ; [5] + 4
  sta GRP1                ; [9] + 3  < 42
  lda AALogo2,Y           ; [12] + 4
  sta GRP0                ; [16] + 3  < 44
  ldx AALogo4,Y           ; [19] + 4
  lda AALogo5,Y           ; [23] + 4
  sta TEMP                ; [27] + 3
  lda AALogo3,Y           ; [30] + 4
  ldy #0                  ; [34] + 2
  sty VBLANK              ; [36] + 3  = 39
  ldy TEMP                ; [39] + 3
  sta GRP1                ; [42] + 3  > 44 < 47
  stx GRP0                ; [45] + 3  > 46 < 50
  sty GRP1                ; [48] + 3  > 49 < 52
  sta GRP0                ; [51] + 3  > 52 < 55
  stx VBLANK              ; [54] + 3  = 57
  dec ROW                 ; [57] + 5
  bpl AALogoLoop          ; [62] + 2/3
  ldx #0                  ; [64] + 2
  stx GRP0                ; [66] + 3
  stx GRP1                ; [69] + 3
  stx GRP0                ; [72] + 3
  stx ENAM0               ; [4] + 3
  stx PF0                 ; [7] + 3
  stx PF2                 ; [10] + 3
  stx COLUBK              ; [13] + 3
  stx NUSIZ0              ; [16] + 3
  stx NUSIZ1              ; [19] + 3
  stx VDELP0              ; [22] + 3
  stx VDELP1              ; [25] + 3
  jmp LeaveStartup

AALogoB
AALogo1
  DC.B  %11111111
  DC.B  %01111111
  DC.B  %00111111
  DC.B  %10011111
  DC.B  %11011111
  DC.B  %01001111
  DC.B  %11101111
  DC.B  %10101101
  DC.B  %10100101
  DC.B  %00110101
  DC.B  %00010100
  DC.B  %00010100
  DC.B  %10010110
  DC.B  %10010110
  DC.B  %10010110
  DC.B  %00110110
  DC.B  %00100110
  DC.B  %00101111
  DC.B  %01101111
  DC.B  %01001111
  DC.B  %11011111
  DC.B  %10011111
  DC.B  %00111111
  DC.B  %01111111
; DC.B  %11111111
AALogo5
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %01111111
  DC.B  %00111111
  DC.B  %10011111
  DC.B  %11011111
  DC.B  %11011001
  DC.B  %01010000
  DC.B  %00010110
  DC.B  %10010111
  DC.B  %11010000
  DC.B  %11010000
  DC.B  %11010110
  DC.B  %11010110
  DC.B  %00010000
  DC.B  %00111001
  DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
AALogo2
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11011011
  DC.B  %11010010
  DC.B  %11010110
  DC.B  %00010110
  DC.B  %00010110
  DC.B  %10110111
  DC.B  %10110111
  DC.B  %10110110
  DC.B  %10100010
  DC.B  %00110111
  DC.B  %01110111
  DC.B  %01111111
  DC.B  %01111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
AALogo3
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %00100110
  DC.B  %00000110
  DC.B  %11010110
  DC.B  %11010110
  DC.B  %00010110
  DC.B  %00010110
  DC.B  %11010010
  DC.B  %11010000
  DC.B  %00110100
  DC.B  %00111111
  DC.B  %11111111
  DC.B  %11111110
  DC.B  %11111110
  DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
; DC.B  %11111111
AALogo4
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111110
  DC.B  %11111110
  DC.B  %11111111
  DC.B  %10111011
  DC.B  %10111011
  DC.B  %10111010
  DC.B  %10000010
  DC.B  %10000010
  DC.B  %11010110
  DC.B  %11010110
  DC.B  %11010110
  DC.B  %11010111
  DC.B  %11000111
  DC.B  %11101111
  DC.B  %11101111
  DC.B  %11101111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
  DC.B  %11111111
EndAALogoB
  if (>AALogoB != >EndAALogoB)
    echo "WARNING: AALogoB crosses a page boundary!"
  endif

  DC.B  "ERIC8"

  echo "----",($FFF4 - *) , "bytes left (BANK1 - COLLISION DETECTION)"

  ORG     $1FF4
  RORG    $FFF4
  DC.B    "BANK1", 0, 0, 0
  DC.W    Init1, Init1

