; -----------------------------------------------------------------------------
; Star Castle 2600 - Copyright (C) 2012 Chris Walton (cd-w) <cwalton@gmail.com>
; Bank 2 - Main Kernel
; -----------------------------------------------------------------------------

  ; Bank 2
  SEG     BANK2
  ORG     $2000
  RORG    $F000

  ; Space for RAM+
  DS.B    512,35

Init2
  nop     $FFFB             ; Switch to Bank 6
EnterRotate
  nop     $FFF8             ; Switch to Bank 3
  ; jmp     PlayerSetup

; -----------------------------------------------------------------------------
; Setup Player Sprite (Ship & Projectile)
; -----------------------------------------------------------------------------

PlayerSetup
  ; Show Projectile On Alternate Frames If Visible
  lda PROJDIR               ; [0] + 3
  bmi ShowShip              ; [3] + 2/3
  lda CYCLE                 ; [5] + 3
  lsr                       ; [8] + 2
  bcs FlickerShip           ; [10] + 2/3
  
  ; Set Projectile Colour
  lda #$0E                  ; [12] + 2
  sta COLUP0                ; [14] + 3
  
  ; Set Projectile Pointers
  lda PROJX_HI              ; [17] + 3
  sta TEMPX                 ; [20] + 3
  lda PROJY_HI              ; [23] + 3
  sta TEMPY                 ; [26] + 3
  sta TEMP2                 ; [29] + 3
  sec                       ; [32] + 2
  ldx PROJFRAME             ; [34] + 3
  lda ProjPtrTable,X        ; [37] + 4
  sbc TEMPY                 ; [41] + 3
  sta PPTR                  ; [44] + 3
  sta PPTR+2                ; [47] + 3
  lda #>Projectile0         ; [50] + 2
  sta PPTR+1                ; [52] + 3
  
  ; Update Projectile Frame (repeat final 4 frames)
  dex                       ; [55] + 2
  bpl StoreProjFrame        ; [57] + 2/3
WrapProjFrame
  ldx #3                    ; [59] + 2
StoreProjFrame
  stx PROJFRAME             ; [61] + 3
  jmp EndShipPtrs           ; [64] + 3        = 67

; -----------------------------------------------------------------------------

FlickerShip
  ; Lighten Ship Colour
  ldx COLMODE               ; [13] + 3
  lda ShipColsLight,X       ; [16] + 4
  jmp StoreShipColour       ; [20] + 3

ShowShip
  ; Normal Ship Colour
  ldx COLMODE               ; [6] + 3
  lda ShipCols,X            ; [9] + 4

StoreShipColour
  sta COLUP0                ; [23] + 3

  ; Check For Explosion
  lda SHIPS                 ; [26] + 3
  bpl ShowShipFrame         ; [29] + 2/3

  ; Explosion Colour
  lda #$0E                  ; [31] + 2
  sta COLUP0                ; [33] + 3

  ; Ship Explosion X Offset
  sec                       ; [36] + 2
  lda SHIPX_HI              ; [38] + 3
  sbc #4                    ; [41] + 2
  cmp #160                  ; [43] + 2
  bcc StoreShipXOffsetA     ; [45] + 2/3
  ; clc
  adc #160-1                ; [47] + 2      -1 for carry set
StoreShipXOffsetA
  sta TEMPX                 ; [49] + 3

  ; Ship Explosion Y Offset
  sec                       ; [52] + 2
  lda SHIPY_HI              ; [54] + 3
  sbc #4                    ; [57] + 2
  cmp #YSTART               ; [59] + 2
  bcc StoreShipYOffsetA     ; [61] + 2/3
  ; clc
  adc #YSTART-1             ; [63] + 2      -1 for carry set
StoreShipYOffsetA
  sta TEMPY                 ; [65] + 3

  ; Ship Explosion Wrap
  ; lda TEMPY
  cmp #(YSTART-8)           ; [68] + 2
  bcc EndShipWrapA          ; [70] + 2/3
  ; sec
  sbc #YSTART               ; [72] + 2
EndShipWrapA
  sta TEMP2                 ; [74] + 3

  ; Ship Explosion Pointers
  lda SHIPDIR_HI            ; [77] + 3
  lsr                       ; [80] + 2
  lsr                       ; [82] + 2
  tax                       ; [84] + 2
  sec                       ; [86] + 2
  lda ShipExplodePtrLo,X    ; [88] + 4
  sbc TEMPY                 ; [92] + 3
  sta PPTR                  ; [95] + 3
  ldy ShipExplodePtrHi,X    ; [98] + 4
  lda ShipExplodePtrLo,X    ; [102] + 4
  jmp StoreShipPtrs         ; [106] + 3     = 109

; -----------------------------------------------------------------------------

ShowShipFrame
  ; Calculate Ship X Offset
  sec                       ; [32] + 2
  ldx SHIPDIR_HI            ; [34] + 3
  lda SHIPX_HI              ; [37] + 3
  sbc ShipXOffset,X         ; [40] + 4
  cmp #160                  ; [44] + 2
  bcc StoreShipXOffsetB     ; [46] + 2/3
  ; clc
  adc #160-1                ; [48] + 2      -1 for carry set
StoreShipXOffsetB
  sta TEMPX                 ; [50] + 3

  ; Calculate Ship Y Offset
  sec                       ; [53] + 2
  lda SHIPY_HI              ; [55] + 3
  sbc ShipYOffset,X         ; [58] + 4
  cmp #YSTART               ; [62] + 2
  bcc StoreShipYOffsetB     ; [64] + 2/3
  ; clc
  adc #YSTART-1             ; [66] + 2      -1 for carry set
StoreShipYOffsetB
  sta TEMPY                 ; [68] + 3

  ; Ship Wrap
  ; lda TEMPY
  cmp #(YSTART-8)           ; [71] + 2
  bcc EndShipWrapB          ; [73] + 2/3
  ; sec
  sbc #YSTART               ; [75] + 2
EndShipWrapB
  sta TEMP2                 ; [77] + 3

  ; Ship Pointers
  ; ldx SHIPDIR_HI
  lda SWCHA                 ; [80] + 3
  and #%00010000            ; [83] + 2
  bne NoShipFlame           ; [85] + 2/3
  lda CYCLE                 ; [87] + 3
  lsr                       ; [90] + 2
  bcc NoShipFlame           ; [92] + 2/3
ShipFlame
  ; Show Flame Frames
  ; sec
  lda ShipFlamePtrLo,X      ; [94] + 4
  sbc TEMPY                 ; [98] + 3
  sta PPTR                  ; [101] + 3
  lda ShipFlamePtrLo,X      ; [104] + 4
  ldy ShipFlamePtrHi,X      ; [108] + 4
  jmp StoreShipPtrs         ; [112] + 3     = 115
NoShipFlame
  ; Show Normal Frames
  sec                       ; [95] + 2
  lda ShipPtrLo,X           ; [97] + 4
  sbc TEMPY                 ; [101] + 3
  sta PPTR                  ; [104] + 3
  lda ShipPtrLo,X           ; [107] + 4
  ldy ShipPtrHi,X           ; [111] + 4
StoreShipPtrs
  sty PPTR+1                ; [115] + 3
  sec                       ; [118] + 2
  sbc TEMP2                 ; [120] + 3
  sta PPTR+2                ; [123] + 3
EndShipPtrs                 ; WORST = 126 CYCLES

; -----------------------------------------------------------------------------

  ; Player Masks
  sec                       ; [0] + 2
  lda #<PlayerMask          ; [2] + 2
  sbc TEMPY                 ; [4] + 3
  sta PMASKPTR              ; [7] + 3
  sec                       ; [10] + 2
  lda #<PlayerMask          ; [12] + 2
  sbc TEMP2                 ; [14] + 3
  sta PMASKPTR+2            ; [17] + 3
  lda #>PlayerMask          ; [20] + 2
  sta PMASKPTR+1            ; [22] + 3

  ; Position Player 0
  ldx #0                    ; [25] + 2
  lda TEMPX                 ; [27] + 3
  jsr XPosition             ; [30] + 6
  jmp BulletSetup
  
  DC.B  "DUCKY"

; -----------------------------------------------------------------------------
; Setup Missile Sprite (Bullets)
; -----------------------------------------------------------------------------

  ALIGN 256

BulletSetup
  ; Show/Hide Bullets
  ldy BPOS                  ; [0] + 3
  lda BULLETY_HI,Y          ; [3] + 4
  ldx BULLETDIR,Y           ; [7] + 4
  bpl StoreBulletY          ; [13] + 2/3
HideBulletY
  lda #YSTART               ; [15] + 2
StoreBulletY
  sta TEMP2                 ; [17] + 3
  sec                       ; [20] + 2
  lda #<BulletData          ; [22] + 2
  sbc TEMP2                 ; [24] + 3
  sta MPTR                  ; [27] + 3
  lda #>BulletData          ; [30] + 2
  sta MPTR+1                ; [32] + 3

  ; Position Missile 0 (Bullet)
  lda BULLETDIR,Y           ; [35] + 4
  bpl ShowBulletX           ; [39] + 2/3
  lda #0                    ; [42] + 2
  beq PositionBullet        ; [44] + 3
ShowBulletX
  lda BULLETX_HI,Y          ; [42] + 4
PositionBullet
  ldx #2                    ; [47] + 2
  jsr XPosition             ; [49] + 6

; -----------------------------------------------------------------------------
; Setup Ball Sprite (Mines)
; -----------------------------------------------------------------------------

  ; Hide Mines During Explosion
  lda MODE                  ; [0] + 3
  cmp #%11100000            ; [3] + 2
  bcs HideMine              ; [5] + 2/3

  ; Hide Mines In State 0
  ldy MPOS                  ; [7] + 3
  lda MINEDIR_R,Y           ; [10] + 4
  and #%11100000            ; [14] + 2
  bne ShowMine              ; [16] + 2/3

HideMine
  sec                       ; [18] + 2
  lda #<MineData            ; [20] + 2
  sbc #YSTART               ; [22] + 2
  sta BPTR                  ; [24] + 3
  lda #>MineData            ; [27] + 2
  jmp EndMines              ; [29] + 3
ShowMine
  sec                       ; [19] + 2
  lda #<MineData            ; [21] + 2
  sbc MINEY_HI_R,Y          ; [23] + 4
  sta BPTR                  ; [27] + 3
  lda #>MineData            ; [30] + 2
EndMines
  sta BPTR+1                ; [32] + 3

  ; Position Ball (Mine)
  ldx #4                    ; [35] + 2
  lda MINEX_HI_R,Y          ; [37] + 4
  jsr XPosition             ; [41] + 6
  jmp KernelSetup

; -----------------------------------------------------------------------------

  PROJDATA0

; -----------------------------------------------------------------------------
; Setup Kernel Variables
; -----------------------------------------------------------------------------

KernelSetup
  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3

  ; Set Sprite Colours
  ldx COLMODE               ; [3] + 3
  lda StarCols2,X           ; [6] + 4
  sta COLUP1                ; [10] + 3

  ; Set 1 Copy Of Player 1
  lda #0                    ; [13] + 2
  sta NUSIZ1                ; [15] + 3
  
  ; Hide Missiles
  sta ENAM0                 ; [18] + 3
  sta ENAM1                 ; [21] + 3

  ; Set Missile Size
  lda #%00010000            ; [24] + 2
  sta CTRLPF                ; [26] + 3
  
  ; Clear Collision Registers
  sta CXCLR                 ; [29] + 3

  ; Set Mine Colour
  ldy MPOS                  ; [32] + 3
  lda MINECOL,Y             ; [35] + 4
  sta COLUPF                ; [39] + 3

  ; Reset Sprite Positions
  lda SF2
  sta AUDF1

  SLEEP 5                   ; [42] + 11
  sta HMCLR                 ; [53] + 3  > 24

  ; Start Kernel
  jmp TopKernel             ; [56] + 3

  ; Ship Rotation Offsets
ShipYOffset
  DC.B  4,4,4,4,4,4,5,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,5,4,4,4,4,4
EndShipYOffset
  if (>ShipYOffset != >EndShipYOffset)
    echo "WARNING: ShipYOffset crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------
; Top Kernels
; -----------------------------------------------------------------------------

  ALIGN 256

TopKernel
  sta RESM1                 ; [59] + 3    = 62
  
  ; Begin Frame
WaitVblank
  lda INTIM
  bne WaitVblank
  sta VBLANK

  ; Set Line Counters
  ldy #(YSTART - 1)
  ldx #(YOFFSET - 1)

  ; Alternate Kernel Depending on Frame
  lda CYCLE
  lsr
  bcs TopKernelA
  jmp TopKernelB

; -----------------------------------------------------------------------------

TopKernelA
TopKernelLoopA
  sta WSYNC                 ; [0]
  
  ; Draw Player
  lda (PPTR),Y              ; [0] + 5
  and (PMASKPTR),Y          ; [5] + 5
  sta GRP0                  ; [10] + 3    > 75 < 26
  
  ; Draw Bullets
  lda (MPTR),Y              ; [13] + 5
  sta ENAM0                 ; [18] + 3

  ; Draw Stars
  lda TopStarsA,X           ; [21] + 4
  sta ENAM1                 ; [25] + 3
  sta HMM1                  ; [28] + 3
  lda #0                    ; [31] + 2
  dex                       ; [33] + 2

  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3
  
  ; Clear Stars
  sta ENAM1                 ; [3] + 3
  
  ; Draw Mines
  lda (BPTR),Y              ; [6] + 5
  sta ENABL                 ; [11] + 3    > 75 < 26

  dey                       ; [14] + 2
  cpy #(YOFFSET + YRINGS)   ; [16] + 2
  bcs TopKernelLoopA        ; [18] + 2/3
  
  nop                       ; [20] + 2
  jmp StartKernelA          ; [22] + 3    = 25

; -----------------------------------------------------------------------------

TopKernelB
TopKernelLoopB
  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3

  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26
  
  ; Draw Bullets
  lda (MPTR),Y              ; [16] + 5
  sta ENAM0                 ; [21] + 3
  
  ; Clear Stars
  lda #0                    ; [24] + 2
  sta ENAM1                 ; [26] + 3

  ; Load Star Data
  lda TopStarsB,X           ; [29] + 4
  sta HMM1                  ; [33] + 3
  
  ; Draw Stars
  sta WSYNC                 ; [0]
  sta ENAM1                 ; [0] + 3
  
  ; Draw Mines
  lda (BPTR),Y              ; [3] + 5
  sta ENABL                 ; [8] + 3    > 75 < 26

  dex                       ; [11] + 2
  dey                       ; [13] + 2
  cpy #(YOFFSET + YRINGS)   ; [15] + 2
  bcs TopKernelLoopB        ; [17] + 2/3
  sta HMCLR                 ; [19] + 3
  jmp StartKernelB          ; [22] + 3    = 25

; -----------------------------------------------------------------------------

  SHIPDATA0

; -----------------------------------------------------------------------------
; Bottom Kernels
; -----------------------------------------------------------------------------

BottomKernelA
BottomKernelLoopA
  sta WSYNC                 ; [0]

  ; Draw Player
  lda (PPTR),Y              ; [0] + 5
  and (PMASKPTR),Y          ; [5] + 5
  sta GRP0                  ; [10] + 3    > 75 < 26

  ; Draw Bullets
  lda (MPTR),Y              ; [13] + 5
  sta ENAM0                 ; [18] + 3

  ; Draw Stars
  lda BottomStarsA,X        ; [21] + 4
  sta ENAM1                 ; [25] + 3
  sta HMM1                  ; [28] + 3
  lda #0                    ; [31] + 2
  dex                       ; [33] + 2

  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3
  
  ; Clear Stars
  sta ENAM1                 ; [3] + 3

  ; Draw Mines
  lda (BPTR),Y              ; [6] + 5
  sta ENABL                 ; [11] + 3    > 75 < 26
 
  ; Loop
  dey                       ; [14] + 2
  bpl BottomKernelLoopA     ; [16] + 2/3
  bmi EndBottomKernel
  
BottomKernelB
BottomKernelLoopB
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3

  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26

  ; Draw Bullets
  lda (MPTR),Y              ; [16] + 5
  sta ENAM0                 ; [21] + 3

  ; Clear Stars
  lda #0                    ; [24] + 2
  sta ENAM1                 ; [26] + 3
  
  ; Load Star Data
  lda BottomStarsB,X        ; [29] + 4
  sta HMM1                  ; [33] + 3

  sta WSYNC                 ; [0]
  sta ENAM1                 ; [0] + 3

  ; Draw Mines
  lda (BPTR),Y              ; [3] + 5
  sta ENABL                 ; [8] + 3    > 75 < 26
 
  ; Loop
  dex                       ; [11] + 2
  dey                       ; [13] + 2
  bpl BottomKernelLoopB     ; [15] + 2/3

EndBottomKernel

  lda SF1
  sta AUDF1

  ; Begin Overscan
  lda #2
  sta WSYNC                 ; [0]
  sta VBLANK
  lda #OVERDELAY
  sta TIM64T

  ; Clear Sprites
  ; ldx #0
  inx
  stx ENAM1
  stx ENAM0
  stx ENABL
  stx GRP0
  stx GRP1

  jmp EnterRotate

  ; DC.B  "DUCK0"

; -----------------------------------------------------------------------------
; Kernel A
; -----------------------------------------------------------------------------

  ALIGN 256

StartKernelA
  ; Set 3 Copies of Player 1
  lda #%00000011            ; [25] + 2
  sta NUSIZ1                ; [27] + 3

  ; Copy Player Pointer
  lda PPTR+2                ; [30] + 3
  sta PPTR+0                ; [33] + 3

  ; Position Player 1
  SLEEP 4                   ; [36] + 4
  sta RESP1                 ; [40] + 3    = 43
  lda #%10100000            ; [43] + 2    (-2)
  sta HMP1                  ; [45] + 3
  lda #%10000000            ; [48] + 2    (0)
  sta HMP0                  ; [50] + 3
  sta HMM0                  ; [53] + 3
  sta HMM1                  ; [56] + 3
  sta HMBL                  ; [59] + 3
  
  ; Copy Mask Pointer
  lda PMASKPTR+2            ; [62] + 3
  sta PMASKPTR+0            ; [65] + 3
  
  ; Move Sprites
  SLEEP 3                   ; [68] + 3
  sta HMOVE                 ; [71] + 3    = 73/74
  SLEEP 3                   ; [74] + 3

; -----------------------------------------------------------------------------

KernelLoop0A
  ; Draw Player
  lda (PPTR),Y              ; [1] + 5
  and (PMASKPTR),Y          ; [6] + 5
  sta GRP0                  ; [11] + 3    > 75 < 26

  ; Draw Bullets (Bit 7 Must Be Set)
  lax (MPTR),Y              ; [14] + 5
  stx ENAM0                 ; [19] + 3    > 75 < 26

  ; Set Colour
  lda RINGCOL0              ; [22] + 3
  sta COLUP1                ; [25] + 3    < 44

  ; Set Position
  sta HMCLR                 ; [28] + 3    > 24
  stx HMP1                  ; [31] + 3    > 24
  
  ; Draw Rings
  ldx #0                    ; [34] + 2
  stx GRP1                  ; [36] + 3    < 44
  lda (LINE2),Y             ; [39] + 5
  sta GRP1                  ; [44] + 3    > 46 < 50
  SLEEP 3                   ; [47] + 3
  stx GRP1                  ; [50] + 3    >= 52 < 55

  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4

  ; Draw Mines
  lax (BPTR),Y              ; [3] + 5
  stx ENABL                 ; [8] + 3     > 75 < 26
  
  ; Set Sprite Positions
  SLEEP 11                  ; [11] + 11
  sta HMCLR                 ; [22] + 3    > 24
  stx HMP0                  ; [25] + 3    > 24
  stx HMM0                  ; [28] + 3    > 24
  stx HMM1                  ; [31] + 3    > 24
  stx HMBL                  ; [34] + 3    > 24
  SLEEP 26                  ; [37] + 26
  
  dey                       ; [63] + 2
  cpy #(YOFFSET + 35)       ; [65] + 2
  bcc StartKernel1A         ; [67] + 2/3
  
  ; Move Sprites
  nop                       ; [69] + 2
  sta HMOVE                 ; [71] + 3    = 73/74

  ; Loop
  bcs KernelLoop0A          ; [74] + 2/3

StartKernel1A
  sta HMOVE                 ; [70] + 3    = 73/74
  jmp KernelLoop1A          ; [73] + 3

EndKernel0A
  if (>StartKernelA != >EndKernel0A)
    echo "WARNING: Kernel0A crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  SHIPDATA1

; -----------------------------------------------------------------------------

KernelLoop1A
  ; Draw Player
  lda (PPTR),Y              ; [0] + 5
  and (PMASKPTR),Y          ; [5] + 5
  sta GRP0                  ; [10] + 3    > 75 < 26

  ; Draw Bullets
  lda (MPTR),Y              ; [13] + 5
  sta ENAM0                 ; [18] + 3

  ; Set Colour
  lda RINGCOL0              ; [21] + 3
  sta COLUP1                ; [24] + 3    < 44

  ; Draw Rings & Set Sprite Positions
  sta HMCLR                 ; [27] + 3    > 24
  lda (LINE0),Y             ; [30] + 5
  sta GRP1                  ; [35] + 3    < 44
  lax (LINE4),Y             ; [38] + 5
  lda #0                    ; [43] + 2
  sta GRP1                  ; [45] + 3    > 46 < 50
  lda #%10000000            ; [48] + 2
  stx GRP1                  ; [50] + 3    >= 52 < 55
  sta HMP1                  ; [53] + 3    > 24

  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4

  ; Draw Mines
  lax (BPTR),Y              ; [3] + 5
  stx ENABL                 ; [8] + 3     > 75 < 26
  
  ; Set Colour
  lda RINGCOL1              ; [11] + 3
  sta COLUP1                ; [14] + 3    <  44

  ; Draw Rings
  lda (LINE1),Y             ; [17] + 5
  sta GRP1                  ; [22] + 3    < 47
  
  ; Set Sprite Positions
  sta HMCLR                 ; [25] + 3    > 24
  stx HMP0                  ; [28] + 3    > 24
  stx HMM0                  ; [31] + 3    > 24
  stx HMM1                  ; [34] + 3    > 24
  stx HMBL                  ; [37] + 3    > 24
  
  ; Draw Rings
  nop                       ; [40] + 2
  lda (LINE3),Y             ; [42] + 5
  sta GRP1                  ; [47] + 3    > 49 < 52
  lda #0                    ; [50] + 2
  sta GRP1                  ; [52] + 3    > 54 < 58
  SLEEP 8                   ; [55] + 8

  dey                       ; [63] + 2
  cpy #(YOFFSET + 30)       ; [65] + 2
  bcc StartKernelLoop2A     ; [67] + 2/3
  
  ; Move Sprites
  sta.w HMOVE               ; [69] + 4    = 73/74
  
  ; Loop
  bcs KernelLoop1A          ; [73] + 3

StartKernelLoop2A
  sta HMOVE                 ; [70] + 3    = 73/74
  jmp KernelLoop2A          ; [73] + 3

EndKernel1A
  if (>KernelLoop1A != >EndKernel1A)
    echo "WARNING: Kernel1A crosses a page boundary!"
  endif

  ; DC.B  "DUCK1"

; -----------------------------------------------------------------------------

  ALIGN 256

KernelLoop2A
  ; Draw Player
  lda (PPTR),Y              ; [0] + 5
  and (PMASKPTR),Y          ; [5] + 5
  sta GRP0                  ; [10] + 3    > 75 < 26

  ; Draw Bullets
  lda (MPTR),Y              ; [13] + 5
  sta ENAM0                 ; [18] + 3    > 75 < 26

  ; Set Colour
  lda RINGCOL0              ; [21] + 3
  sta COLUP1                ; [24] + 3    < 44

  ; Draw Rings
  sta HMCLR                 ; [27] + 3    > 24
  lda (LINE0),Y             ; [30] + 5
  sta GRP1                  ; [35] + 3    < 44
  lax (LINE4),Y             ; [38] + 5
  lda #0                    ; [43] + 2
  sta GRP1                  ; [45] + 3    > 46 < 50
  lda #%10000001            ; [48] + 2
  stx GRP1                  ; [50] + 3    >= 52 < 55
  sta HMP1                  ; [53] + 3    > 24

  ; Set 2 copies
  sta NUSIZ1                ; [56] + 3    < 44
  
  ; Set Colour
  lda RINGCOL1              ; [59] + 2
  sta COLUP1                ; [61] + 3    <  44

  ; Store Line Counter
  sty TEMP                  ; [64] + 3

  ; Load Projectile
  lax (BPTR),Y              ; [67] + 5

  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4

  ; Draw Mines
  stx ENABL                 ; [3] + 3     > 75 < 26

  ; Draw Rings
  lda (LINE1),Y             ; [6] + 5
  sta GRP1                  ; [11] + 3    < 47
  lda (LINE3),Y             ; [14] + 5
  
  ; Set Sprite Positions
  SLEEP 4                   ; [19] + 4
  sta HMCLR                 ; [23] + 3    > 24
  stx HMP0                  ; [26] + 3    > 24
  stx HMM0                  ; [29] + 3    > 24
  stx HMM1                  ; [32] + 3    > 24
  stx HMBL                  ; [35] + 3    > 24
  
  ; Draw Rings
  ldx RINGCOL2              ; [38] + 3
  ldy RINGCOL1              ; [41] + 3
  stx.w COLUP1              ; [44] + 4    = 48
  sta GRP1                  ; [48] + 3    > 49 < 52
  sty COLUP1                ; [51] + 3    = 53/54

  ; Restore Counter
  ldy TEMP                  ; [54] + 3

  ; Set 3 Copies
  ldx #%00000011            ; [57] + 2
  stx NUSIZ1                ; [59] + 3    < 44
  nop                       ; [62] + 2
  
  dey                       ; [64] + 2
  cpy #(YOFFSET + 28)       ; [66] + 2
  bcc KernelLoop3A          ; [68] + 2/3
  sta HMOVE                 ; [70] + 3    = 73/74
  bcs KernelLoop2A          ; [73] + 2/3

EndKernel2A
  if (>KernelLoop2A != >EndKernel2A)
    echo "WARNING: Kernel2A crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  SHIPDATA2

; -----------------------------------------------------------------------------

KernelLoop3A
  sta HMOVE                 ; [71] + 3    = 73/74

  ; Draw Player
  lda (PPTR),Y              ; [74] + 5
  and (PMASKPTR),Y          ; [3] + 5
  sta GRP0                  ; [8] + 3    > 75 < 26

  ; Draw Bullets
  lda (MPTR),Y              ; [11] + 5
  sta ENAM0                 ; [16] + 3

  ; Set Colour
  lda RINGCOL0              ; [19] + 3
  sta COLUP1                ; [22] + 3    < 44

  ; Draw Rings
  lda (LINE0),Y             ; [25] + 5
  and #%11111000            ; [30] + 2
  sta GRP1                  ; [32] + 3    < 44
  lax (LINE2),Y             ; [35] + 5
  lda (LINE4),Y             ; [40] + 5
  stx GRP1                  ; [45] + 3    > 46 < 50
  and #%00011111            ; [48] + 2
  sta GRP1                  ; [50] + 3    >= 52 < 55

  ; Move Sprites
  sta HMCLR                 ; [53] + 3
  lda #%10000000            ; [56] + 2
  sta HMP1                  ; [58] + 3    > 24
  
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4

  ; Draw Mines
  lax (BPTR),Y              ; [3] + 5
  stx ENABL                 ; [8] + 3     > 75 < 26
  
  ; Set Colour
  lda RINGCOL2              ; [11] + 3
  sta COLUP1                ; [14] + 3    <  44

  ; Draw Rings
  lda (LINE1),Y             ; [17] + 5
  sta GRP1                  ; [22] + 3    < 47

  ; Set Sprite Positions
  sta HMCLR                 ; [25] + 3    > 24
  stx HMP0                  ; [28] + 3    > 24
  stx HMM0                  ; [31] + 3    > 24
  stx HMM1                  ; [34] + 3    > 24
  stx HMBL                  ; [37] + 3    > 24

  ; Draw Rings
  nop                       ; [40] + 2
  lda (LINE3),Y             ; [42] + 5
  sta GRP1                  ; [47] + 3    > 49 < 52
  lda #0                    ; [50] + 2
  sta GRP1                  ; [52] + 3    > 54 < 58

  SLEEP 9                   ; [55] + 9

  dey                       ; [64] + 2
  cpy #(YOFFSET + 12)       ; [66] + 2
  bcs KernelLoop3A          ; [68] + 2/3
  
  sta HMOVE                 ; [70] + 3    = 73/74
  jmp KernelLoop4A          ; [73] + 3

EndKernel3A
  if (>KernelLoop3A != >EndKernel3A)
    echo "WARNING: Kernel3A crosses a page boundary!"
  endif

  DC.B  "DUCK2"

; -----------------------------------------------------------------------------

  ALIGN 256

KernelLoop4A
  ; Draw Player
  lda (PPTR),Y              ; [0] + 5
  and (PMASKPTR),Y          ; [5] + 5
  sta GRP0                  ; [10] + 3    > 75 < 26

  ; Draw Bullets
  lda (MPTR),Y              ; [13] + 5
  sta ENAM0                 ; [18] + 3    > 75 < 26

  ; Set Colour
  lda RINGCOL0              ; [21] + 3
  sta COLUP1                ; [24] + 3    < 44

  ; Draw Rings
  sta HMCLR                 ; [27] + 3    > 24
  lda (LINE0),Y             ; [30] + 5
  sta GRP1                  ; [35] + 3    < 44
  lax (LINE4),Y             ; [38] + 5
  lda #0                    ; [43] + 2
  sta GRP1                  ; [45] + 3    > 46 < 50
  lda #%10000001            ; [48] + 2
  stx GRP1                  ; [50] + 3    >= 52 < 55
  sta HMP1                  ; [53] + 3    > 24

  ; Set 2 copies
  sta NUSIZ1                ; [56] + 3     < 44
  
  ; Set Colour
  lda RINGCOL1              ; [59] + 2
  sta COLUP1                ; [61] + 3    <  44

  ; Store Line Counter
  sty TEMP                  ; [64] + 3

  ; Load Projectile
  lax (BPTR),Y              ; [67] + 5

  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4

  ; Draw Mines
  stx ENABL                 ; [3] + 3     > 75 < 26

  ; Draw Rings
  lda (LINE1),Y             ; [6] + 5
  sta GRP1                  ; [11] + 3    < 47
  lda (LINE3),Y             ; [14] + 5
  
  ; Set Sprite Positions
  SLEEP 4                   ; [19] + 4
  sta HMCLR                 ; [23] + 3    > 24
  stx HMP0                  ; [26] + 3    > 24
  stx HMM0                  ; [29] + 3    > 24
  stx HMM1                  ; [32] + 3    > 24
  stx HMBL                  ; [35] + 3    > 24
  
  ; Draw Rings
  ldx RINGCOL2              ; [38] + 3
  ldy RINGCOL1              ; [41] + 3
  stx.w COLUP1              ; [44] + 4    = 48
  sta GRP1                  ; [48] + 3    > 49 < 52
  sty COLUP1                ; [51] + 3    = 53/54

  ; Restore Counter
  ldy TEMP                  ; [54] + 3

  ; Set 3 Copies
  ldx #%10000011            ; [57] + 2
  stx NUSIZ1                ; [59] + 3    < 44
  nop                       ; [62] + 2
  
  dey                       ; [64] + 2
  cpy #(YOFFSET + 10)       ; [66] + 2
  bcc StartKernel5A         ; [68] + 2/3

  sta HMOVE                 ; [70] + 3    = 73/74
  bcs KernelLoop4A          ; [73] + 2/3

StartKernel5A
  sta HMOVE                 ; [71] + 3    = 73/74
  jmp KernelLoop5A          ; [74] + 3

EndKernel4A
  if (>KernelLoop4A != >EndKernel4A)
    echo "WARNING: Kernel4A crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  SHIPDATA3

; -----------------------------------------------------------------------------

KernelLoop5A
  ; Draw Player
  lda (PPTR),Y              ; [1] + 5
  and (PMASKPTR),Y          ; [6] + 5
  sta GRP0                  ; [11] + 3    > 75 < 26

  ; Draw Bullets
  lda (MPTR),Y              ; [14] + 5
  sta ENAM0                 ; [19] + 3

  ; Set Colour
  lda RINGCOL0              ; [22] + 3
  sta COLUP1                ; [25] + 3    < 44

  ; Darw Rings & Set Sprite Positions
  sta HMCLR                 ; [28] + 3    > 24
  lda (LINE0),Y             ; [31] + 5
  sta GRP1                  ; [36] + 3    < 44
  lax (LINE4),Y             ; [39] + 5
  lda #0                    ; [44] + 2
  sta GRP1                  ; [46] + 3    > 46 < 50
  lda #%10000000            ; [49] + 2
  stx GRP1                  ; [51] + 3    >= 52 < 55
  sta HMP1                  ; [54] + 3    > 24

  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4

  ; Draw Mines
  lax (BPTR),Y              ; [3] + 5
  stx ENABL                 ; [8] + 3     > 75 < 26
  
  ; Set Colour
  lda RINGCOL1              ; [11] + 3
  sta COLUP1                ; [14] + 3    <  44

  ; Draw Rings
  lda (LINE1),Y             ; [17] + 5
  sta GRP1                  ; [22] + 3    < 47
  
  ; Set Sprite Positions
  sta HMCLR                 ; [25] + 3    > 24
  stx HMP0                  ; [28] + 3    > 24
  stx HMM0                  ; [31] + 3    > 24
  stx HMM1                  ; [34] + 3    > 24
  stx HMBL                  ; [37] + 3    > 24
  
  ; Draw Rings
  nop                       ; [40] + 2
  lda (LINE3),Y             ; [42] + 5
  sta GRP1                  ; [47] + 3    > 49 < 52
  lda #0                    ; [50] + 2
  sta GRP1                  ; [52] + 3    > 54 < 58
  SLEEP 9                   ; [55] + 9

  dey                       ; [64] + 2
  cpy #(YOFFSET + 5)        ; [66] + 2
  bcc StartKernelLoop6A     ; [68] + 2/3

  ; Move Sprites
  sta.w HMOVE               ; [70] + 4    = 73/74
  bcs KernelLoop5A          ; [74] + 3
  
StartKernelLoop6A
  sta HMOVE                 ; [71] + 3    = 73/74
  jmp KernelLoop6A          ; [74] + 3

EndKernel5A
  if (>KernelLoop5A != >EndKernel5A)
    echo "WARNING: Kernel5A crosses a page boundary!"
  endif

  DC.B  "DUCK3"

; -----------------------------------------------------------------------------

  ALIGN 256

KernelLoop6A
  ; Draw Player
  lda (PPTR),Y              ; [1] + 5
  and (PMASKPTR),Y          ; [6] + 5
  sta GRP0                  ; [11] + 3    > 75 < 26

  ; Draw Bullets (Bit 7 Must Be Set)
  lax (MPTR),Y              ; [14] + 5
  stx ENAM0                 ; [19] + 3    > 75 < 26

  ; Set Colour
  lda RINGCOL0              ; [22] + 3
  sta COLUP1                ; [25] + 3    < 44

  ; Set Position
  sta HMCLR                 ; [28] + 3    > 24
  stx HMP1                  ; [31] + 3    > 24
  
  ; Draw Rings
  ldx #0                    ; [34] + 2
  stx GRP1                  ; [36] + 3    < 44
  lda (LINE2),Y             ; [39] + 5
  sta GRP1                  ; [44] + 3    > 46 < 50
  SLEEP 3                   ; [47] + 3
  stx GRP1                  ; [50] + 3    >= 52 < 55

  ; Move Sprites
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4

  ; Draw Mines
  lax (BPTR),Y              ; [3] + 5
  stx ENABL                 ; [8] + 3     > 75 < 26
  
  ; Set Sprite Positions
  SLEEP 11                  ; [11] + 11
  sta HMCLR                 ; [22] + 3    > 24
  stx HMP0                  ; [25] + 3    > 24
  stx HMM0                  ; [28] + 3    > 24
  stx HMM1                  ; [31] + 3    > 24
  stx HMBL                  ; [34] + 3    > 24

  dey                       ; [37] + 2
  cpy #(YOFFSET + 0)        ; [39] + 2
  bcc StartBottom           ; [41] + 2/3
  
  SLEEP 28                  ; [43] + 28
  
  ; Move Sprites
  sta HMOVE                 ; [71] + 3    = 73/74

  ; Loop
  bcs KernelLoop6A          ; [74] + 2/3
  
StartBottom
  ldx COLMODE               ; [44] + 3
  lda StarCols2,X           ; [47] + 4
  sta COLUP1                ; [51] + 3
  lda #0                    ; [54] + 2
  sta NUSIZ1                ; [56] + 3
  ldx #(YOFFSET - 1)        ; [59] + 2
  sta HMCLR                 ; [61] + 3
  jmp BottomKernelA         ; [64] + 3

EndKernel6A
  if (>KernelLoop6A != >EndKernel6A)
    echo "WARNING: Kernel6A crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  SHIPDATA4

; -----------------------------------------------------------------------------
; Kernel B
; -----------------------------------------------------------------------------

StartKernelB
  ; Set 3 Copies of Player 1
  lda #%00000011            ; [25] + 2
  sta NUSIZ1                ; [27] + 3

  ; Copy Player Pointer
  lda PPTR+2                ; [30] + 3
  sta PPTR+0                ; [33] + 3
  
  ; Copy Mask Pointer
  lda PMASKPTR+2            ; [36] + 3
  sta PMASKPTR+0            ; [39] + 3

  ; Position Player 1
  sta RESP1                 ; [42] + 3    = 45

KernelLoop0B
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4
  
  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26

  ; Draw Bullets
  lax (MPTR),Y              ; [16] + 5
  stx ENAM0                 ; [21] + 3    > 75 < 26

  ; Set Start Colour
  lda RINGCOL0              ; [24] + 3
  sta COLUP1                ; [27] + 3    < 44

  ; Rings
  lda (LINE1),Y             ; [30] + 5
  sta GRP1                  ; [35] + 3    < 47
  SLEEP 3                   ; [38] + 3

  ; Draw Rings
  lda (LINE3),Y             ; [41] + 5
  sta GRP1                  ; [46] + 3    >= 49 < 52
  sta HMCLR                 ; [49] + 3    > 24
  lda #0                    ; [52] + 2
  sta GRP1                  ; [54] + 3    > 54 < 58
  stx HMBL                  ; [57] + 3    > 24
  stx HMM0                  ; [60] + 3    > 24
  stx HMM1                  ; [63] + 3    > 24
  stx HMP0                  ; [66] + 3    > 24
  
  ; Move Sprites
  nop                       ; [69] + 2
  sta HMOVE                 ; [71] + 3    = 73/74
  
  ; Draw Mines
  lax (BPTR),Y              ; [74] + 5
  stx ENABL                 ; [3] + 3     > 75 < 26

  ; Position Sprites
  SLEEP 16                  ; [6] + 16
  sta HMCLR                 ; [22] + 3    > 24
  stx HMP1                  ; [25] + 3    > 24

  ; Loop
  dey                       ; [28] + 2
  cpy #(YOFFSET + 35)       ; [30] + 2
  bcs KernelLoop0B          ; [32] + 2/3
  jmp KernelLoop1B          ; [34] + 3

EndKernel0B
  if (>StartKernelB != >EndKernel0B)
    echo "WARNING: Kernel0B crosses a page boundary!"
  endif

  DC.B  "DUCK4"

; -----------------------------------------------------------------------------

  ALIGN 256

KernelLoop1B
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4
  
  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26

  ; Draw Bullets
  lax (MPTR),Y              ; [16] + 5
  stx ENAM0                 ; [21] + 3    > 75 < 26

  ; Clear Rings
  lda #0                    ; [24] + 2
  sta GRP1                  ; [26] + 3    > 54 < 58

  ; Draw Rings
  sta HMCLR                 ; [29] + 3    > 24
  stx HMBL                  ; [32] + 3    > 24
  stx HMM0                  ; [35] + 3    > 24
  stx HMM1                  ; [38] + 3    > 24
  stx HMP0                  ; [41] + 3    > 24
  
  ; Move Sprites
  SLEEP 27                  ; [44] + 27
  sta HMOVE                 ; [71] + 3    = 73/74
  
  ; Draw Mines
  lax (BPTR),Y              ; [74] + 5
  stx ENABL                 ; [3] + 3     > 75 < 26
  
  ; Set Start Colour
  lda RINGCOL1              ; [6] + 3
  sta COLUP1                ; [9] + 3    < 44
 
  ; Position Sprites
  SLEEP 17                  ; [12] + 17
  sta HMCLR                 ; [29] + 3    > 24
  stx HMP1                  ; [32] + 3    > 24

  ; Clear Rings
  ldx #0                    ; [35] + 2
  stx GRP1                  ; [37] + 3    < 44
  lda (LINE2),Y             ; [40] + 5
  sta GRP1                  ; [45] + 3    > 46 < 50
  dey                       ; [48] + 2
  stx GRP1                  ; [50] + 3    >= 52 < 55

  ; Loop
  cpy #(YOFFSET + 30)       ; [53] + 2
  bcs KernelLoop1B          ; [55] + 2/3
  jmp KernelLoop2B

EndKernel1B
  if (>KernelLoop1B != >EndKernel1B)
    echo "WARNING: Kernel1B crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

ShipCols
  DC.B  $88, $B8, $08       ; NTSC/PAL60/B&W
ShipColsLight
  DC.B  $8A, $BA, $0A       ; NTSC/PAL60/B&W
StarCols2
  DC.B  $84, $B4, $04       ; NTSC/PAL60/B&W

  SHIPDATA5
  
; -----------------------------------------------------------------------------

KernelLoop2B
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4
  
  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26

  ; Draw Bullets
  lax (MPTR),Y              ; [16] + 5
  stx ENAM0                 ; [21] + 3    > 75 < 26

  ; Empty Rings
  lda #0                    ; [24] + 2
  sta GRP1                  ; [26] + 3    < 47

  ; Set Sprite Positions
  sta HMCLR                 ; [29] + 3    > 24
  stx HMP0                  ; [32] + 3    > 24
  stx HMM0                  ; [35] + 3    > 24
  stx HMBL                  ; [38] + 3    > 24
  stx HMM1                  ; [41] + 3    > 24
  
  ; Move Sprites
  SLEEP 26                  ; [44] + 26
  sta HMOVE                 ; [70] + 3    = 73

  ; Draw Mines
  lax (BPTR),Y              ; [73] + 5
  stx ENABL                 ; [2] + 3     > 75 < 26
  
  ; Set Colour
  lda RINGCOL2              ; [5] + 3
  sta COLUP1                ; [8] + 3     < 44

  ; Position Sprites
  SLEEP 17                  ; [11] + 17
  sta HMCLR                 ; [28] + 3    > 24
  stx HMP1                  ; [31] + 3    > 24

  ; Draw Rings
  ldx #0                    ; [34] + 2
  stx GRP1                  ; [36] + 3    < 44
  lda (LINE2),Y             ; [39] + 5
  sta GRP1                  ; [44] + 3    > 46 < 50
  dey                       ; [47] + 2
  stx GRP1                  ; [49] + 3    >= 52 < 55

  cpy #(YOFFSET + 28)       ; [52] + 2
  bpl KernelLoop2B          ; [54] + 2/3
  jmp KernelLoop3B          ; [56] + 3

EndKernel2B
  if (>KernelLoop2B != >EndKernel2B)
    echo "WARNING: Kernel2B crosses a page boundary!"
  endif

  DC.B  "DUCK5"

; -----------------------------------------------------------------------------

  ALIGN 256

KernelLoop3B
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4

  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26

  ; Draw Bullets
  lax (MPTR),Y              ; [16] + 5
  stx ENAM0                 ; [21] + 3    > 75 < 26

  ; Empty Rings
  lda #0                    ; [24] + 2
  sta GRP1                  ; [26] + 3    < 47

  ; Set Sprite Positions
  stx HMCLR                 ; [29] + 3    > 24
  stx HMP0                  ; [32] + 3    > 24
  stx HMM0                  ; [35] + 3    > 24
  stx HMBL                  ; [38] + 3    > 24
  stx HMM1                  ; [41] + 3    > 24

  ; Move Sprites
  SLEEP 26                  ; [44] + 26
  sta HMOVE                 ; [70] + 3    = 73

  ; Draw Mines
  lda (BPTR),Y              ; [73] + 5
  sta ENABL                 ; [2] + 3    > 75 < 26
  
    ; Set Start Colour
  lda RINGCOL1              ; [5] + 3
  sta COLUP1                ; [8] + 3    < 44
  
  ; Draw Rings
  SLEEP 11                  ; [11] + 11
  lda (LINE0),Y             ; [22] + 5
  and #%00000111            ; [27] + 2
  sta GRP1                  ; [29] + 3    < 44
  lax (LINE2),Y             ; [32] + 5
  lda (LINE4),Y             ; [37] + 5
  and #%11100000            ; [42] + 2
  stx GRP1                  ; [44] + 3    > 46 < 50
  ldx #%10000000            ; [47] + 2
  sta GRP1                  ; [49] + 3    >= 52 < 55
  
  ; Position Sprites
  sta HMCLR                 ; [52] + 3    > 24
  stx HMP1                  ; [55] + 3    > 24

  dey                       ; [58] + 2
  cpy #(YOFFSET + 12)       ; [60] + 2
  bpl KernelLoop3B          ; [62] + 2/3
  jmp KernelLoop4B          ; [64] + 3

EndKernel3B
  if (>KernelLoop3B != >EndKernel3B)
    echo "WARNING: Kernel3B crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  DS.B  4,0
  SHIPDATA6
  
; -----------------------------------------------------------------------------

KernelLoop4B
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4
  
  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26

  ; Draw Bullets
  lax (MPTR),Y              ; [16] + 5
  stx ENAM0                 ; [21] + 3    > 75 < 26

  ; Empty Rings
  lda #0                    ; [24] + 2
  sta GRP1                  ; [26] + 3    < 47

  ; Set Sprite Positions
  sta HMCLR                 ; [29] + 3    > 24
  stx HMP0                  ; [32] + 3    > 24
  stx HMM0                  ; [35] + 3    > 24
  stx HMBL                  ; [38] + 3    > 24
  stx HMM1                  ; [41] + 3    > 24
  
  ; Move Sprites
  SLEEP 26                  ; [44] + 26
  sta HMOVE                 ; [70] + 3    = 73

  ; Draw Mines
  lax (BPTR),Y              ; [73] + 5
  stx ENABL                 ; [2] + 3     > 75 < 26
  
  ; Set Colour
  lda RINGCOL2              ; [5] + 3
  sta COLUP1                ; [8] + 3     < 44

  ; Position Sprites
  SLEEP 17                  ; [11] + 17
  sta HMCLR                 ; [28] + 3    > 24
  stx HMP1                  ; [31] + 3    > 24

  ; Draw Rings
  ldx #0                    ; [34] + 2
  stx GRP1                  ; [36] + 3    < 44
  lda (LINE2),Y             ; [39] + 5
  sta GRP1                  ; [44] + 3    > 46 < 50
  dey                       ; [47] + 2
  stx GRP1                  ; [49] + 3    >= 52 < 55

  cpy #(YOFFSET + 10)       ; [54] + 2
  bpl KernelLoop4B          ; [56] + 2/3
  jmp KernelLoop5B          ; [58] + 3

EndKernel4B
  if (>KernelLoop4B != >EndKernel4B)
    echo "WARNING: Kernel4B crosses a page boundary!"
  endif
  
  DC.B  "DUCK6"
  
; -----------------------------------------------------------------------------

  ALIGN 256

KernelLoop5B
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4
  
  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26

  ; Draw Bullets
  lax (MPTR),Y              ; [16] + 5
  stx ENAM0                 ; [21] + 3    > 75 < 26

  ; Clear Rings
  lda #0                    ; [24] + 2
  sta GRP1                  ; [26] + 3    > 54 < 58

  ; Draw Rings
  sta HMCLR                 ; [29] + 3    > 24
  stx HMBL                  ; [32] + 3    > 24
  stx HMM0                  ; [35] + 3    > 24
  stx HMM1                  ; [38] + 3    > 24
  stx HMP0                  ; [41] + 3    > 24
  
  ; Move Sprites
  SLEEP 27                  ; [44] + 27
  sta HMOVE                 ; [71] + 3    = 73/74
  
  ; Draw Mines
  lax (BPTR),Y              ; [74] + 5
  stx ENABL                 ; [3] + 3     > 75 < 26
  
  ; Set Start Colour
  lda RINGCOL1              ; [6] + 3
  sta COLUP1                ; [9] + 3    < 44
 
  ; Position Sprites
  SLEEP 17                  ; [12] + 17
  sta HMCLR                 ; [29] + 3    > 24
  stx HMP1                  ; [32] + 3    > 24

  ; Clear Rings
  ldx #0                    ; [35] + 2
  stx GRP1                  ; [37] + 3    < 44
  lda (LINE2),Y             ; [40] + 5
  sta GRP1                  ; [45] + 3    > 46 < 50
  dey                       ; [48] + 2
  stx GRP1                  ; [50] + 3    >= 52 < 55

  ; Loop
  cpy #(YOFFSET + 5)        ; [53] + 2
  bcs KernelLoop5B          ; [55] + 2/3
  jmp KernelLoop6B

; -----------------------------------------------------------------------------

  DS.B  7,0
  SHIPDATA7

; -----------------------------------------------------------------------------

KernelLoop6B
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3     > 74 < 4
  
  ; Draw Player
  lda (PPTR),Y              ; [3] + 5
  and (PMASKPTR),Y          ; [8] + 5
  sta GRP0                  ; [13] + 3    > 75 < 26

  ; Draw Bullets
  lax (MPTR),Y              ; [16] + 5
  stx ENAM0                 ; [21] + 3    > 75 < 26

  ; Set Start Colour
  lda RINGCOL0              ; [24] + 3
  sta COLUP1                ; [27] + 3    < 44

  ; Rings
  lda (LINE1),Y             ; [30] + 5
  sta GRP1                  ; [35] + 3    < 47
  SLEEP 3                   ; [38] + 3

  ; Draw Rings
  lda (LINE3),Y             ; [41] + 5
  sta GRP1                  ; [46] + 3    >= 49 < 52
  sta HMCLR                 ; [49] + 3    > 24
  lda #0                    ; [52] + 2
  sta GRP1                  ; [54] + 3    > 54 < 58
  stx HMBL                  ; [57] + 3    > 24
  stx HMM0                  ; [60] + 3    > 24
  stx HMM1                  ; [63] + 3    > 24
  stx HMP0                  ; [66] + 3    > 24
  
  ; Move Sprites
  nop                       ; [69] + 2
  sta HMOVE                 ; [71] + 3    = 73/74
  
  ; Draw Mines
  lax (BPTR),Y              ; [74] + 5
  stx ENABL                 ; [3] + 3     > 75 < 26

  ; Position Sprites
  SLEEP 16                  ; [6] + 16
  sta HMCLR                 ; [22] + 3    > 24
  stx HMP1                  ; [25] + 3    > 24

  ; Loop
  dey                       ; [28] + 2
  cpy #(YOFFSET + 0)        ; [30] + 2
  bcs KernelLoop6B          ; [32] + 2/3

  ; Bottom Kernel
  ldx COLMODE               ; [34] + 3
  lda StarCols2,X           ; [37] + 4
  sta COLUP1                ; [41] + 3
  lda #0                    ; [44] + 2
  sta NUSIZ1                ; [46] + 3
  ldx #(YOFFSET - 1)        ; [49] + 2
  jmp BottomKernelB         ; [51] + 3

  ; DC.B  "DUCK7"

; -----------------------------------------------------------------------------
; Sprite Data
; -----------------------------------------------------------------------------

  ALIGN 256
  
  ; Ship Sprite Pointers
ShipPtrTableA
ShipPtrLo
  DC.B  <Ship0, <Ship1, <Ship2, <Ship3, <Ship4, <Ship5, <Ship6, <Ship7
  DC.B  <Ship8, <Ship9, <Ship10, <Ship11, <Ship12, <Ship13, <Ship14, <Ship15
  DC.B  <Ship16, <Ship17, <Ship18, <Ship19, <Ship20, <Ship21, <Ship22, <Ship23
  DC.B  <Ship24, <Ship25, <Ship26, <Ship27, <Ship28, <Ship29, <Ship30, <Ship31
ShipPtrHi
  DC.B  >Ship0, >Ship1, >Ship2, >Ship3, >Ship4, >Ship5, >Ship6, >Ship7
  DC.B  >Ship8, >Ship9, >Ship10, >Ship11, >Ship12, >Ship13, >Ship14, >Ship15
  DC.B  >Ship16, >Ship17, >Ship18, >Ship19, >Ship20, >Ship21, >Ship22, >Ship23
  DC.B  >Ship24, >Ship25, >Ship26, >Ship27, >Ship28, >Ship29, >Ship30, >Ship31
ShipFlamePtrLo
  DC.B  <ShipFlame0, <ShipFlame1, <ShipFlame2, <ShipFlame3
  DC.B  <ShipFlame4, <ShipFlame5, <ShipFlame6, <ShipFlame7
  DC.B  <ShipFlame8, <ShipFlame9, <ShipFlame10, <ShipFlame11
  DC.B  <ShipFlame12, <ShipFlame13, <ShipFlame14, <ShipFlame15
  DC.B  <ShipFlame16, <ShipFlame17, <ShipFlame18, <ShipFlame19
  DC.B  <ShipFlame20, <ShipFlame21, <ShipFlame22, <ShipFlame23
  DC.B  <ShipFlame24, <ShipFlame25, <ShipFlame26, <ShipFlame27
  DC.B  <ShipFlame28, <ShipFlame29, <ShipFlame30, <ShipFlame31
EndShipPtrTableA
  if (>ShipPtrTableA != >EndShipPtrTableA)
    echo "WARNING: ShipPtrTableA crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  SHIPDATA8

; -----------------------------------------------------------------------------

ShipPtrTableB
ShipFlamePtrHi
  DC.B  >ShipFlame0, >ShipFlame1, >ShipFlame2, >ShipFlame3
  DC.B  >ShipFlame4, >ShipFlame5, >ShipFlame6, >ShipFlame7
  DC.B  >ShipFlame8, >ShipFlame9, >ShipFlame10, >ShipFlame11
  DC.B  >ShipFlame12, >ShipFlame13, >ShipFlame14, >ShipFlame15
  DC.B  >ShipFlame16, >ShipFlame17, >ShipFlame18, >ShipFlame19
  DC.B  >ShipFlame20, >ShipFlame21, >ShipFlame22, >ShipFlame23
  DC.B  >ShipFlame24, >ShipFlame25, >ShipFlame26, >ShipFlame27
  DC.B  >ShipFlame28, >ShipFlame29, >ShipFlame30, >ShipFlame31
ShipExplodePtrLo
  DC.B  <ShipExplode10, <ShipExplode9, <ShipExplode8, <ShipExplode7
  DC.B  <ShipExplode6, <ShipExplode5, <ShipExplode4, <ShipExplode3
  DC.B  <ShipExplode2, <ShipExplode1, <ShipExplode0
ShipExplodePtrHi
  DC.B  >ShipExplode10, >ShipExplode9, >ShipExplode8, >ShipExplode7
  DC.B  >ShipExplode6, >ShipExplode5, >ShipExplode4, >ShipExplode3
  DC.B  >ShipExplode2, >ShipExplode1, >ShipExplode0
EndShipPtrTableB
  if (>ShipPtrTableB != >EndShipPtrTableB)
    echo "WARNING: ShipPtrTableB crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Sprite Positioning (X = Object)
XPosition
  sta WSYNC                 ; [0]
  nop                       ; [0] + 2
  sec                       ; [2] + 2
  ; Divide Position by 15
Divide15
  sbc #15                   ; [4] + 2
  bcs Divide15              ; [6] + 2/3
  tay                       ; [8] + 2
  ; Store Coarse and Fine Position
  lda FineTuneEnd,Y         ; [10] + 5*
  sta HMP0,X                ; [15] + 4
  sta RESP0,X               ; [19] + 4
  rts                       ; Positions: [23/27/33/37/43/47/53/57/63/67]
EndXPosition
  if (>XPosition != >EndXPosition)
    echo "WARNING: XPosition crosses a page boundary!"
  endif

  DC.B  "DUCK8"

  ALIGN 256

  ; Sprite Fine Tuning Table
HMoveTable
  DC.B  %01100000           ; - 6 (Left)
  DC.B  %01010000           ; - 5
  DC.B  %01000000
  DC.B  %00110000
  DC.B  %00100000
  DC.B  %00010000
  DC.B  %00000000           ; Centre
  DC.B  %11110000
  DC.B  %11100000
  DC.B  %11010000
  DC.B  %11000000
  DC.B  %10110000
  DC.B  %10100000
  DC.B  %10010000           ; + 7
  DC.B  %10000000           ; + 8 (Right)
FineTuneEnd = HMoveTable - 241
EndHMoveTable
  if (>HMoveTable != >EndHMoveTable)
    echo "WARNING: HMoveTable crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Bullet Data
StartBulletData
  DS.B  YSTART+2,%10000000
BulletData
  DC.B  %10000010
  DS.B  YSTART+2,%10000000
EndBulletData
  if (>StartBulletData != >EndBulletData)
    echo "WARNING: BulletData crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Star Data
TopStarsA
  DC.B  %00000000
  DC.B  %00110010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %11100000
  DC.B  %10000010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01000010
TopStarsB
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000010
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00110000
  DC.B  %01110010
  DC.B  %00010000
  DC.B  %01110000
EndTopStars
  if (>TopStarsA != >EndTopStars)
    echo "WARNING: TopStars crosses a page boundary!"
  endif
  
  ; DC.B  "DUCK9"
  
  ALIGN 256

  ; Projectile Pointers
ProjPtrTable
  DC.B  <Projectile0, <Projectile1, <Projectile2, <Projectile3
  DC.B  <Projectile4, <Projectile4, <Projectile5, <Projectile5
  DC.B  <Projectile6, <Projectile6, <Projectile7, <Projectile7
  DC.B  <Projectile8, <Projectile8, <Projectile9, <Projectile9
EndProjPtrTable
  if (>ProjPtrTable != >EndProjPtrTable)
    echo "WARNING: ProjPtrTable crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Mine Data
StartMineData
  DS.B  YSTART+2,%10000000
MineData
  DC.B  %10000010
  DC.B  %10000010
  DS.B  YSTART+2,%10000000
EndMineData
  if (>StartMineData!= >EndMineData)
    echo "WARNING: MineData crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Star Data
BottomStarsA
  DC.B  %00000000
  DC.B  %00000010
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000010
  DC.B  %00100000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01010000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
BottomStarsB
  DC.B  %00000010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00100000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %00000000
  DC.B  %10110010
  DC.B  %00100000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110010
  DC.B  %10000000
  DC.B  %10000000
  DC.B  %10000000
EndBottomStars
  if (>BottomStarsA != >EndBottomStars)
    echo "WARNING: BottomStars crosses a page boundary!"
  endif

  ; DC.B  "DUCK10"
  
  ALIGN 256

StartPlayerMask
  DS.B  YSTART+8,0
PlayerMask
  DS.B  8,%11111111
  DS.B  YSTART,0
EndPlayerMask
  if (>StartPlayerMask != >EndPlayerMask)
    echo "WARNING: PlayerMask crosses a page boundary!"
  endif

  ; Ship Rotation Offsets
ShipXOffset
  DC.B  3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,3,3,3,2,3,3,3,3,3,3,3,3,3,3,3,2,3
EndShipXOffset
  if (>ShipXOffset != >EndShipXOffset)
    echo "WARNING: ShipXOffset crosses a page boundary!"
  endif
  
  echo "----",($FFF4 - *) , "bytes left (BANK2 - MAIN KERNEL)"

  ORG     $2FF4
  RORG    $FFF4
  DC.B    "BANK2", 0, 0, 0
  DC.W    Init2, Init2
  

