; -----------------------------------------------------------------------------
; Star Castle 2600 - Copyright (C) 2012 Chris Walton (cd-w) <cwalton@gmail.com>
; Bank 3 - Ring Rotation
; -----------------------------------------------------------------------------

  ; Rotate Ring 0 Macro
  MAC ROTATE0               ; 41 BYTES
.SEC          SET {1}
  ; Update Position
  ldx OFFSET0+.SEC          ; [0] + 3
  dex                       ; [3] + 2
  bpl .StoreRotate0         ; [5] + 2/3
  ldx #107                  ; [7] + 2
.StoreRotate0
  stx OFFSET0+.SEC          ; [9] + 3
  ; Calculate Ring Coordinate
  ldy Ring0Data,X           ; [12] + 4
  clc                       ; [16] + 2
  lda Ring0Position,X       ; [18] + 4
  adc ColumnOffset,Y        ; [22] + 4
  tax                       ; [26] + 2
  ; Rotate Sector Map
  lda SECTORMAP0_R+.SEC     ; [28] + 4
  cmp #$80                  ; [32] + 2
  rol                       ; [34] + 2
  ; Check For Add/Delete Pixel
  sta SECTORMAP0_W+.SEC     ; [36] + 4
  bmi .SectorEmpty0         ; [40] + 2/3
  ; Add Pixel
  lda GRID_R,X              ; [42] + 4
  ora PatternData,Y         ; [46] + 4
  bne .EndSector0           ; [50] + 3
.SectorEmpty0
  ; Delete Pixel
  lda GRID_R,X              ; [43] + 4
  and PatternDataInv,Y      ; [47] + 4
.EndSector0 
  sta GRID_W,X              ; [53] + 4
  ENDM                      ; WORST = 57 CYCLES (*12 = 684 CYCLES = 9 LINES)

  ; Rotate Ring 1 Macro
  MAC ROTATE1
.SEC          SET {1}
  ; Update Position
  ldx OFFSET1+.SEC
  dex
  bpl .StoreRotate1
  ldx #83
.StoreRotate1
  stx OFFSET1+.SEC
  ; Calculate Ring Coordinate
  ldy Ring1Data,X
  clc
  lda Ring1Position,X
  adc ColumnOffset,Y
  tax
  ; Rotate Sector Map
  lda SECTORMAP1_R+.SEC
  cmp #$80
  rol
  ; Check For Add/Delete Pixel
  sta SECTORMAP1_W+.SEC
  bmi .SectorEmpty1
  ; Add Pixel
  lda GRID_R,X
  ora PatternData,Y
  bne .EndSector1
.SectorEmpty1
  ; Delete Pixel
  lda GRID_R,X
  and PatternDataInv,Y
.EndSector1
  sta GRID_W,X
  ENDM

  ; Rotate Ring 2 Macro
  MAC ROTATE2
.SEC          SET {1}
  ; Update Position
  ldx OFFSET2+.SEC
  dex
  bpl .StoreRotate2
  ldx #59
.StoreRotate2
  stx OFFSET2+.SEC
  ; Calculate Ring Coordinate
  ldy Ring2Data,X
  clc
  lda Ring2Position,X
  adc ColumnOffset,Y
  tax
  ; Rotate Sector Map
  lda SECTORMAP2_R+.SEC
  cmp #$80
  rol
  ; Check For Add/Delete Pixel
  sta SECTORMAP2_W+.SEC
  bmi .SectorEmpty2
  ; Add Pixel
  lda GRID_R,X
  ora PatternData,Y
  bne .EndSector2
.SectorEmpty2
  ; Delete Pixel
  lda GRID_R,X
  and PatternDataInv,Y
.EndSector2
  sta GRID_W,X
  ENDM

; -----------------------------------------------------------------------------

  ; Bank 3
  SEG     BANK3
  ORG     $3000
  RORG    $F000

  ; Space for RAM+
  DS.B    512,35
  
Init3
  nop     $FFFB             ; Switch to Bank 6
EnterGun
  nop     $FFF9             ; Switch to Bank 4
  jmp     ShipCollisions
EnterExplosion
  nop     $FFF9             ; Switch to Bank 4

; -----------------------------------------------------------------------------
; Ship/Mine/Ring Collisions (must be done immediately after kernel)
; -----------------------------------------------------------------------------

ShipCollisions
  ; Skip If Already Dead
  ldx SHIPS
  bmi SkipMineRingCollision
  ; Skip Projectile Collisions
  lda PROJDIR
  bmi CheckMineRingColl
  lda CYCLE
  lsr
  bcc SkipMineRingCollision
CheckMineRingColl
  ; Only Check For Ring Collisions if Right Difficulty Set To A
  bit SWCHB
  bpl SkipShipRingCollision
  ; Check For Ring Collision
  bit CXPPMM
  bmi ExplodeShip
SkipShipRingCollision
  ; Check For Mine Collision
  bit CXP0FB
  bvc SkipMineRingCollision
ExplodeShip
  ; Explode Ship
  txa
  ora #%10000000
  sta SHIPS
  lda #10*4
  sta SHIPDIR_HI
  ; Play Sound
  lda #169
  cmp SFX
  bcc SkipMineRingCollision
  sta SFX
SkipMineRingCollision

; -----------------------------------------------------------------------------
; Ring Explosion
; -----------------------------------------------------------------------------

  ; Check For Ring Explosion Mode
  lda MODE
  cmp #%11100000
  bcc SectorClear
 
  ; Update Explosion Counter
  ldy EXPLODE
  beq ExplosionTransition
  dec EXPLODE
SkipExplosionUpdate
  ldx ExplodeFrame,Y
  jmp EnterExplosion

ExplosionTransition
  ; Check If Dead
  lda SHIPS
  bmi SkipExplosionUpdate
  ; Score + Wave Mode
  lda #%01011111
  sta MODE
  bne SkipExplosionUpdate

; -----------------------------------------------------------------------------
; Clear Sectors
; -----------------------------------------------------------------------------

SectorClear
  ; Check For Sector Clearing
  lda BCLEAR                ; [0] + 3
  bpl StartClearSector      ; [3] + 2/3
  jmp SectorGaps            ; [5] + 3

StartClearSector
  ; Branch To Clearing Routine
  ldy #0                    ; [6] + 2         Starting Score
  lsr                       ; [8] + 2
  bcs ClearSector1          ; [10] + 2/3
  lsr                       ; [12] + 2
  tax                       ; [14] + 2
  bcc ClearSector0          ; [16] + 2/3
  jmp ClearSector2          ; [18] + 3

; -----------------------------------------------------------------------------

ClearSector0
  ; Check For Empty Sector
  lda SECTORMAP0_R,X        ; [19] + 4
  cmp #$FF                  ; [23] + 2
  bne ClearSector0A         ; [25] + 2/3
  ; Decrease Sector Count & Set Score
  dec SECTORS               ; [27] + 5
  ldy #%00010000            ; [32] + 2
ClearSector0A
  sty BCLEAR                ; [34] + 3
  ; Rotate SectorMap For Operation
  cmp #$80                  ; [37] + 2
  rol                       ; [39] + 2
  sta TEMPMAP               ; [41] + 3
  ; Get Starting Offset
  ldy NextSectorIndexB,X    ; [44] + 4
  ldx OFFSET0,Y             ; [48] + 4
  dex                       ; [52] + 2
  bpl StoreSector0A         ; [54] + 2/3
  ldx #107                  ; [56] + 2
StoreSector0A
  txs                       ; [58] + 2
  ; Set Loop Counter
  lda #8                    ; [60] + 2
  sta COUNT                 ; [62] + 3  = 65
  ; Iterate Sector Coordinates
ClearSector0Loop
  ; Fetch Sector Offset
  ldy Ring0Data,X           ; [0] + 4
  clc                       ; [4] + 2
  lda Ring0Position,X       ; [6] + 4
  adc ColumnOffset,Y        ; [10] + 4
  tax                       ; [14] + 2
  ; Rotate Sector Map
  lda TEMPMAP               ; [16] + 3
  cmp #$80                  ; [19] + 2
  rol                       ; [21] + 2
  sta TEMPMAP               ; [23] + 3
  ; Skip Deletion For Dotted Segments
  bpl SkipSector00          ; [26] + 2/3
  lda GRID_R,X              ; [28] + 4
  and PatternDataInv,Y      ; [32] + 4
  sta GRID_W,X              ; [36] + 4
SkipSector00
  ; Next Iteration
  tsx                       ; [40] + 2
  dex                       ; [42] + 2
  bpl StoreSector0B         ; [44] + 2/3
  ldx #107                  ; [46] + 2
StoreSector0B
  txs                       ; [48] + 3
  dec COUNT                 ; [51] + 5
  bpl ClearSector0Loop      ; [56] + 2/3
  bmi FinishSectorClear1    ; WORST CASE = 59 * 9 = 531 (7 SCANLINES)

; -----------------------------------------------------------------------------

ClearSector1
  lsr
  tax
  lda SECTORMAP1_R,X
  cmp #$FF
  bne ClearSector1A
  dec SECTORS
  ldy #%00100000
ClearSector1A
  sty BCLEAR
  cmp #$80
  rol
  sta TEMPMAP
  ldy NextSectorIndexB,X
  ldx OFFSET1,Y
  dex
  bpl StoreSector1A
  ldx #83
StoreSector1A
  txs
  lda #6
  sta COUNT
ClearSector1Loop
  ldy Ring1Data,X
  clc
  lda Ring1Position,X
  adc ColumnOffset,Y
  tax
  lda TEMPMAP
  cmp #$80
  rol
  sta TEMPMAP
  bpl SkipSector01
  lda GRID_R,X
  and PatternDataInv,Y
  sta GRID_W,X
SkipSector01
  tsx
  dex
  bpl StoreSector1B
  ldx #83
StoreSector1B
  txs
  dec COUNT
  bpl ClearSector1Loop
FinishSectorClear1
  jmp FinishSectorClear
EndSectorClear
  if (>SectorClear != >EndSectorClear)
    echo "WARNING: SectorClear0/1 crosses a page boundary!"
  endif

  ; DC.B  "FISH"

; -----------------------------------------------------------------------------

  ALIGN 256

ClearSector2
  lda SECTORMAP2_R,X
  cmp #$FF
  bne ClearSector2A
  dec SECTORS
  ldy #1
  sty BREACH
  ldy #%00110000
ClearSector2A
  sty BCLEAR
  cmp #$80
  rol
  sta TEMPMAP
  ldy NextSectorIndexB,X
  ldx OFFSET2,Y
  dex
  bpl StoreSector2A
  ldx #59
StoreSector2A
  txs
  lda #4
  sta COUNT
ClearSector2Loop
  ldy Ring2Data,X
  clc
  lda Ring2Position,X
  adc ColumnOffset,Y
  tax
  lda TEMPMAP
  cmp #$80
  rol
  sta TEMPMAP
  bpl SkipSector02
  lda GRID_R,X
  and PatternDataInv,Y
  sta GRID_W,X
SkipSector02
  tsx
  dex
  bpl StoreSector2B
  ldx #59
StoreSector2B
  txs
  dec COUNT
  bpl ClearSector2Loop

FinishSectorClear
  ; Update Score (BCD Mode)
  sed
  clc
  lda BCLEAR
  adc SCORE+0
  sta SCORE+0
  lda SCORE+1
  adc #0
  sta SCORE+1
  lda SCORE+2
  adc #0
  sta SCORE+2
  cld
  
  ; Reset Stack & Clear Entry
  ldx #$FF
  txs
  stx BCLEAR
  jmp EnterGun              ; Skip Ring Rotation

EndSector2Clear
  if (>ClearSector2 != >EndSector2Clear)
    echo "WARNING: ClearSector2 crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------
; SectorGaps
; -----------------------------------------------------------------------------

SectorGaps
  ; Reset Breach
  lda #0                    ; [0] + 2
  sta BREACH                ; [2] + 3

  ; Check For Empty Outer Loop
  lda #$FF                  ; [5] + 2
  and SECTORMAP0_R+0        ; [7] + 4
  and SECTORMAP0_R+1        ; [11] + 4
  and SECTORMAP0_R+2        ; [15] + 4
  and SECTORMAP0_R+3        ; [19] + 4
  and SECTORMAP0_R+4        ; [23] + 4
  and SECTORMAP0_R+5        ; [27] + 4
  and SECTORMAP0_R+6        ; [31] + 4
  and SECTORMAP0_R+7        ; [35] + 4
  and SECTORMAP0_R+8        ; [39] + 4
  and SECTORMAP0_R+9        ; [43] + 4
  and SECTORMAP0_R+10       ; [47] + 4
  and SECTORMAP0_R+11       ; [51] + 4
  cmp #$FF                  ; [55] + 2
  bne CheckRingGap          ; [57] + 2/3
  jmp RegenerateRing        ; TOTAL = 60

; -----------------------------------------------------------------------------

CheckRingGap
  ; Search for Ring Gap
  ldy GUNDIR
  lda DirSectors,Y
  ldx #11
GapSearch
  ldy OFFSET2,X             ; [0] + 4
  cmp Div60,Y               ; [4] + 4
  beq FoundSector           ; [8] + 2/3
  dex                       ; [10] + 2
  bpl GapSearch             ; [12] + 2/3
  bmi EndSearch             ; WORST = 15 * 12 = 180 (2.3 SCANLINES)
FoundSector
  lda SECTORMAP2_R,X
  cmp #$FF
  bne EndSearch
  inc BREACH
EndSearch

  ; Rotate Rings
  jmp StartRotate

; -----------------------------------------------------------------------------

  ; Mapping From Ring2 Position To Sector Number
Div60
  DS.B  5,0
  DS.B  5,1
  DS.B  5,2
  DS.B  5,3
  DS.B  5,4
  DS.B  5,5
  DS.B  5,6
  DS.B  5,7
  DS.B  5,8
  DS.B  5,9
  DS.B  5,10
  DS.B  5,11
EndDiv60
  if (>Div60 != >EndDiv60)
    echo "WARNING: Div60 table crosses a page boundary!"
  endif

  DC.B  "JUP"

; -----------------------------------------------------------------------------
; Rotate Ring 0
; -----------------------------------------------------------------------------

  ALIGN 256

StartRotate
Rotate0A
  lda WAVE
  and #%00011111
  tax
  clc
  lda RING0COUNT
  adc Ring0Speeds,X
  sta RING0COUNT
  bcs DoRotate0
  jmp Rotate1A
DoRotate0
  clc
  ROTATE0 11
  ROTATE0 10
  ROTATE0 9
  ROTATE0 8
  jmp Rotate0B
EndRotate0A
  if (>Rotate0A != >EndRotate0A)
    echo "WARNING: Rotate0A crosses a page boundary!"
  endif

  ; Conversion From Sprite Direction To Ring Sectors
DirSectors
  DC.B  5, 5, 6, 6, 6, 7, 7, 7
  DC.B  8, 8, 9, 9, 9, 10, 10, 10
  DC.B  11, 11, 11, 0, 0, 0, 1, 1
  DC.B  2, 2, 2, 3, 3, 3, 4, 4
EndDirSectors
  if (>DirSectors != >EndDirSectors)
    echo "WARNING: DirSectors table crosses a page boundary!"
  endif

  DC.B  "IO"
  ALIGN 256
  
Rotate0B
  ROTATE0 7
  ROTATE0 6
  ROTATE0 5
  ROTATE0 4
  ROTATE0 3
  jmp Rotate0C
EndRotate0B
  if (>Rotate0B != >EndRotate0B)
    echo "WARNING: Rotate0B crosses a page boundary!"
  endif

  DC.B  "EUROPA"
  ALIGN 256
  
Rotate0C
  ROTATE0 2
  ROTATE0 1
  ROTATE0 0
  ; jmp Rotate1A
EndRotate0C
  if (>Rotate0C != >EndRotate0C)
    echo "WARNING: Rotate0C crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------
; Rotate Ring 1
; -----------------------------------------------------------------------------

Rotate1A
  lda WAVE
  and #%00011111
  tax
  clc
  lda RING1COUNT
  adc Ring1Speeds,X
  sta RING1COUNT
  bcs DoRotate1
  jmp Rotate2A
DoRotate1
  clc
  ROTATE1 11
  jmp Rotate1B
EndRotate1A
  if (>Rotate1A != >EndRotate1A)
    echo "WARNING: Rotate1A crosses a page boundary!"
  endif

  DC.B  "GANYMEDE"
  ALIGN 256
  
Rotate1B
  ROTATE1 10
  ROTATE1 9
  ROTATE1 8
  ROTATE1 7
  ROTATE1 6
  jmp Rotate1C
EndRotate1B
  if (>Rotate1B != >EndRotate1B)
    echo "WARNING: Rotate1B crosses a page boundary!"
  endif

  DC.B  "CALLISTO"
  ALIGN 256

Rotate1C
  ROTATE1 5
  ROTATE1 4
  ROTATE1 3
  ROTATE1 2
  ROTATE1 1
  jmp Rotate1D
EndRotate1C
  if (>Rotate1C != >EndRotate1C)
    echo "WARNING: Rotate1C crosses a page boundary!"
  endif
  
  DC.B  "AMALTHEA"
  ALIGN 256
  
Rotate1D
  ROTATE1 0
  ; jmp Rotate2A
EndRotate1D
  if (>Rotate1D != >EndRotate1D)
    echo "WARNING: Rotate1D crosses a page boundary!"
  endif
  
; -----------------------------------------------------------------------------
; Rotate Ring 2
; -----------------------------------------------------------------------------

Rotate2A
  lda WAVE
  and #%00011111
  tax
  clc
  lda RING2COUNT
  adc Ring2Speeds,X
  sta RING2COUNT
  bcs DoRotate2
  jmp EnterGun
DoRotate2
  clc
  ROTATE2 11
  ROTATE2 10
  ROTATE2 9
  jmp Rotate2B
EndRotate2A
  if (>Rotate2A != >EndRotate2A)
    echo "WARNING: Rotate2A crosses a page boundary!"
  endif

  DC.B  "HIMALIA"
  ALIGN 256

Rotate2B
  ROTATE2 8
  ROTATE2 7
  ROTATE2 6
  ROTATE2 5
  ROTATE2 4
  jmp Rotate2C
EndRotate2B
  if (>Rotate2B != >EndRotate2B)
    echo "WARNING: Rotate2B crosses a page boundary!"
  endif

  DC.B  "ELARA"
  ALIGN 256
  
Rotate2C
  ROTATE2 3
  ROTATE2 2
  ROTATE2 1
  ROTATE2 0
  jmp EnterGun
EndRotate2C
  if (>Rotate2C != >EndRotate2C)
    echo "WARNING: Rotate2C crosses a page boundary!"
  endif

; -----------------------------------------------------------------------------
; Regenerate Inner Ring
; -----------------------------------------------------------------------------

  ; Regenerate Inner Ring
RegenerateRing
  ldx #11
SectorCopyLoop
  lda SECTORMAP1_R,X        ; [0] + 4
  sta SECTORMAP0_W,X        ; [4] + 4
  lda SECTORMAP2_R,X        ; [8] + 4
  sta SECTORMAP1_W,X        ; [12] + 4
  lda #0                    ; [16] + 2
  sta SECTORMAP2_W,X        ; [18] + 4
  dex                       ; [22] + 2
  bpl SectorCopyLoop        ; [24] + 2/3
                            ; WORST = 27 * 11 = 297 (4 SCANLINES)
  ; Increase Sector Count
  clc
  lda SECTORS
  adc #12
  sta SECTORS
  jmp EnterGun               ; Skip Ring Rotation
EndRegenerateRing
  if (>RegenerateRing != >EndRegenerateRing)
    echo "WARNING: RegenerateRing crosses a page boundary!"
  endif

  DC.B  "PASIPHAE"

; -----------------------------------------------------------------------------
; Game Data
; -----------------------------------------------------------------------------

  ALIGN 256

  ; Ring 0 - 108 bytes
  ; Pattern Index | Column << 4
Ring0Data
  DC.B  0 | 1 << 4, 1 | 1 << 4, 2 | 1 << 4, 3 | 1 << 4
  DC.B  4 | 1 << 4, 5 | 1 << 4, 6 | 1 << 4, 7 | 1 << 4
  DC.B  0 | 0 << 4, 1 | 0 << 4, 2 | 0 << 4, 3 | 0 << 4
  DC.B  4 | 0 << 4, 4 | 0 << 4, 5 | 0 << 4, 5 | 0 << 4
  DC.B  6 | 0 << 4, 6 | 0 << 4, 6 | 0 << 4, 7 | 0 << 4
  DC.B  7 | 0 << 4, 7 | 0 << 4, 7 | 0 << 4, 7 | 0 << 4
  DC.B  7 | 0 << 4, 7 | 0 << 4, 7 | 0 << 4, 6 | 0 << 4
  DC.B  6 | 0 << 4, 6 | 0 << 4, 5 | 0 << 4, 5 | 0 << 4
  DC.B  4 | 0 << 4, 4 | 0 << 4, 3 | 0 << 4, 2 | 0 << 4
  DC.B  1 | 0 << 4, 0 | 0 << 4, 7 | 1 << 4, 6 | 1 << 4
  DC.B  5 | 1 << 4, 4 | 1 << 4, 3 | 1 << 4, 2 | 1 << 4
  DC.B  1 | 1 << 4, 0 | 1 << 4, 7 | 2 << 4, 6 | 2 << 4
  DC.B  5 | 2 << 4, 4 | 2 << 4, 3 | 2 << 4, 2 | 2 << 4
  DC.B  1 | 2 << 4, 0 | 2 << 4, 7 | 3 << 4, 6 | 3 << 4
  DC.B  5 | 3 << 4, 4 | 3 << 4, 3 | 3 << 4, 2 | 3 << 4
  DC.B  1 | 3 << 4, 0 | 3 << 4, 7 | 4 << 4, 6 | 4 << 4
  DC.B  5 | 4 << 4, 4 | 4 << 4, 3 | 4 << 4, 3 | 4 << 4
  DC.B  2 | 4 << 4, 2 | 4 << 4, 1 | 4 << 4, 1 | 4 << 4
  DC.B  1 | 4 << 4, 0 | 4 << 4, 0 | 4 << 4, 0 | 4 << 4
  DC.B  0 | 4 << 4, 0 | 4 << 4, 0 | 4 << 4, 0 | 4 << 4
  DC.B  0 | 4 << 4, 1 | 4 << 4, 1 | 4 << 4, 1 | 4 << 4
  DC.B  2 | 4 << 4, 2 | 4 << 4, 3 | 4 << 4, 3 | 4 << 4
  DC.B  4 | 4 << 4, 5 | 4 << 4, 6 | 4 << 4, 7 | 4 << 4
  DC.B  0 | 3 << 4, 1 | 3 << 4, 2 | 3 << 4, 3 | 3 << 4
  DC.B  4 | 3 << 4, 5 | 3 << 4, 6 | 3 << 4, 7 | 3 << 4
  DC.B  0 | 2 << 4, 1 | 2 << 4, 2 | 2 << 4, 3 | 2 << 4
  DC.B  4 | 2 << 4, 5 | 2 << 4, 6 | 2 << 4, 7 | 2 << 4
EndRing0Data
  if (>Ring0Data != >EndRing0Data)
    echo "WARNING: Ring0Data table crosses a page boundary!"
  endif

  ; Ring 1 - 84 bytes
  ; Pattern Index | Column << 4
Ring1Data
  DC.B  7 | 2 << 4, 6 | 2 << 4, 5 | 2 << 4, 4 | 2 << 4
  DC.B  3 | 2 << 4, 2 | 2 << 4, 1 | 2 << 4, 0 | 2 << 4
  DC.B  7 | 3 << 4, 6 | 3 << 4, 5 | 3 << 4, 4 | 3 << 4
  DC.B  3 | 3 << 4, 2 | 3 << 4, 1 | 3 << 4, 0 | 3 << 4
  DC.B  7 | 4 << 4, 7 | 4 << 4, 6 | 4 << 4, 6 | 4 << 4
  DC.B  6 | 4 << 4, 5 | 4 << 4, 5 | 4 << 4, 5 | 4 << 4
  DC.B  5 | 4 << 4, 5 | 4 << 4, 5 | 4 << 4, 5 | 4 << 4
  DC.B  5 | 4 << 4, 6 | 4 << 4, 6 | 4 << 4, 6 | 4 << 4
  DC.B  7 | 4 << 4, 7 | 4 << 4, 0 | 3 << 4, 1 | 3 << 4
  DC.B  2 | 3 << 4, 3 | 3 << 4, 4 | 3 << 4, 5 | 3 << 4
  DC.B  6 | 3 << 4, 7 | 3 << 4, 0 | 2 << 4, 1 | 2 << 4
  DC.B  2 | 2 << 4, 3 | 2 << 4, 4 | 2 << 4, 5 | 2 << 4
  DC.B  6 | 2 << 4, 7 | 2 << 4, 0 | 1 << 4, 1 | 1 << 4
  DC.B  2 | 1 << 4, 3 | 1 << 4, 4 | 1 << 4, 5 | 1 << 4
  DC.B  6 | 1 << 4, 7 | 1 << 4, 0 | 0 << 4, 0 | 0 << 4
  DC.B  1 | 0 << 4, 1 | 0 << 4, 1 | 0 << 4, 2 | 0 << 4
  DC.B  2 | 0 << 4, 2 | 0 << 4, 2 | 0 << 4, 2 | 0 << 4
  DC.B  2 | 0 << 4, 2 | 0 << 4, 2 | 0 << 4, 1 | 0 << 4
  DC.B  1 | 0 << 4, 1 | 0 << 4, 0 | 0 << 4, 0 | 0 << 4
  DC.B  7 | 1 << 4, 6 | 1 << 4, 5 | 1 << 4, 4 | 1 << 4
  DC.B  3 | 1 << 4, 2 | 1 << 4, 1 | 1 << 4, 0 | 1 << 4
EndRing1Data
  if (>Ring1Data != >EndRing1Data)
    echo "WARNING: Ring1Data table crosses a page boundary!"
  endif

  ; Ring2 - 60 bytes
  ; Pattern Index | Column << 4
Ring2Data
  DC.B  6 | 2 << 4, 7 | 2 << 4, 0 | 1 << 4, 1 | 1 << 4
  DC.B  2 | 1 << 4, 3 | 1 << 4, 4 | 1 << 4, 4 | 1 << 4
  DC.B  4 | 1 << 4, 5 | 1 << 4, 5 | 1 << 4, 5 | 1 << 4
  DC.B  5 | 1 << 4, 5 | 1 << 4, 5 | 1 << 4, 4 | 1 << 4
  DC.B  4 | 1 << 4, 4 | 1 << 4, 3 | 1 << 4, 2 | 1 << 4
  DC.B  1 | 1 << 4, 0 | 1 << 4, 7 | 2 << 4, 6 | 2 << 4
  DC.B  6 | 2 << 4, 5 | 2 << 4, 4 | 2 << 4, 3 | 2 << 4
  DC.B  2 | 2 << 4, 1 | 2 << 4, 1 | 2 << 4, 0 | 2 << 4
  DC.B  7 | 3 << 4, 6 | 3 << 4, 5 | 3 << 4, 4 | 3 << 4
  DC.B  3 | 3 << 4, 3 | 3 << 4, 3 | 3 << 4, 2 | 3 << 4
  DC.B  2 | 3 << 4, 2 | 3 << 4, 2 | 3 << 4, 2 | 3 << 4
  DC.B  2 | 3 << 4, 3 | 3 << 4, 3 | 3 << 4, 3 | 3 << 4
  DC.B  4 | 3 << 4, 5 | 3 << 4, 6 | 3 << 4, 7 | 3 << 4
  DC.B  0 | 2 << 4, 1 | 2 << 4, 1 | 2 << 4, 2 | 2 << 4
  DC.B  3 | 2 << 4, 4 | 2 << 4, 5 | 2 << 4, 6 | 2 << 4
EndRing2Data
  if (>Ring2Data != >EndRing2Data)
    echo "WARNING: Ring2Data table crosses a page boundary!"
  endif

  DC.B  "LEDA"

  ALIGN 256

  ; Ring Positions (Y Coordinates)
Ring0Position
  DC.B  1, 1, 1, 2, 2, 3, 3, 4
  DC.B  5, 6, 7, 8, 9, 10, 11, 12
  DC.B  13, 14, 15, 16, 17, 18, 19, 20
  DC.B  21, 22, 23, 24, 25, 26, 27, 28
  DC.B  29, 30, 31, 32, 33, 34, 35, 36
  DC.B  36, 37, 37, 38, 38, 38, 39, 39
  DC.B  39, 39, 39, 39, 39, 39, 38, 38
  DC.B  38, 37, 37, 36, 36, 35, 34, 33
  DC.B  32, 31, 30, 29, 28, 27, 26, 25
  DC.B  24, 23, 22, 21, 20, 19, 18, 17
  DC.B  16, 15, 14, 13, 12, 11, 10, 9
  DC.B  8, 7, 6, 5, 4, 3, 3, 2
  DC.B  2, 1, 1, 1, 0, 0, 0, 0
  DC.B  0, 0, 0, 0
EndRing0Position
  if (>Ring0Position != >EndRing0Position)
    echo "WARNING: Ring0Position table crosses a page boundary!"
  endif

Ring1Position
  DC.B  5, 5, 5, 5
  DC.B  5, 5, 5, 5, 6, 6, 7, 7
  DC.B  8, 9, 10, 11, 12, 13, 14, 15
  DC.B  16, 16, 17, 18, 19, 20, 21, 22
  DC.B  23, 23, 24, 25, 26, 27, 28, 29
  DC.B  30, 31, 32, 32, 33, 33, 34, 34
  DC.B  34, 34, 34, 34, 34, 34, 33, 33
  DC.B  32, 32, 31, 30, 29, 28, 27, 26
  DC.B  25, 24, 23, 23, 22, 21, 20, 19
  DC.B  18, 17, 16, 16, 15, 14, 13, 12
  DC.B  11, 10, 9, 8, 7, 7, 6, 6
EndRing1Position
  if (>Ring1Position != >EndRing1Position)
    echo "WARNING: Ring1Position table crosses a page boundary!"
  endif

Ring2Position
  DC.B  11, 11, 11, 12, 13, 14, 15, 16
  DC.B  17, 17, 18, 19, 20, 21, 22, 22
  DC.B  23, 24, 25, 26, 27, 28, 28, 28
  DC.B  29, 29, 29, 29, 29, 29, 28, 28
  DC.B  28, 27, 26, 25, 24, 23, 22, 22
  DC.B  21, 20, 19, 18, 17, 17, 16, 15
  DC.B  14, 13, 12, 11, 11, 11, 10, 10
  DC.B  10, 10, 10, 10
EndRing2Position
  if (>Ring2Position != >EndRing2Position)
    echo "WARNING: Ring2Position table crosses a page boundary!"
  endif

  DC.B  "KALE"

  ALIGN 256
  
  ; Ring Column Start (GRID_R/GRID_W)
ColumnOffset
  DS.B  16, 0
  DS.B  16, 40
  DS.B  16, 80
  DS.B  16, 120
  DS.B  16, 160
EndColumnOffset
  if (>ColumnOffset != >EndColumnOffset)
    echo "WARNING: ColumnOffset table crosses a page boundary!"
  endif

  ; Ring Pattern Data (Adding)
PatternData
  REPEAT 10
  DC.B  %00000001
  DC.B  %00000010
  DC.B  %00000100
  DC.B  %00001000
  DC.B  %00010000
  DC.B  %00100000
  DC.B  %01000000
  DC.B  %10000000
  REPEND
EndPatternData
  if (>PatternData != >EndPatternData)
    echo "WARNING: PatternData table crosses a page boundary!"
  endif

  ; Ring Inverse Pattern Data (Deleting)
PatternDataInv
  REPEAT 10
  DC.B  %11111110
  DC.B  %11111101
  DC.B  %11111011
  DC.B  %11110111
  DC.B  %11101111
  DC.B  %11011111
  DC.B  %10111111
  DC.B  %01111111
  REPEND
EndPatternDataInv
  if (>PatternDataInv != >EndPatternDataInv)
    echo "WARNING: PatternDataInv table crosses a page boundary!"
  endif

NextSectorIndexB
  DC.B  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 0
EndNextSectorIndexB
  if (>NextSectorIndexB != >EndNextSectorIndexB)
    echo "WARNING: NextSectorIndexB crosses a page boundary!"
  endif

  DC.B  "KORE"

  ALIGN 256

  ; Explosion Frames (120 entries)
ExplodeFrame
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5  ;
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5  ;
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5  ;
  DC.B 15*5
  DC.B 15*5
  DC.B 15*5
  DC.B 15*5
  DC.B 15*5
  DC.B 15*5
  DC.B 15*5
  DC.B 15*5 ;
  DC.B 15*5
  DC.B 15*5
  DC.B 14*5
  DC.B 14*5
  DC.B 14*5
  DC.B 14*5
  DC.B 14*5
  DC.B 14*5 ;
  DC.B 14*5
  DC.B 14*5
  DC.B 14*5
  DC.B 14*5
  DC.B 13*5
  DC.B 13*5
  DC.B 13*5
  DC.B 13*5 ;
  DC.B 13*5
  DC.B 13*5
  DC.B 12*5
  DC.B 12*5
  DC.B 12*5
  DC.B 12*5
  DC.B 12*5
  DC.B 12*5 ;
  DC.B 11*5
  DC.B 11*5
  DC.B 11*5
  DC.B 11*5
  DC.B 10*5
  DC.B 10*5
  DC.B 10*5
  DC.B 10*5 ;
  DC.B 9*5
  DC.B 9*5
  DC.B 9*5
  DC.B 9*5
  DC.B 8*5
  DC.B 8*5
  DC.B 8*5
  DC.B 8*5  ;
  DC.B 8*5
  DC.B 8*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5
  DC.B 0*5  ;
  DC.B 8*5
  DC.B 8*5
  DC.B 8*5
  DC.B 8*5
  DC.B 8*5
  DC.B 8*5
  DC.B 7*5
  DC.B 7*5  ;
  DC.B 7*5
  DC.B 7*5
  DC.B 7*5
  DC.B 7*5
  DC.B 6*5
  DC.B 6*5
  DC.B 6*5
  DC.B 6*5  ;
  DC.B 6*5
  DC.B 6*5
  DC.B 5*5
  DC.B 5*5
  DC.B 5*5
  DC.B 5*5
  DC.B 5*5
  DC.B 5*5  ;
  DC.B 4*5
  DC.B 4*5
  DC.B 4*5
  DC.B 4*5
  DC.B 3*5
  DC.B 3*5
  DC.B 3*5
  DC.B 3*5  ;
  DC.B 2*5
  DC.B 2*5
  DC.B 2*5
  DC.B 2*5
  DC.B 1*5
  DC.B 1*5
  DC.B 1*5
  DC.B 1*5  ; 119
EndExplodeFrame
  if (>ExplodeFrame != >EndExplodeFrame)
    echo "WARNING: ExplodeFrame table crosses a page boundary!"
  endif

  ; Wave Speed Table
RingSpeeds
Ring0Speeds
  DC.B  85, 102, 128, 85, 102, 128, 85, 102, 128, 85, 102
  DC.B  102, 128, 171, 102, 128, 171, 102, 128, 171, 128, 255
  DC.B  128, 171, 256, 128, 171, 256, 128, 171, 255, 255
Ring1Speeds
  DC.B  66, 80, 100, 66, 80, 100, 66, 80, 100, 85, 102
  DC.B  80, 100, 133, 80, 100, 133, 80, 100, 133, 128, 128
  DC.B  100, 133, 199, 100, 133, 199, 100, 133, 199, 255
Ring2Speeds
  DC.B  47, 57, 71, 47, 57, 71, 47, 57, 71, 85, 102
  DC.B  57, 71, 95, 57, 71, 95, 57, 71, 95, 128, 64
  DC.B  71, 95, 142, 71, 95, 142, 71, 95, 142, 255
EndRingSpeeds
  if (>RingSpeeds != >EndRingSpeeds)
    echo "WARNING: RingSPeeds Table crosses a page boundary!"
  endif

  echo "----",($FFF4 - *) , "bytes left (BANK3 - RING ROTATION)"

  ORG     $3FF4
  RORG    $FFF4
  DC.B    "BANK3", 0, 0, 0
  DC.W    Init3, Init3

