; -----------------------------------------------------------------------------
; Star Castle 2600 - Copyright (C) 2012 Chris Walton (cd-w) <cwalton@gmail.com>
; Bank 6 - Startup Code & Hiscore Table
; -----------------------------------------------------------------------------

  ; Bank 6
  SEG     BANK6
  ORG     $6000
  RORG    $F000
  
  ; Space for RAM+
  DS.B    512,35

ShowTitleScreen
  nop     $FFFA           ; Switch to Bank 5
  ; Entry Point
  jmp     Start
EnterGame
  nop     $FFF5           ; Switch to Bank 0
  ; End Of Game
  jmp     EndOfGame
ShowStartupScreen
  nop     $FFF6           ; Switch to Bank 1
  jmp     EndStartupScreen
  ; End Of Title Screen
  jmp     TitlesLoop
PlayTitleMusic
  nop     $FFFA           ; Switch to Bank 5
  ; End Of Title Music
  jmp     ResumeTitleMusic

; -----------------------------------------------------------------------------
; STARTUP
; -----------------------------------------------------------------------------

Start
  ; Check for 7800 Console (needed for pause button handling)
  sei
  ldy #$FF
  lda $D0
  cmp #$2C
  bne Not7800
  lda $D1
  cmp #$A9
  bne Not7800
  iny                     ; 7800 - bit7 = 0
Not7800                   ; 2600 - bit7 = 1

  ; Clear RIOT RAM
  ldx #0
  txa
  cld
ClearMemLoop
  dex
  txs
  pha
  bne ClearMemLoop        ; SP=$FF, X = A = 0

  ; Store 7800 Console Bit
  tya
  and #%10000000
  sta INITIALS+1

  ; Clear RAM+ Memory
  jsr StartScreen
  jsr ClearExtraRam
  
  ; Startup Delay
  lda #127
  sta CYCLE

EndStartupScreen
  jsr NextFrame

  ; Update Counter
  dec CYCLE
  beq FinishStartup

  ; Check For Wipe Scores (Fire + Right)
  lda INPT4
  bmi StartupKernel
  ; Debounce Fire
  lda #%01000000
  sta DEBOUNCE
  ; Check For Right Joystick
  bit SWCHA
  bmi FinishStartup
  jmp WipeHiScores

StartupKernel
  ; Show Startup Logo
  jsr PositionText
  jsr StartScreen
  jmp ShowStartupScreen

FinishStartup
  ; Read Hiscore Table
  ldx #1
  jsr DoOperation
  ; Get Stored Colour Mode
  lda COLMODE_R
  sta COLMODE

; -----------------------------------------------------------------------------
; TITLE SCREENS
; -----------------------------------------------------------------------------

  ; Display Title Screen
  ldx #127
  stx SLOWCYCLE

StartTitles
  ; Title Starting Message
  lda #0
  sta MESSAGE
  
  ; Reset Title Tune
  sta AUDV0
  sta AUDV1
  sta MEASURE
  sta BEAT
  sta TEMPOCOUNT
  
  ; Next Message Timer
  lda #63
  sta MTIMER
  jmp SkipTitleVBlank

TitlesLoop
  BEGIN_VBLANK

SkipTitleVBlank
  ; Update Frame Counter 
  dec CYCLE
  
  ; Update Slow Cycle & Message Timer
  lax CYCLE
  and #%00000011
  cmp #%00000011
  bne EndSlowCycle
  dec SLOWCYCLE
EndSlowCycle
  txa
  lsr
  bcs SkipTimerReset
  dec MTIMER
  bpl SkipTimerReset
  ldx MESSAGE
  dex
  bpl SkipMessageReset
  ldx #2
SkipMessageReset
  stx MESSAGE
  lda MessageDelay,X
  sta MTIMER
SkipTimerReset

  ; Select Button
  lda SWCHB
  and #%00000010
  bne SelectNotPressed
  ; Debounce Select Button
  lda DEBOUNCE
  and #%00000001
  bne EndSelectPressed
  ; Update Colour Mode
  ldx COLMODE
  cpx #2
  bcs ResetColMode
  inx
  jmp StoreColMode
ResetColMode
  ldx #0
StoreColMode
  stx COLMODE
  ; Debounce Select Button
  lda DEBOUNCE
  ora #%00000001
  bne StoreSelectDebounce
SelectNotPressed
  lda DEBOUNCE
  and #%11111110
StoreSelectDebounce
  sta DEBOUNCE
EndSelectPressed

  ; Joystick Left/Right
  lda SWCHA
  and #%11000000
  cmp #%11000000
  beq ResetTitleJoy
  ; Debounce Left/Right
  lda DEBOUNCE
  and #%00000010
  bne EndTitleJoy
  ; Flip Screen
  lda SLOWCYCLE
  eor #%10000000
  ora #%01111111
  sta SLOWCYCLE
  lda DEBOUNCE
  ora #%00000010
  bne StoreTitleJoy
ResetTitleJoy
  lda DEBOUNCE
  and #%11111101
StoreTitleJoy
  sta DEBOUNCE
EndTitleJoy

  ; Fire Button Start
  jsr CheckTitleFire
  bit DEBOUNCE
  bpl NoTitleFire
  jsr StartScreen
  jsr ClearExtraRam
  BEGIN_OVERSCAN
  jmp EnterGame
NoTitleFire

  ; Play Title Screen Music
  jmp PlayTitleMusic
ResumeTitleMusic
  
  ; Show Tiles Screen or HiScore Table
  bit SLOWCYCLE
  bmi HiScorePhase
  jmp ShowTitleScreen
HiScorePhase
  jmp HiScoreTable

; -----------------------------------------------------------------------------
; HISCORE UPDATE
; -----------------------------------------------------------------------------

EndOfGame
  ; Silence Sound Effects
  lda #0
  sta AUDV0
  sta AUDV1

  jsr StartScreen
  jsr ClearExtraRam
  jsr NextFrame
  
  ; Debounce (assume fire is still pressed)
  lda #%01000000
  sta DEBOUNCE

  ; Read Hi Score Table
  ldx #1
  jsr DoOperation
  
  ; Check For Empty Score
  lda SCORE+2
  ora SCORE+1
  ora SCORE+0
  beq FinishHiScores
  
  ; Calculate Position
  ldx #MAXSCORES-1
FindPositionLoop
  ldy ScoreOffsets,X        ; [0] + 4
  lda SCORE+2               ; [4] + 3
  cmp HI_READ+2,Y           ; [7] + 4
  bcc NextScorePosition     ; [11] + 2/3  <
  bne StoreScorePosition    ; [13] + 2/3  >
  lda SCORE+1               ; [15] + 3
  cmp HI_READ+1,Y           ; [18] + 4
  bcc NextScorePosition     ; [22] + 2/3  <
  bne StoreScorePosition    ; [24] + 2/3  >
  lda SCORE+0               ; [26] + 3
  cmp HI_READ+0,Y           ; [29] + 4
  bcc NextScorePosition     ; [33] + 2/3  <
StoreScorePosition
  stx HPOS                  ; [0] + 3
  ; Don't Copy In Lowest Position
  cpx #0                    ; [3] + 2
  beq NameEntry             ; [5] + 2/3
  ; Move Entries Down 
  ldx #0                    ; [7] + 2
  stx HROW                  ; [9] + 3 = 12
CopyLoop
  ; ldx HROW
  lda ScoreOffsets,X        ; [0] + 4
  ldy ScoreOffsets+1,X      ; [4] + 4
  tax                       ; [8] + 2
  lda HI_READ+0,Y           ; [10] + 4
  sta HI_WRITE+0,X          ; [14] + 4
  lda HI_READ+1,Y           ; [18] + 4
  sta HI_WRITE+1,X          ; [22] + 4
  lda HI_READ+2,Y           ; [26] + 4
  sta HI_WRITE+2,X          ; [30] + 4
  lda HI_READ+3,Y           ; [34] + 4
  sta HI_WRITE+3,X          ; [38] + 4
  lda HI_READ+4,Y           ; [42] + 4
  sta HI_WRITE+4,X          ; [46] + 4
  lda HI_READ+5,Y           ; [50] + 4
  sta HI_WRITE+5,X          ; [54] + 4
  ldx HROW                  ; [58] + 3
  inx                       ; [61] + 2
  cpx HPOS                  ; [63] + 3
  bcs NameEntry             ; [66] + 2/3
  stx HROW                  ; [69] + 3
  jmp CopyLoop              ; [72] + 3 = 75
NextScorePosition
  dex                       ; [36] + 2
  bpl FindPositionLoop      ; [38] + 2/3
                            ; WORST CASE = 41*8 + 40 + 12 + 8*75 
                            ;            = 980 (APPROX 13 SCANLINES)
FinishHiScores
  ; Debounce Fire Button
  lda #%01000000
  sta DEBOUNCE
  ; Show HiScore Table
  lda #255
  sta SLOWCYCLE
  jmp StartTitles

NameEntry
  ; Reflect PF and Set Ball Size
  ldx #%00100001
  stx CTRLPF

  ; Reset Cursor Position
  ldx #0
  stx CURSOR
  
  ; Unpack Stored Initials
  lax INITIALS+0
  and #%00011111
  sta NAME+0
  txa
  and #%11100000
  lsr
  lsr
  lsr
  sta TEMP
  lax INITIALS+1
  and #%00000011
  ora TEMP
  sta NAME+1
  txa
  and #%01111100
  lsr
  lsr
  sta NAME+2

NameLoop
  ; Update Game Cycle
  dec CYCLE

  ; Vector Text Setup
  jsr SetScorePointers

  ; Play Sound Effects
  jsr HiScoreSFX

  ; Move Cursor With Joystick
  lax SWCHA
  and #%11110000
  cmp #%11110000
  beq ResetNameJoy
  ; Debounce Joystick
  lda DEBOUNCE
  and #%00000010
  bne EndNameJoy
  ; Move Cursor
  ldx CURSOR
  ldy NAME,X
  bit SWCHA
  bpl CursorRight
  bvc CursorLeft
  jmp CheckCursorDown
CursorLeft
  dex
  bpl StoreCursor
  ldx #2
  bne StoreCursor
CursorRight
  inx
  cpx #3
  bcc StoreCursor
  ldx #0
  beq StoreCursor
CheckCursorDown
  lda #%00100000
  and SWCHA
  bne CheckCursorUp
  dey
  jmp StoreName
CheckCursorUp
  lda #%00010000
  and SWCHA
  bne EndNameJoy
  iny
StoreName
  tya
  and #%00011111
  sta NAME,X
  stx CURSOR
  ldx #((2<<5)|3)
  bne StoreNameSFX
StoreCursor
  stx CURSOR
  ldx #((1<<5)|3)
StoreNameSFX
  stx SFX
  lda DEBOUNCE
  ora #%00000010
  bne StoreNameJoy
ResetNameJoy
  lda DEBOUNCE
  and #%11111101
StoreNameJoy
  sta DEBOUNCE
EndNameJoy

  ; Check If Fire Pressed
  jsr CheckTitleFire
  bit DEBOUNCE
  bpl EndCheckNameFire
  ; Copy Score Into Table
  ldx HPOS
  ldy ScoreOffsets,X
  lda SCORE+0
  sta HI_WRITE+0,Y
  lda SCORE+1
  sta HI_WRITE+1,Y
  lda SCORE+2
  sta HI_WRITE+2,Y
  ; Copy Name Into Table
  jsr PackName
  lda INITIALS+0
  sta HI_WRITE+3,Y
  lda INITIALS+1
  and #%01111111
  sta HI_WRITE+4,Y
  ; Copy Wave Into Table
  clc
  lda WAVE
  and #%00111111
  adc #1
  sta HI_WRITE+5,Y
  ; Store Colour Mode
  lda COLMODE
  sta COLMODE_W
  ; Write Score Table
  ldx #2
  jsr DoOperation
  jmp FinishHiScores
EndCheckNameFire
  
  ; Start Screen
  jsr StartScreen
  ldy #40
  jsr SkipLinesB
  
  ; Show New HiScore Message
  ldx COLMODE
  lda HeadingCol,X
  sta COLUP1
  sta COLUP0
  jsr PositionText
  ldy #(3*12)
  jsr LoadText
  jsr NormalTextKernel

  ; Display Last Score
  ldx COLMODE
  lda ScoreCol,X
  sta COLUP1
  sta COLUP0
  
  ; Create Text Showing Score and Table Position
  ldy HPOS
  lda (SLLPTR),Y
  sta HTEXT0
  lda (SLHPTR),Y
  sta HTEXT1
  ldy #10
  lda (HDPTR),Y
  sta HTEXT2
  sta HTEXT3
  sta HTEXT4
  lda <#HL_GAP
  sta HTEXT5
  ldx #<SCORE
  jsr CopyScore
  jsr NormalTextCopy
  jsr NormalTextKernel
  ldy #15
  jsr SkipLinesB

  ; Show Enter Name Message
  ldx COLMODE
  lda HeadingCol,X
  sta COLUP1
  sta COLUP0
  ldy #(4*12)
  jsr LoadText
  jsr NormalTextKernel
  
  ldy #4
  jsr SkipLinesB

  ; Default Name Colours
  ldx COLMODE
  lda ScoreCol,X
  sta NAMECOL+0
  sta NAMECOL+1
  sta NAMECOL+2

  ; Set Cursor Colour
  ldy CURSOR
  lda CYCLE
  and #%00001000
  bne NameFlash
  nop
  beq SkipNameFlash
NameFlash
  lda #0
  sta NAMECOL,Y
SkipNameFlash

  ; Show Name
  jsr NameKernel
  ldy #12
  jsr SkipLinesB

  ; Set Fire Message Colour
  ldx COLMODE
  lda CYCLE
  and #%00100000
  bne LoadFireCol
  ldy #0
  beq StoreFireCol
LoadFireCol
  ldy FireCol,X
StoreFireCol
  sty COLUP0
  sty COLUP1

  ; Show Fire To Save Message
  ldy #(5*12)
  jsr PositionText
  jsr LoadText
  jsr NormalTextKernel

  ; Skip to Next Frame
  jsr NextFrame
  jmp NameLoop

; -----------------------------------------------------------------------------
; HIGH SCORE TABLE
; -----------------------------------------------------------------------------

HiScoreTable
  ; Set Vector Font Pointers
  jsr SetScorePointers
  
  ; Set Sprite Positions
  jsr PositionText

  ; Set Heading Colour
  ldx COLMODE
  lda CYCLE
  and #%00010000
  bne LoadHeadingCol
  ldy #0
  beq StoreHeadingCol
LoadHeadingCol
  ldy HeadingCol,X
StoreHeadingCol
  sty COLUP0
  sty COLUP1

  ; Start Screen
  jsr StartScreen

  ; Load Heading Text
  ldy #6*12
  jsr LoadText
  
  ; Show Heading
  jsr NormalTextKernel

  ; Adjust Sprite Positions
  jsr PositionHiScore

  ; Set Lines For Score Table
  ldx #MAXSCORES-1
  stx HROW

  ; Show Coloured Score Table
ScoreTableLoop
  sta WSYNC               ; [0]

  ; Set Row Colour
  ldx COLMODE
  lda ScoreCol,X
  sta COLUP0
  sta COLUP1

  ; Show Score Position (1 to 10)
  ldy HROW
  ldx #<HL_GAP
  stx HTEXT2
  lda (SLLPTR),Y
  sta HTEXT1
  lda (SLHPTR),Y
  sta HTEXT0

  ; Copy Score
  ldx ScoreOffsets,Y
  lda HI_READ+$00,X
  sta HSCORE+0
  lda HI_READ+$01,X
  sta HSCORE+1

  lda HI_READ+$02,X
  sta HSCORE+2
  stx TEMP
  ldx #<HSCORE
  jsr CopyScore
  
  ; Copy Name
  ldx TEMP
  lda HI_READ+$03,X
  and #%00011111
  tay
  
  lda (SLPTR),Y
  sta HTEXT3
  lda HI_READ+$03,X
  and #%11100000
  lsr
  lsr
  lsr
  sta TEMP
  lda HI_READ+$04,X
  and #%00000011
  ora TEMP
  tay
  lda (SLPTR),Y
  sta HTEXT4
  lda HI_READ+$04,X
  and #%01111100
  lsr
  lsr
  tay
  lda (SLPTR),Y
  sta HTEXT5
  
  ; Copy Wave Number
  lda HI_READ+$05,X
  tax
  lda HiScoreBCDTable,X
  lsr
  lsr
  lsr
  lsr
  tay
  lda (HDPTR),Y
  sta HTEXT12
  lda HiScoreBCDTable,X
  and #%00001111
  tay
  lda (HDPTR),Y
  sta HTEXT13

  ; Show Line
  jsr HiScoreTextCopy
  jsr HiScoreKernel
  dec HROW 
  bmi EndScoreTableLoop
  jmp ScoreTableLoop
EndScoreTableLoop

  ; Reset Sprites
  lda #0
  sta NUSIZ0
  sta NUSIZ1
  sta VDELP0
  sta VDELP1

  BEGIN_OVERSCAN
  jmp TitlesLoop

; -----------------------------------------------------------------------------
; WIPE HISCORE TABLES
; -----------------------------------------------------------------------------

WipeHiScores
  ; Initialise Prompt
  ldx #0
  stx HROW

HiScoreWipeLoop
  ; Update Frame Counter
  dec CYCLE

  ; Play Sound Effects
  jsr HiScoreSFX

  ; Joystick Up/Down
  lda SWCHA
  and #%00110000
  cmp #%00110000
  beq ResetWipeJoy
  ; Debounce Left/Right
  lda DEBOUNCE
  and #%00000010
  bne EndWipeJoy
  ; Play Sound
  lda #((1<<5)|3)
  sta SFX
  ; Invert Selection
  lda HROW
  eor #%11111111
  sta HROW
  lda DEBOUNCE
  ora #%00000010
  bne StoreWipeDebounce
ResetWipeJoy
  lda DEBOUNCE
  and #%11111101
StoreWipeDebounce
  sta DEBOUNCE
EndWipeJoy

  ; Check If Fire Pressed
  jsr CheckTitleFire
  bit DEBOUNCE
  bpl EndCheckWipeFire
  ; Check Result
  lda HROW
  beq EndWipeScores
  ; Wipe Score Table
  ldx #2
  jsr DoOperation
EndWipeScores
  jmp FinishStartup
EndCheckWipeFire

  ; Reflect PF and Set PF Colour
  ldx #%00000001
  stx CTRLPF
  ldx #$44                  ; Red (NTSC) / Orange (PAL)
  stx COLUPF

  ; Set Message Colour
  ldx #$0E
  stx COLUP1
  stx COLUP0

  ; Position Sprites and Load First Message
  jsr PositionText
  ldy #(0*12)
  jsr LoadText

  ; Start Screen and Show First Message
  jsr StartScreen
  ldy #55
  jsr SkipLinesB
  jsr NormalTextKernel
  ldy #27
  jsr SkipLinesB

  ; Load "Wipe" Message
  ldy #(1*12)
  jsr LoadText
  
  ; Show Wipe Message
  sta WSYNC
  ldx #%11111000
  lda HROW
  beq SkipDoWipe
  bne ShowDoWipe
SkipDoWipe
  ldx #0
ShowDoWipe
  stx PF2
  jsr NormalTextKernel
  sta WSYNC
  ldx #0
  stx PF2
  ldy #6
  jsr SkipLinesB

  ; Load "Ignore" Message
  ldy #(2*12)
  jsr LoadText
  
  ; Show Ignore Message
  sta WSYNC
  ldx #%11111000
  lda HROW
  bne SkipWipeIgnore
  beq ShowWipeIgnore
SkipWipeIgnore
  ldx #0
ShowWipeIgnore
  stx PF2
  jsr NormalTextKernel
  sta WSYNC
  ldx #0
  stx PF2
  
  ; Skip To Next Frame
  jsr NextFrame
  jmp HiScoreWipeLoop

; -----------------------------------------------------------------------------
; FLASH OPERATIONS
; -----------------------------------------------------------------------------

  ; X = Operation:
  ; 1 = Read Score Table
  ; 2 = Write Score Table

DoOperation
  ; Store Operation & Index
  stx OPERATION_W

  ; Do Operation
  jmp DoStall
Resume
  ; Set Vblank Timer
  ldx #VBLANKDELAY
  stx TIM64T

  ; Reset Sprites
  ldx #0
  stx REFP0
  stx REFP1
  stx GRP0
  stx GRP1
  stx GRP0

  ; Check Error
  ; IF (MELODY)
  ; lda ERRORCODE_R
  ; beq EndOperation
  ; HANDLE ERROR ???
  ; ENDIF
EndOperation
  rts

; -----------------------------------------------------------------------------
; HISCORE SUBROUTINES
; -----------------------------------------------------------------------------

  ; Copy text Into Buffer For Display (Combining Pairs Of Characters)
HiScoreTextCopy
  ldx HTEXT13
  ldy HTEXT12
  lda HRChars+4,X
  ora HLChars+4,Y
  sta HBUFF+34
  lda HRChars+3,X
  ora HLChars+3,Y
  sta HBUFF+33
  lda HRChars+2,X
  ora HLChars+2,Y
  sta HBUFF+32
  lda HRChars+1,X
  ora HLChars+1,Y
  sta HBUFF+31
  lda HRChars+0,X
  ora HLChars+0,Y
  sta HBUFF+30
NormalTextCopy
  ldx HTEXT11
  ldy HTEXT10
  lda HRChars+4,X
  ora HLChars+4,Y
  sta HBUFF+29
  lda HRChars+3,X
  ora HLChars+3,Y
  sta HBUFF+28
  lda HRChars+2,X
  ora HLChars+2,Y
  sta HBUFF+27
  lda HRChars+1,X
  ora HLChars+1,Y
  sta HBUFF+26
  lda HRChars+0,X
  ora HLChars+0,Y
  sta HBUFF+25
  ldx HTEXT9
  ldy HTEXT8
  lda HRChars+4,X
  ora HLChars+4,Y
  sta HBUFF+24
  lda HRChars+3,X
  ora HLChars+3,Y
  sta HBUFF+23
  lda HRChars+2,X
  ora HLChars+2,Y
  sta HBUFF+22
  lda HRChars+1,X
  ora HLChars+1,Y
  sta HBUFF+21
  lda HRChars+0,X
  ora HLChars+0,Y
  sta HBUFF+20
  ldx HTEXT7
  ldy HTEXT6
  lda HRChars+4,X
  ora HLChars+4,Y
  sta HBUFF+19
  lda HRChars+3,X
  ora HLChars+3,Y
  sta HBUFF+18
  lda HRChars+2,X
  ora HLChars+2,Y
  sta HBUFF+17
  lda HRChars+1,X
  ora HLChars+1,Y
  sta HBUFF+16
  lda HRChars+0,X
  ora HLChars+0,Y
  sta HBUFF+15
  ldx HTEXT5
  ldy HTEXT4
  lda HRChars+4,X
  ora HLChars+4,Y
  sta HBUFF+14
  lda HRChars+3,X
  ora HLChars+3,Y
  sta HBUFF+13
  lda HRChars+2,X
  ora HLChars+2,Y
  sta HBUFF+12
  lda HRChars+1,X
  ora HLChars+1,Y
  sta HBUFF+11
  lda HRChars+0,X
  ora HLChars+0,Y
  sta HBUFF+10
  ldx HTEXT3
  ldy HTEXT2
  lda HRChars+4,X
  ora HLChars+4,Y
  sta HBUFF+9
  lda HRChars+3,X
  ora HLChars+3,Y
  sta HBUFF+8
  lda HRChars+2,X
  ora HLChars+2,Y
  sta HBUFF+7
  lda HRChars+1,X
  ora HLChars+1,Y
  sta HBUFF+6
  lda HRChars+0,X
  ora HLChars+0,Y
  sta HBUFF+5
  ldx HTEXT1
  ldy HTEXT0
  lda HRChars+4,X
  ora HLChars+4,Y
  sta HBUFF+4
  lda HRChars+3,X
  ora HLChars+3,Y
  sta HBUFF+3
  lda HRChars+2,X
  ora HLChars+2,Y
  sta HBUFF+2
  lda HRChars+1,X
  ora HLChars+1,Y
  sta HBUFF+1
  lda HRChars+0,X
  ora HLChars+0,Y
  sta HBUFF+0
  rts

; -----------------------------------------------------------------------------

; Copy Score (Score Start Pointer Is In X)
CopyScore
  lda $02,X
  lsr
  lsr
  lsr
  lsr
  tay
  lda (HDPTR),Y
  sta HTEXT6
  lda $02,X
  and #%00001111
  tay
  lda (HDPTR),Y
  sta HTEXT7
  lda $01,X
  lsr
  lsr
  lsr
  lsr
  tay
  lda (HDPTR),Y
  sta HTEXT8
  lda $01,X
  and #%00001111
  tay
  lda (HDPTR),Y
  sta HTEXT9
  lda $00,X
  lsr
  lsr
  lsr
  lsr
  tay
  lda (HDPTR),Y
  sta HTEXT10
  lda $00,X
  and #%00001111
  tay
  lda (HDPTR),Y
  sta HTEXT11
  rts

; -----------------------------------------------------------------------------

  ; Pack NAME Into 2 Bytes 
PackName
  lda INITIALS+1
  and #%10000000
  sta TEMP2               ; Preserve 2600/7800 Bit
  lda NAME+0
  sta TEMP
  lax NAME+1
  and #%00011100
  asl
  asl
  asl
  ora TEMP
  sta INITIALS+0
  txa
  and #%00000011
  sta TEMP
  lda NAME+2
  asl
  asl
  ora TEMP
  ora TEMP2
  sta INITIALS+1
  rts

; -----------------------------------------------------------------------------

SetScorePointers
  ; Set Pointers
  lda CYCLE
  lsr
  bcs HiscoreBPointers
HiscoreAPointers
  lda #<ScoreLettersA
  sta SLPTR
  lda #<HiScoreDigitsA
  sta HDPTR
  lda #<ScoreLabelsLoA
  sta SLLPTR
  lda #<ScoreLabelsHiA
  sta SLHPTR
  lda #>ScoreLettersA
  sta SLPTR+1
  lda #>HiScoreDigitsA
  sta HDPTR+1
  sta SLLPTR+1
  sta SLHPTR+1
  jmp EndHiscorePointers
HiscoreBPointers
  lda #<ScoreLettersB
  sta SLPTR
  lda #<HiScoreDigitsB
  sta HDPTR
  lda #<ScoreLabelsLoB
  sta SLLPTR
  lda #<ScoreLabelsHiB
  sta SLHPTR
  lda #>ScoreLettersB
  sta SLPTR+1
  lda #>HiScoreDigitsB
  sta HDPTR+1
  sta SLLPTR+1
  sta SLHPTR+1
EndHiscorePointers
  rts

; -----------------------------------------------------------------------------

  ; Copy Text into ZP RAM (Y Contains Message Offset)
LoadText
  lda CYCLE
  lsr
  bcc LoadFlickerText
  SLEEP 4
  bcs LoadNormalText
LoadFlickerText
  tya
  adc #(7*12)
  tay
LoadNormalText
  lda HiScoreMsg+11,Y
  sta HTEXT11
  lda HiScoreMsg+10,Y
  sta HTEXT10
  lda HiScoreMsg+9,Y
  sta HTEXT9
  lda HiScoreMsg+8,Y
  sta HTEXT8
  lda HiScoreMsg+7,Y
  sta HTEXT7
  lda HiScoreMsg+6,Y
  sta HTEXT6
  lda HiScoreMsg+5,Y
  sta HTEXT5
  lda HiScoreMsg+4,Y
  sta HTEXT4
  lda HiScoreMsg+3,Y
  sta HTEXT3
  lda HiScoreMsg+2,Y
  sta HTEXT2
  lda HiScoreMsg+1,Y
  sta HTEXT1
  lda HiScoreMsg+0,Y
  sta HTEXT0
  jmp NormalTextCopy
EndLoadText

; -----------------------------------------------------------------------------

CheckTitleFire
  ; Debounce Fire Button
  lax DEBOUNCE
  bit INPT4
  bmi ResetTitleFire
  ; Ignore If Previously Pressed
  and #%01000000
  bne ClearTitleFire
  ; Set Fire Pressed & State
  txa
  ora #%11000000
  bne StoreTitleFire
ResetTitleFire
  and #%00111111
  jmp StoreTitleFire
ClearTitleFire
  txa
  and #%01111111
StoreTitleFire
  sta DEBOUNCE
  rts

; -----------------------------------------------------------------------------

HiScoreSFX
  ; Play High Score SFX
  ldx #>HSoundTab
  stx SPTR+1
  lax SFX
  beq EndHSounds
  and #%00011111
  tay
  dey
  bpl PlayHSound
  ldx #0
  stx SFX
  beq EndHSounds
PlayHSound
  sty TEMP
  txa
  and #%11100000
  ora TEMP
  sta SFX
  lsr
  lsr
  lsr
  lsr
  lsr
  tax
  lda HSoundTab,X
  sta SPTR
  lda (SPTR),Y
  sta AUDF1
  ldx #5
  stx AUDC1
EndHSounds
  stx AUDV1
  rts

; -----------------------------------------------------------------------------

  ; Binary To BCD Table (100 Entries)
HiScoreBCDTable
  DC.B  %00000000, %00000001, %00000010, %00000011, %00000100
  DC.B  %00000101, %00000110, %00000111, %00001000, %00001001
  DC.B  %00010000, %00010001, %00010010, %00010011, %00010100
  DC.B  %00010101, %00010110, %00010111, %00011000, %00011001
  DC.B  %00100000, %00100001, %00100010, %00100011, %00100100
  DC.B  %00100101, %00100110, %00100111, %00101000, %00101001
  DC.B  %00110000, %00110001, %00110010, %00110011, %00110100
  DC.B  %00110101, %00110110, %00110111, %00111000, %00111001
  DC.B  %01000000, %01000001, %01000010, %01000011, %01000100
  DC.B  %01000101, %01000110, %01000111, %01001000, %01001001
  DC.B  %01010000, %01010001, %01010010, %01010011, %01010100
  DC.B  %01010101, %01010110, %01010111, %01011000, %01011001
  DC.B  %01100000, %01100001, %01100010, %01100011, %01100100
  DC.B  %01100101, %01100110, %01100111, %01101000, %01101001
  DC.B  %01110000, %01110001, %01110010, %01110011, %01110100
  DC.B  %01110101, %01110110, %01110111, %01111000, %01111001
  DC.B  %10000000, %10000001, %10000010, %10000011, %10000100
  DC.B  %10000101, %10000110, %10000111, %10001000, %10001001
  DC.B  %10010000, %10010001, %10010010, %10010011, %10010100
  DC.B  %10010101, %10010110, %10010111, %10011000, %10011001
EndHiScoreBCDTable
  if (>HiScoreBCDTable != >EndHiScoreBCDTable)
    echo "WARNING: HiScoreBCDTable crosses a page boundary!"
  endif

  DC.B  "AA0"

; -----------------------------------------------------------------------------
; HISCORE DISPLAY KERNELS
; -----------------------------------------------------------------------------

  ALIGN   256

NameKernel
  sta WSYNC                 ; [0]
  
  ; Reset Movements
  sta HMCLR                 ; [0] + 3

  ; Player 0 Two Copies Close
  ldx #%11110001            ; [3] + 2
  stx HMP0                  ; [5] + 3
  stx NUSIZ0                ; [8] + 3

  ; No Sprite Delays
  ldx #0                    ; [11] + 2
  stx NUSIZ1                ; [13] + 3
  stx VDELP0                ; [16] + 3
  stx VDELP1                ; [19] + 3

  ; Name Pointers
  ldy NAME+0                ; [22] + 3
  lda (SLPTR),Y             ; [25] + 5
  sta HPTR0                 ; [30] + 3
  ldy NAME+1                ; [33] + 3
  lda (SLPTR),Y             ; [36] + 5
  
  ; Sprite Positions (Coarse)
  sta RESP0                 ; [41] + 3 = 44 EXACT
  sta RESP1                 ; [44] + 3 = 47 EXACT
  
  sta HPTR1                 ; [47] + 3
  ldy NAME+2                ; [50] + 3
  lda (SLPTR),Y             ; [53] + 5
  sta HPTR2                 ; [58] + 3
  
  ; Name High Pointers
  ldx #>HLChars             ; [61] + 2
  stx HPTR0+1               ; [63] + 3
  stx HPTR1+1               ; [66] + 3
  stx HPTR2+1               ; [69] + 3

  ; Move Sprites (Fine)
  sta HMOVE                 ; [72] + 3

  ; Show Name
  ldy #4                    ; [13] + 2
NameKernelLoop
  sta WSYNC                 ; [0]
  lda NAMECOL+0             ; [0] + 3
  sta COLUP0                ; [3] + 3   < 46
  lda (HPTR0),Y             ; [6] + 5
  sta GRP0                  ; [11] + 3  < 46
  lda NAMECOL+1             ; [14] + 3
  sta COLUP1                ; [17] + 3  < 48
  lda (HPTR1),Y             ; [20] + 5
  sta GRP1                  ; [25] + 3  < 48
  ldx NAMECOL+2             ; [28] + 3
  lda (HPTR2),Y             ; [31] + 5
  SLEEP 7                   ; [36] + 7
  dey                       ; [43] + 2
  sta GRP0                  ; [45] + 3  > 48 < 52
  stx COLUP0                ; [48] + 3  > 48 < 52
  bpl NameKernelLoop
  
  ; Clear Sprites
  sta WSYNC
  iny
  sty GRP1
  sty GRP0
  sty GRP1
  rts
EndNameKernel
  if (>NameKernel != >EndNameKernel)
    echo "WARNING: Name Kernel Crosses Page Boundary!"
  endif

; -----------------------------------------------------------------------------

PositionHiScore
  ; Position Sprites
  sta WSYNC                 ; [0]
  SLEEP 5                   ; [0] + 5
  ; Set 3 sprite copies (medium) and no delay
  ldx #%00000110            ; [5] + 2
  stx NUSIZ0                ; [7] + 3
  stx VDELP0                ; [10] + 3
  stx VDELP1                ; [13] + 3
  ; Set 3 sprite copies (close)
  ldx #%00000011            ; [16] + 2
  stx NUSIZ1                ; [18] + 3
  ; Position Sprites
  ldx #%11100000            ; [21] + 2
  stx HMP0                  ; [23] + 3
  nop                       ; [26] + 2
  sta RESP0                 ; [28] + 3  = 31
  stx HMP1                  ; [31] + 3
  nop                       ; [34] + 2
  sta RESP1                 ; [36] + 3  = 39
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3
  rts                       ; [8] + 6 = 14
EndPositionHiScore
  if (>PositionHiScore != >EndPositionHiScore)
    echo "WARNING: PositionHiScore Crosses Page Boundary!"
  endif

; -----------------------------------------------------------------------------

PositionText
  sta WSYNC                 ; [0]
  ; Set 3 Copies Close and Delay
  lda #%00110011            ; [0] + 2
  sta VDELP0                ; [2] + 3
  sta VDELP1                ; [5] + 3
  sta NUSIZ0                ; [8] + 3
  sta NUSIZ1                ; [11] + 3
  ; Coarse Position
  sta HMCLR                 ; [14] + 3
  lda #%11100000            ; [17] + 2
  sta HMP0                  ; [19] + 3
  sta.w RESM0               ; [22] + 4  = 26
  sta HMM0                  ; [26] + 3
  lda #%11110000            ; [29] + 2
  sta HMP1                  ; [31] + 3
  lda #0                    ; [34] + 2
  sta RESP0                 ; [36] + 3  = 39
  sta RESP1                 ; [39] + 3  = 42
  sta ENAM0                 ; [42] + 3
  sta ENAM1                 ; [45] + 3
  ; Fine Position
  sta WSYNC                 ; [0]
  sta HMOVE                 ; [0] + 3
  rts                       ; [8] + 6 = 14
EndPositionText
  if (>PositionText != >EndPositionText)
    echo "WARNING: PositionText Crosses Page Boundary!"
  endif

; -----------------------------------------------------------------------------

  ; Clear RAM+ Memory
ClearExtraRam
  ldy #255
  lda #0
ClearRamLoop
  sta RAM_W,Y               ; [0] + 4
  dey                       ; [4] + 2
  bne ClearRamLoop          ; [6] + 2/3
  sta RAM_W                 ; TOTAL = 9 * 256 = 2304 (30 SCANLINES)
  rts

; -----------------------------------------------------------------------------

  ; Wait For Start Of Screen
StartScreen
  BEGIN_FRAME
  rts

  ; Wait For Next Frame
NextFrame
  BEGIN_OVERSCAN
  BEGIN_VBLANK
  rts
  
  ; Skip Screen Lines
SkipLinesB
  sta WSYNC
  dey
  bpl SkipLinesB
  rts

  DC.B  "AA3"

; -----------------------------------------------------------------------------

  ALIGN   256

  ; Display Hi Score Table Line (Unrolled)
HiScoreKernel
  sta WSYNC                 ; [0]
  SLEEP 5                   ; [0] + 5 
  lda #%00000011            ; [5] + 2
  sta NUSIZ1                ; [7] + 3
  lda HBUFF+4               ; [10] + 3
  sta GRP0                  ; [13] + 3   < 34
  lda HBUFF+19              ; [16] + 3
  sta GRP1                  ; [19] + 3  < 42
  ldx HBUFF+14              ; [22] + 3
  ldy HBUFF+34              ; [25] + 3
  lda HBUFF+24              ; [28] + 3
  SLEEP 3                   ; [31] + 3
  sta GRP0                  ; [34] + 3  > 36 < 44
  nop                       ; [37] + 2
  lda HBUFF+29              ; [39] + 3
  sta GRP1                  ; [42] + 3  > 44 < 47
  lda HBUFF+9               ; [45] + 3
  sta GRP1                  ; [48] + 3  > 49 < 52 
  stx GRP0                  ; [51] + 3  > 46 < 55
  lda #%00000110            ; [54] + 2
  sta NUSIZ1                ; [56] + 3  > 54 < 63
  sty GRP1                  ; [59] + 3  > 54 < 63
  sta WSYNC                 ; [0]
  SLEEP 5                   ; [0] + 5
  lda #%00000011            ; [5] + 2
  sta NUSIZ1                ; [7] + 3
  lda HBUFF+3               ; [10] + 3
  sta GRP0                  ; [13] + 3   < 34
  lda HBUFF+18              ; [16] + 3
  sta GRP1                  ; [19] + 3  < 42
  ldx HBUFF+13              ; [22] + 3
  ldy HBUFF+33              ; [25] + 3
  lda HBUFF+23              ; [28] + 3
  SLEEP 3                   ; [31] + 3
  sta GRP0                  ; [34] + 3  > 36 < 44
  nop                       ; [37] + 2
  lda HBUFF+28              ; [39] + 3
  sta GRP1                  ; [42] + 3  > 44 < 47
  lda HBUFF+8               ; [45] + 3
  sta GRP1                  ; [48] + 3  > 49 < 52 
  stx GRP0                  ; [51] + 3  > 46 < 55
  lda #%00000110            ; [54] + 2
  sta NUSIZ1                ; [56] + 3  > 54 < 63
  sty GRP1                  ; [59] + 3  > 54 < 63
  sta WSYNC                 ; [0]
  SLEEP 5                   ; [0] + 5
  lda #%00000011            ; [5] + 2
  sta NUSIZ1                ; [7] + 3
  lda HBUFF+2               ; [10] + 3
  sta GRP0                  ; [13] + 3   < 34
  lda HBUFF+17              ; [16] + 3
  sta GRP1                  ; [19] + 3  < 42
  ldx HBUFF+12              ; [22] + 3
  ldy HBUFF+32              ; [25] + 3
  lda HBUFF+22              ; [28] + 3
  SLEEP 3                   ; [31] + 3
  sta GRP0                  ; [34] + 3  > 36 < 44
  nop                       ; [37] + 2
  lda HBUFF+27              ; [39] + 3
  sta GRP1                  ; [42] + 3  > 44 < 47
  lda HBUFF+7               ; [45] + 3
  sta GRP1                  ; [48] + 3  > 49 < 52 
  stx GRP0                  ; [51] + 3  > 46 < 55
  lda #%00000110            ; [54] + 2
  sta NUSIZ1                ; [56] + 3  > 54 < 63
  sty GRP1                  ; [59] + 3  > 54 < 63
  sta WSYNC                 ; [0]
  SLEEP 5                   ; [0] + 5
  lda #%00000011            ; [5] + 2
  sta NUSIZ1                ; [7] + 3
  lda HBUFF+1               ; [10] + 3
  sta GRP0                  ; [13] + 3   < 34
  lda HBUFF+16              ; [16] + 3
  sta GRP1                  ; [19] + 3  < 42
  ldx HBUFF+11              ; [22] + 3
  ldy HBUFF+31              ; [25] + 3
  lda HBUFF+21              ; [28] + 3
  SLEEP 3                   ; [31] + 3
  sta GRP0                  ; [34] + 3  > 36 < 44
  nop                       ; [37] + 2
  lda HBUFF+26              ; [39] + 3
  sta GRP1                  ; [42] + 3  > 44 < 47
  lda HBUFF+6               ; [45] + 3
  sta GRP1                  ; [48] + 3  > 49 < 52 
  stx GRP0                  ; [51] + 3  > 46 < 55
  lda #%00000110            ; [54] + 2
  sta NUSIZ1                ; [56] + 3  > 54 < 63
  sty GRP1                  ; [59] + 3  > 54 < 63
  sta WSYNC                 ; [0]
  SLEEP 5                   ; [0] + 5
  lda #%00000011            ; [5] + 2
  sta NUSIZ1                ; [7] + 3
  lda HBUFF+0               ; [10] + 3
  sta GRP0                  ; [13] + 3   < 34
  lda HBUFF+15              ; [16] + 3
  sta GRP1                  ; [19] + 3  < 42
  ldx HBUFF+10              ; [22] + 3
  ldy HBUFF+30              ; [25] + 3
  lda HBUFF+20              ; [28] + 3
  SLEEP 3                   ; [31] + 3
  sta GRP0                  ; [34] + 3  > 36 < 44
  nop                       ; [37] + 2
  lda HBUFF+25              ; [39] + 3
  sta GRP1                  ; [42] + 3  > 44 < 47
  lda HBUFF+5               ; [45] + 3
  sta GRP1                  ; [48] + 3  > 49 < 52 
  stx GRP0                  ; [51] + 3  > 46 < 55
  lda #%00000110            ; [54] + 2
  sta NUSIZ1                ; [56] + 3  > 54 < 63
  sty GRP1                  ; [59] + 3  > 54 < 63
  lda #0                    ; [62] + 2
  sta GRP0                  ; [64] + 3  > 57
  sta GRP1                  ; [67] + 3  > 65
  sta GRP0                  ; [70] + 3
  rts                       ; [5] + 6 = 11
EndHiScoreKernel
  if (>HiScoreKernel != >EndHiScoreKernel)
    echo "WARNING: HiScoreKernel Crosses Page Boundary!"
  endif

  ; Sound Effects (Lower = Higher Priority)
  ; 1 = Select
  ; 2 = Move
HSoundTab
  DC.B  0, <HSelectSnd, <HMoveSnd
HSelectSnd
  DC.B  31, 31, 31, 31
HMoveSnd
  DC.B  24, 24, 24, 24

  DC.B  "AA4"
  
; -----------------------------------------------------------------------------
; HISCORE DATA
; -----------------------------------------------------------------------------

  ALIGN 256

  ; HiScore Message
HiScoreMsg
  ; Wipe Scores
  DC.B  <HL_WW, <HL_II, <HL_PP, <HL_EE, <HL_GAP, <HL_SS
  DC.B  <HL_CC, <HL_OO, <HL_RR, <HL_EE, <HL_SS, <HL_QMARK             ; 0
  ; Wipe
  DC.B  <HL_GAP, <HL_GAP, <HL_GAP, <HL_WW, <HL_II, <HL_PP
  DC.B  <HL_EE, <HL_BANG, <HL_BANG, <HL_GAP, <HL_GAP, <HL_GAP         ; 1
  ; Ignore
  DC.B  <HL_GAP, <HL_GAP, <HL_GAP, <HL_II, <HL_GG, <HL_NN
  DC.B  <HL_OO, <HL_RR, <HL_EE, <HL_GAP, <HL_GAP, <HL_GAP             ; 2
  ; New Hiscore
  DC.B  <HL_NN, <HL_EE, <HL_WW, <HL_GAP, <HL_HH, <HL_II
  DC.B  <HL_SS, <HL_CC, <HL_OO, <HL_RR, <HL_EE, <HL_BANG              ; 3
  ; Enter Name
  DC.B  <HL_GAP, <HL_EE, <HL_NN, <HL_TT, <HL_EE, <HL_RR
  DC.B  <HL_GAP, <HL_NN, <HL_AA, <HL_MM, <HL_EE, <HL_GAP              ; 4
  ; Fire To Save
  DC.B  <HL_FF, <HL_II, <HL_RR, <HL_EE, <HL_GAP, <HL_TT
  DC.B  <HL_OO, <HL_GAP, <HL_SS, <HL_AA, <HL_VV, <HL_EE               ; 5
  ; Hall Of Fame
  DC.B  <HL_HH, <HL_AA, <HL_LL, <HL_LL, <HL_GAP, <HL_OO
  DC.B  <HL_FF, <HL_GAP, <HL_FF, <HL_AA, <HL_MM, <HL_EE               ; 6
  ; Wipe Scores
  DC.B  <DL_WW, <DL_II, <DL_PP, <DL_EE, <DL_GAP, <DL_SS
  DC.B  <DL_CC, <DL_OO, <DL_RR, <DL_EE, <DL_SS, <DL_QMARK             ; 7
  ; Wipe
  DC.B  <DL_GAP, <DL_GAP, <DL_GAP, <DL_WW, <DL_II, <DL_PP
  DC.B  <DL_EE, <DL_BANG, <DL_BANG, <DL_GAP, <DL_GAP, <DL_GAP         ; 8
  ; Ignore
  DC.B  <DL_GAP, <DL_GAP, <DL_GAP, <DL_II, <DL_GG, <DL_NN
  DC.B  <DL_OO, <DL_RR, <DL_EE, <DL_GAP, <DL_GAP, <DL_GAP             ; 9
  ; New Hiscore
  DC.B  <DL_NN, <DL_EE, <DL_WW, <DL_GAP, <DL_HH, <DL_II
  DC.B  <DL_SS, <DL_CC, <DL_OO, <DL_RR, <DL_EE, <DL_BANG              ; 10
  ; Enter Name
  DC.B  <DL_GAP, <DL_EE, <DL_NN, <DL_TT, <DL_EE, <DL_RR
  DC.B  <DL_GAP, <DL_NN, <DL_AA, <DL_MM, <DL_EE, <DL_GAP              ; 11
  ; Fire To Save
  DC.B  <DL_FF, <DL_II, <DL_RR, <DL_EE, <DL_GAP, <DL_TT
  DC.B  <DL_OO, <DL_GAP, <DL_SS, <DL_AA, <DL_VV, <DL_EE               ; 12
  ; Hall Of Fame
  DC.B  <DL_HH, <DL_AA, <DL_LL, <DL_LL, <DL_GAP, <DL_OO
  DC.B  <DL_FF, <DL_GAP, <DL_FF, <DL_AA, <DL_MM, <DL_EE               ; 13
EndHiScoreMsg
  if (>HiScoreMsg != >EndHiScoreMsg)
    echo "WARNING: HiScoreMsg crosses a page boundary!"
  endif

  ; Score Digits Table (30 Entries)
HiScoreDigitsA
  DC.B  <HL_00, <HL_11, <HL_22, <HL_33, <HL_44
  DC.B  <HL_55, <HL_66, <HL_77, <HL_88, <HL_99, <HL_DOT
HiScoreDigitsB
  DC.B  <DL_00, <DL_11, <DL_22, <DL_33, <DL_44
  DC.B  <DL_55, <DL_66, <DL_77, <DL_88, <DL_99, <DL_GAP
ScoreLabelsLoA
  DC.B  <HL_00, <HL_99, <HL_88, <HL_77, <HL_66
  DC.B  <HL_55, <HL_44, <HL_33, <HL_22, <HL_11
ScoreLabelsLoB
  DC.B  <DL_00, <DL_99, <DL_88, <DL_77, <DL_66
  DC.B  <DL_55, <DL_44, <DL_33, <DL_22, <DL_11
ScoreLabelsHiA
  DC.B  <HL_11
ScoreLabelsHiB
  DC.B  <HL_GAP, <HL_GAP, <HL_GAP, <HL_GAP, <HL_GAP
  DC.B  <HL_GAP, <HL_GAP, <HL_GAP, <HL_GAP, <HL_GAP
EndHiScoreDigits
  if (>HiScoreDigitsA != >EndHiScoreDigits)
    echo "WARNING: HiScoreDigits crosses a page boundary!"
  endif

  ; Score Tables
ScoreOffsets
  DC.B  54, 48, 42, 36, 30, 24, 18, 12, 6, 0

  ; Title Message Delays
MessageDelay
  DC.B  127, 63, 63

  DC.B  "AA5"

  ALIGN   256

  ; Small Font LHS
  LEFTCHARS

  ; Score Letters Table (32 Entries)
ScoreLettersA
  DC.B  <HL_AA, <HL_BB, <HL_CC, <HL_DD, <HL_EE, <HL_FF, <HL_GG, <HL_HH
  DC.B  <HL_II, <HL_JJ, <HL_KK, <HL_LL, <HL_MM, <HL_NN, <HL_OO, <HL_PP
  DC.B  <HL_QQ, <HL_RR, <HL_SS, <HL_TT, <HL_UU, <HL_VV, <HL_WW, <HL_XX
  DC.B  <HL_YY, <HL_ZZ, <HL_DOT, <HL_BANG
  DC.B  <HL_QMARK, <HL_DASH, <HL_COLON, <HL_GAP
ScoreLettersB
  DC.B  <DL_AA, <DL_BB, <DL_CC, <DL_DD, <DL_EE, <DL_FF, <DL_GG, <DL_HH
  DC.B  <DL_II, <DL_JJ, <DL_KK, <DL_LL, <DL_MM, <DL_NN, <DL_OO, <DL_PP
  DC.B  <DL_QQ, <DL_RR, <DL_SS, <DL_TT, <DL_UU, <DL_VV, <DL_WW, <DL_XX
  DC.B  <DL_YY, <DL_ZZ, <DL_DOT, <DL_BANG
  DC.B  <DL_QMARK, <DL_DASH, <DL_COLON, <DL_GAP
EndScoreLetters
  if (>ScoreLettersA != >EndScoreLetters)
    echo "WARNING: ScoreLetters crosses a page boundary!"
  endif

  DC.B  "AA6"

  ALIGN 256

  ; Small Font RHS
  RIGHTCHARS

  ; Score Colours
FireCol
  DC.B  $4C, $6C, $0C       ; NTSC/PAL60/B&W
ScoreCol
  DC.B  $8C, $BC, $0C       ; NTSC/PAL60/B&W
HeadingCol
  DC.B  $1C, $2C, $0C       ; NTSC/PAL60/B&W
  
  DC.B  "AA7"

; -----------------------------------------------------------------------------

  ALIGN 256

NormalTextKernel
  ldx #4
  stx TEMP2
  ldy HBUFF+0,X
  jmp HiScoreTextStart
HiScoreTextLoop
  ldx TEMP2                 ; [62] + 3
  ldy HBUFF+0,X             ; [65] + 4                  0A 0B 1A 1B
HiScoreTextStart
  sta WSYNC                 ; [0]
  SLEEP 5                   ; [0] + 5
  nop                       ; [5] + 2
  sty GRP0                  ; [7] + 3                  0 - - -
  lda HBUFF+5,X             ; [10] + 4
  sta GRP1                  ; [14] + 3  < 42            - 0 1 0
  lda HBUFF+10,X            ; [17] + 4
  sta GRP0                  ; [21] + 3  < 44            2 0 - 1
  ldy HBUFF+20,X            ; [24] + 4
  lda HBUFF+25,X            ; [28] + 4
  sta TEMP                  ; [32] + 3
  lda HBUFF+15,X            ; [35] + 4
  ldx TEMP                  ; [39] + 3
  sta GRP1                  ; [42] + 3  > 44 < 47       - 2 3 1
  sty GRP0                  ; [45] + 3  > 46 < 50       4 2 - 3
  stx GRP1                  ; [48] + 3  > 49 < 52       - 4 5 3
  sta GRP0                  ; [51] + 3  > 52 < 55       - 4 - 5
  dec TEMP2                 ; [54] + 5
  bpl HiScoreTextLoop       ; [59] + 2/3
  ldx #0                    ; [61] + 2
  stx GRP1                  ; [63] + 3
  stx GRP0                  ; [66] + 3
  stx GRP1                  ; [69] + 3
  rts                       ; [72] + 6
EndNormalTextKernel
  if (>NormalTextKernel != >EndNormalTextKernel)
    echo "WARNING: NormalTextKernel Crosses Page Boundary!"
  endif

; -----------------------------------------------------------------------------
; STALL CODE
; -----------------------------------------------------------------------------

DoStall
  ; Initialise & Reposition Stall Sprites
  ldx #1
StallPos
  sta WSYNC                 ; [0]
  ldy #0                    ; [0] + 2
  sty GRP0                  ; [2] + 3
  sty GRP1                  ; [5] + 3
  sty VDELP0                ; [8] + 3
  sty VDELP1                ; [11] + 3
  sty NUSIZ0                ; [14] + 3
  sty NUSIZ1                ; [17] + 3
  sty COLUPF                ; [20] + 3
  sty PF0                   ; [23] + 3
  sty PF1                   ; [26] + 3
  sty PF2                   ; [29] + 3
  ldy #$0E                  ; [32] + 2    White
  sty COLUP1                ; [34] + 3
  ldy #$94                  ; [37] + 2    Blue (PAL & NTSC)
  sty COLUP0                ; [39] + 3
  sta RESP0,X               ; [42] + 4    = 46
  dex
  bpl StallPos

  ; Copy Stall Code
  BEGIN_FRAME
  ldx #(EndStall - RAMSPINNER)
CopyStall
  lda $FFA0,X
  sta RAMSPINNER,X
  dex
  bpl CopyStall

  ; Begin Stall
  BEGIN_OVERSCAN
WaitOverscan
  lda INTIM
  bne WaitOverscan
  jmp StartStall

  echo "----",($FFA0 - *) , "bytes left (BANK6 - HISCORE TABLE)"

; -----------------------------------------------------------------------------

  ORG   $6FA0
  RORG  RAMSPINNER

  ; Spinner Data
Spinner
  DC.B %00001111
  DC.B %00001111
  DC.B %00001111
  DC.B %00001111
  DC.B %11111111
  DC.B %11111111
  DC.B %11111111
  DC.B %11111111
  DC.B %11110000
  DC.B %11110000
  DC.B %11110000
  DC.B %11110000

  ; Spinner Loop
SpriteLoop
SpriteOffset
  lda Spinner,X
  sta GRP0
  dex
  sta WSYNC
  sta WSYNC
  bpl SpriteLoop

  ; End Of Frame
  ldx #115
  stx VBLANK
LastLoop
  dex
  sta WSYNC
  bpl LastLoop
  stx GRP1

  ; Entry Point
StartStall
  ; Do Vertical Sync
  lda #%00001110
SyncLoop
  sta WSYNC
  sta VSYNC
  lsr
  bne SyncLoop

  ; Check End of Stall
  IF (MELODY)
  bit $1FF4
  bvc FinishStall
  ELSE
  IF (FAKESPIN)
  cpy #0
  beq FinishStall
  nop
  ELSE
  jmp FinishStall
  nop
  nop
  ENDIF
  ENDIF

  ; Calculate Spinner Frame
  dey
  sty REFP0
  tya
  lsr
  lsr
  and #%00000100
  ora #RAMSPINNER
  sta SpriteOffset+1

  ; Start Of Frame
  ldx #126
FirstLoop
  dex
  sta WSYNC
  bne FirstLoop
  stx VBLANK

  ; Draw Sprite
  ldx #7
  bne SpriteLoop

  ; Resume
FinishStall
  jmp ($FFFE)
EndStall                  ; Leave Space For Stack!

  echo "----",($100 - *) , " bytes left (BANK 6 - STALL CODE)"

; -----------------------------------------------------------------------------

  ORG     $6FF4
  RORG    $FFF4
  DC.B    "BANK6", 0, 0, 0
  DC.W    Start, Resume

